#!/usr/bin/perl
my $buildno = '1.0.2016.11.04';
#
# Aminosan
# This is a script to select amino-acid substitution model for multi-
# partitioned data set. Official web site of this script is
# http://www.fifthdimension.jp/products/aminosan/ .
# To know script details, see above URL.
# 
# If you publish your study using Aminosan, please cite the following.
# Tanabe, AS (2011) "Kakusan4 and Aminosan: two programs for comparing
#   nonpartitioned, proportional, and separate models for combined
#   molecular phylogenetic analyses of multilocus sequence data",
#   Molecular Ecology Resources, vol.11, pp.914-921.
# 
# Copyright (C) 2008-2012  Akifumi S. Tanabe
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

use strict;

my $readseq = 'readseq';
my $paupcmd = 'paup';
my $codeml = 'codeml';
my $tf = 'tf';
my $raxml = 'raxmlHPC-PTHREADS';
my $protdist = 'protdist';
my $neighbor = 'neighbor';
my $protpars = 'protpars';

my $lnlcalc = 'codeml';

#my @allmodel = ('Poisson', 'Poisson+F', 'BLOSUM62', 'BLOSUM62+F', 'cpREV', 'cpREV+F', 'Dayhoff', 'Dayhoff+F', 'JTT', 'JTT+F', 'LG', 'LG+F', 'mtArt', 'mtArt+F', 'mtMam', 'mtMam+F', 'mtREV', 'mtREV+F', 'PMB', 'PMB+F', 'rtREV', 'rtREV+F', 'HIVb', 'HIVb+F', 'HIVw', 'HIVw+F', 'VT', 'VT+F', 'WAG', 'WAG+F', 'FLU', 'FLU+F');
#my @allheterogeneity = ('Homogeneous', 'Gamma', 'Invariant', 'GammaInvariant');

my @modelformrbayes = ('Poisson', 'Poisson+F', 'BLOSUM62', 'BLOSUM62+F', 'cpREV', 'cpREV+F', 'Dayhoff', 'Dayhoff+F', 'JTT', 'JTT+F', 'LG', 'LG+F', 'mtArt', 'mtArt+F', 'mtMam', 'mtMam+F', 'mtREV', 'mtREV+F', 'mtZoa', 'mtZoa+F', 'PMB', 'PMB+F', 'rtREV', 'rtREV+F', 'HIVb', 'HIVb+F', 'HIVw', 'HIVw+F', 'VT', 'VT+F', 'WAG', 'WAG+F', 'FLU', 'FLU+F', 'stmtREV', 'stmtREV+F');
my @heterogeneityformrbayes = ('Homogeneous', 'Gamma', 'Invariant', 'GammaInvariant', 'AGamma', 'Gene', 'Gene1Gamma', 'GeneNGamma', 'Gene1AGamma', 'GeneNAGamma');

my @modelfortf = ('Poisson', 'Poisson+F', 'BLOSUM62', 'BLOSUM62+F', 'cpREV', 'cpREV+F', 'Dayhoff', 'Dayhoff+F', 'JTT', 'JTT+F', 'LG', 'LG+F', 'mtArt', 'mtArt+F', 'mtMam', 'mtMam+F', 'mtREV', 'mtREV+F', 'mtZoa', 'mtZoa+F', 'PMB', 'PMB+F', 'rtREV', 'rtREV+F', 'HIVb', 'HIVb+F', 'HIVw', 'HIVw+F', 'VT', 'VT+F', 'WAG', 'WAG+F', 'FLU', 'FLU+F', 'stmtREV', 'stmtREV+F');
my @heterogeneityfortf = ('Homogeneous', 'Gamma', 'Invariant', 'GammaInvariant');

my @modelforphyml = ('Poisson', 'Poisson+F', 'BLOSUM62', 'BLOSUM62+F', 'cpREV', 'cpREV+F', 'Dayhoff', 'Dayhoff+F', 'JTT', 'JTT+F', 'LG', 'LG+F', 'mtArt', 'mtArt+F', 'mtMam', 'mtMam+F', 'mtREV', 'mtREV+F', 'mtZoa', 'mtZoa+F', 'PMB', 'PMB+F', 'rtREV', 'rtREV+F', 'HIVb', 'HIVb+F', 'HIVw', 'HIVw+F', 'VT', 'VT+F', 'WAG', 'WAG+F', 'FLU', 'FLU+F', 'stmtREV', 'stmtREV+F');
my @heterogeneityforphyml = ('Homogeneous', 'Gamma', 'Invariant', 'GammaInvariant');

my @modelforraxml = ('BLOSUM62', 'BLOSUM62+F', 'cpREV', 'cpREV+F', 'Dayhoff', 'Dayhoff+F', 'JTT', 'JTT+F', 'LG', 'LG+F', 'mtArt', 'mtArt+F', 'mtMam', 'mtMam+F', 'mtREV', 'mtREV+F', 'mtZoa', 'mtZoa+F', 'PMB', 'PMB+F', 'rtREV', 'rtREV+F', 'HIVb', 'HIVb+F', 'HIVw', 'HIVw+F', 'VT', 'VT+F', 'WAG', 'WAG+F', 'FLU', 'FLU+F', 'stmtREV', 'stmtREV+F', 'LG4M', 'LG4M+F', 'LG4X', 'LG4X+F');
my @heterogeneityforraxml = ('Gamma');

my $treefile;
my $comparemix;
my $interactive = 0;
my $aafreq = 'Empirical';
my $ngammacat;
my $invariant = 0;
my $n_gam = 0;
my $agamma = 0;
my $numthreads = 1;
my $difftreeeachpart = 0;
my $algorithm = 'NJK83';
my $calcwhole = 0;
my $ntax;
my $nchar;
my $maindir;
my $taxlength = 6;
my @fullpathname;
my @inputfile;
my @taxa;
my @partname;
my @allpart;
my %part2site;
my %models;
my %heterogeneities;
my %model;
my %heterogeneity;
my %dat;
my %seqs;
my %nparam;
my %samplesize;
my %LnL;
my %criteria;
my %weight;
my %output;
my %suffixes;
my %calced;

# Main routine

print <<"_END";
Aminosan $buildno
=======================================================================

This is a script to select amino-acid substitution model for multi-
partitioned data set. Official web site of this script is
http://www.fifthdimension.jp/products/aminosan/ .
To know script details, see above URL.

If you publish your study using Aminosan, please cite the following.
Tanabe, AS (2011) \"Kakusan4 and Aminosan: two programs for comparing
  nonpartitioned, proportional, and separate models for combined
  molecular phylogenetic analyses of multilocus sequence data\",
  Molecular Ecology Resources, vol.11, pp.914-921.

Copyright (C) 2008-2012  Akifumi S. Tanabe

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

_END

# parse command line options
&parseOptions();

# check required and obstructive files
&checkFiles();

# convert sequences to non-interleaved NEXUS format
&convertSequences();

# split sequences to specified partitions
if (@inputfile == 1) {
	&splitSequences();
}
# combine sequences
else {
	&combineSequences();
}

# set partitions
if (@partname) {
	&setPartitions();
}

# output configurations
&outputConfigurations();

# set candidate models
&setCandidateModels();

# run pre-selection chi-square test
&runChiSquareTest();

# set the number of model parameters
&setModelParameters();

# make starting tree
&makeStartingTree();

# set sample sizes
&setSampleSizes();

# analyze partitions
&analyzeEachPartition();

# calculate model selection criteria
&calculateCriteria();

# output files
&outputFiles();

if ($comparemix && %suffixes) {
	&compareMixedModels();
}

# exit
if ($interactive) {
	print("The model selection has been finished.\nThe results was output to \"$maindir\".\nPlease press enter to close.\n");
	<STDIN>;
}
exit;

sub parseOptions {
	print("Parsing command line options...\n");
	my %infile;
	my @infile;
	my %site2part;
	foreach (@ARGV) {
		# -h option
		if (/^-+(?:help|h)/i) {
			&helpMessage();
		}
		# -i option
		elsif (/^-+(?:interactive|i)=(.+)/i) {
			if ($1 =~ /^Enable$/i) {
				$interactive = 1;
			}
			elsif ($1 =~ /^Disable$/i) {
				$interactive = 0;
			}
			else {
				&errorMessage(__LINE__, 'Interactive mode option is not valid.');
			}
		}
		# -w option
		elsif (/^-+(?:calcwhole|w)=(.+)/i) {
			if ($1 =~ /^Enable$/i) {
				$calcwhole = 1;
			}
			elsif ($1 =~ /^Disable$/i) {
				$calcwhole = 0;
			}
			else {
				&errorMessage(__LINE__, 'The "calcwhole" option is not valid.');
			}
		}
		# -f option
		elsif (/^-+(?:aafreq|f)=(.+)/i) {
			if ($1 =~ /^Empirical$/i) {
				$aafreq = 'Empirical';
			}
			elsif ($1 =~ /^Estimate$/i) {
				$aafreq = 'Estimate';
			}
			else {
				&errorMessage(__LINE__, 'AAFreq option is not valid.');
			}
		}
		# -p option
		elsif (/^-+(?:partition|p)=(.+)/i) {
			foreach my $partition (split(/,/, $1)) {
				if ($partition =~ s/^([^:]+)://) {
					my $partname = $1;
					my @range;
					if ($partname eq 'whole') {
						&errorMessage(__LINE__, 'Partition name "whole" is reserved by Aminosan.');
					}
					unless ($part2site{$partname}) {
						push(@partname, $partname);
					}
					else {
						&errorMessage(__LINE__, "Partition name \"$partname\" is doubly specified.");
					}
					# case of normal partition
					if ($partition =~ /^(\d+)-(\d+)$/) {
						@range = &range2list($1, $2, 1);
					}
					else {
						&errorMessage(__LINE__, 'Partition specification is not valid.');
					}
					# Set site numbers to partition name
					foreach my $site (@range) {
						unless ($site2part{$site}) {
							$site2part{$site} = $partname;
						}
						else {
							&errorMessage(__LINE__, "Site $site is overlapping between \"$site2part{$site}\" and \"$partname\".");
						}
						push(@{$part2site{$partname}}, $site);
					}
				}
				else {
					&errorMessage(__LINE__, 'Partition name is not specified.');
				}
			}
		}
		# -g option
		elsif (/^-+(?:ngammacat|g)=(.+)/i) {
			if ($1 =~ /^(\d+)$/) {
				if ($1 < 2) {
					&errorMessage(__LINE__, 'The number of rate categories is not valid.');
				}
				else {
					$ngammacat = $1;
				}
			}
			else {
				&errorMessage(__LINE__, 'The number of rate categories is not an integer.');
			}
		}
		# -v option
		elsif (/^-+(?:pinvar|v)=(.+)/i) {
			if ($1 =~ /^Enable$/i) {
				$invariant = 1;
			}
			elsif ($1 =~ /^Disable$/i) {
				$invariant = 0;
			}
			else {
				&errorMessage(__LINE__, 'The "pinvar" option is not valid.');
			}
		}
		# -m option
		elsif (/^-+(?:ngam|m)=(.+)/i) {
			if ($1 =~ /^Enable$/i) {
				$n_gam = 1;
			}
			elsif ($1 =~ /^Disable$/i) {
				$n_gam = 0;
			}
			else {
				&errorMessage(__LINE__, 'The "ngam" option is not valid.');
			}
		}
		# -r option
		elsif (/^-+(?:agamma|r)=(.+)/i) {
			if ($1 =~ /^Enable$/i) {
				$agamma = 1;
			}
			elsif ($1 =~ /^Disable$/i) {
				$agamma = 0;
			}
			else {
				&errorMessage(__LINE__, 'The "agamma" option is not valid.');
			}
		}
#		# -a option
#		elsif (/^-+(?:algorithm|a)=(.+)/i) {
#			if ($1 =~ /^NJPoisson$/i) {
#				$algorithm = 'NJPoisson';
#			}
#			elsif ($1 =~ /^NJLogDet$/i) {
#				$algorithm = 'NJLogDet';
#			}
#			elsif ($1 =~ /^SARandom$/i) {
#				$algorithm = 'SARandom';
#			}
#			elsif ($1 =~ /^SAClosest$/i) {
#				$algorithm = 'SAClosest';
#			}
#			elsif ($1 =~ /^SAFurthest$/i) {
#				$algorithm = 'SAFurthest';
#			}
#			else {
#				&errorMessage(__LINE__, 'Algorithm option of tree generation is not valid.');
#			}
#		}
		# -t option
		elsif (/^-+(?:tree|t)=(.+)/i) {
			if ($1 =~ /^Different$/i) {
				$difftreeeachpart = 1;
			}
			elsif ($1 =~ /^Common$/i) {
				$difftreeeachpart = 0;
			}
			else {
				&errorMessage(__LINE__, 'Tree option for parameter optimization is not valid.');
			}
		}
		# -o option
		elsif (/^-+(?:output|o)=(.+)/i) {
			foreach (split(/,/, $1)) {
				if (/^MrBayes$/i) {
					$output{'mrbayes'} = 1;
					$lnlcalc = 'tf';
				}
				elsif (/^Treefinder$/i) {
					$output{'treefinder'} = 1;
					$lnlcalc = 'tf';
				}
				elsif (/^PHYML$/i) {
					$output{'phyml'} = 1;
				}
				elsif (/^RAxML$/i) {
					$output{'raxml'} = 1;
					$lnlcalc = 'raxml';
				}
				#elsif (/^all$/i) {
				#	$output{'all'} = 1;
				#}
				else {
					&errorMessage(__LINE__, 'Specified output option is not valid.');
				}
			}
		}
		# -n option
		elsif (/^-+(?:numthreads|n)=(.+)/i) {
			if ($1 =~ /^(\d+)$/) {
				if ($1 == 0) {
					&errorMessage(__LINE__, 'The number of threads is not valid.');
				}
				else {
					$numthreads = $1;
				}
			}
			else {
				&errorMessage(__LINE__, 'The number of threads is not an integer.');
			}
		}
		# -l option
		elsif (/^-+(?:likelihood|l)=(.+)/i) {
			if ($1 =~ /^CODEML$/i) {
				$lnlcalc = 'codeml';
			}
			elsif ($1 =~ /^TF$/i) {
				$lnlcalc = 'tf';
			}
			elsif ($1 =~ /^RAxML$/i) {
				$lnlcalc = 'raxml';
			}
			else {
				&errorMessage(__LINE__, 'Likelihood calculator option is not valid.');
			}
		}
		# --treefile option
		elsif (/^-+treefile=(.+)/i) {
			if (-e $1 && !-d $1 && !-z $1) {
				$treefile = $1;
			}
			else {
				&errorMessage(__LINE__, "Specified tree file \"$1\" does not exist or this is not a file.");
			}
		}
		# --comparemix option
		elsif (/^-+comparemix=(.+)/i) {
			if ($1 =~ /^Enable$/i) {
				$comparemix = 1;
			}
			elsif ($1 =~ /^Disable$/i) {
				$comparemix = 0;
			}
			else {
				&errorMessage(__LINE__, 'The "comparemix" option is not valid.');
			}
		}
		# input file
		else {
			s/^"(.+)"$/$1/;
			if ($^O eq 'MSWin32') {
				s/\\/\//g;
			}
			s/([^\\]) /$1\\ /g;
			foreach (glob($_)) {
				if ($infile{$_}) {
					&errorMessage(__LINE__, "\"$_\" is doubly specified.");
				}
				elsif (-e $_ && !-d $_ && !-z $_) {
					$infile{$_} = 1;
					push(@infile, $_);
				}
				else {
					&errorMessage(__LINE__, "\"$_\" does not exist or this is not a file.");
				}
			}
		}
	}
	# Is the input file specified?
	unless (%infile) {
		print("No input files are specified.\nEntering interactive mode.\nSpecified options are ignored.\nSpecify an input file name.\nNote that you can use wild card.\n");
		$interactive = 1;
		my $response = 1;
		while ($response) {
			$response = <STDIN>;
			$response =~ s/\r?\n?$//;
			$response =~ s/^\s*(.+?)\s*$/$1/;
			$response =~ s/^"(.+)"$/$1/;
			if ($^O eq 'MSWin32') {
				$response =~ s/\\/\//g;
			}
			$response =~ s/([^\\]) /$1\\ /g;
			unless ($response) {
				if (%infile) {
					print("OK. Input file specification has terminated.\n\n");
					last;
				}
				else {
					$response = 1;
					print("Input is empty.\nSpecify an input file name.\n");
				}
			}
			else {
				foreach (glob($response)) {
					if (!$infile{$_} && -e $_ && !-d $_ && !-z $_) {
						$infile{$_} = 1;
						push(@infile, $_);
						print("\"$_\" was accepted.\n");
					}
					elsif ($infile{$_}) {
						print("\"$_\" is already accepted.\n");
					}
					else {
						print("\"$_\" does not exist or this is not a file.\n");
					}
				}
				if (%infile) {
					print("Specify an input file name or just press enter to leave input file specification.\n");
				}
				else {
					print("Specify an input file name.\n");
				}
			}
		}
	}
	undef(%infile);
	my $datadir;
	my $coding;
	foreach (@infile) {
		my $fullpathname = File::Spec->rel2abs($_);
		push(@fullpathname, $fullpathname);
		my ($volume, $directories, $file) = File::Spec->splitpath($fullpathname);
		push(@inputfile, $file);
		if (@fullpathname == 1) {
			$datadir = $volume . $directories;
		}
	}
	$maindir = $datadir . $inputfile[0] . '.aminosan';
	print("Log, result and configuration files will be output to \"$maindir\".\n\n");
	if ($interactive) {
		print("OUTPUT OPTIONS\n\n");
		my $loop = 1;
		while ($loop) {
			my $response;
			print("Which is a target analysis software? (MrBayes/Treefinder/PHYML/RAxML)\n(default: RAxML)\n");
			$response = <STDIN>;
			$response =~ s/\r?\n?$//;
			if ($response =~ /^\s*PHYML\s*$/i) {
				$output{'phyml'} = 1;
				print("OK. The model configuration files for PHYML will be output.\n\n");
				last;
			}
			elsif ($response =~ /^\s*RAxML\s*$/i || $response =~ /^\s*rax\s*$/i || $response =~ /^\s*$/) {
				$output{'raxml'} = 1;
				print("OK. The model configuration files for RAxML will be output.\n\n");
				last;
			}
			elsif ($response =~ /^\s*Treefinder\s*$/i || $response =~ /^\s*tf\s*$/i) {
				$output{'treefinder'} = 1;
				$output{'mrbayes'} = 1;
				print("OK. The model configuration files for Treefinder will be output.\n\n");
				last;
			}
			elsif ($response =~ /^\s*MrBayes\s*$/i || $response =~ /^\s*mb\s*$/i) {
				$output{'mrbayes'} = 1;
				$output{'treefinder'} = 1;
				print("OK. The model configuration files for MrBayes will be output.\n\n");
				last;
			}
			else {
				print("Cannot understand your answer.\n\n");
			}
		}
		print("ANALYSIS OPTIONS\n\n");
		if (@fullpathname > 1 && $output{'phyml'} != 1) {
			while ($loop) {
				my $response;
				print("You input multiple files.\nDo you want to consider nonpartitioning of loci? (y/n)\nIf you say yes, applying nonpartitioned models to all-loci-concatenated sequences will be considered.\n(default: n)\n");
				$response = <STDIN>;
				$response =~ s/\r?\n?$//;
				if ($response =~ /^\s*(?:yes|y)\s*$/i) {
					$calcwhole = 1;
					print("OK. Nonpartitioning of loci is enabled.\n\n");
					last;
				}
				elsif ($response =~ /^\s*(?:no|n)\s*$/i || $response =~ /^\s*$/) {
					$calcwhole = 0;
					print("OK. Nonpartitioning of loci is disabled.\n\n");
					last;
				}
				else {
					print("Cannot understand your answer.\n\n");
				}
			}
		}
		elsif (@fullpathname > 1 && $output{'phyml'} == 1) {
			$calcwhole = 1;
			print("Because you enabled output option for PHYML, nonpartitioning of loci is forced to enable.\n\n");
		}
		if (@fullpathname > 1 && ($output{'treefinder'} || $output{'mrbayes'} || $output{'raxml'})) {
			while ($loop) {
				my $response;
				if ($output{'raxml'} && !$output{'treefinder'} && !$output{'mrbayes'}) {
					print("You input multiple files.\nDo you want to compare nonpartitioned, equalrate, and separate models on all-loci concatenated sequences? (y/n)\n");
				}
				else {
					print("You input multiple files.\nDo you want to compare nonpartitioned, equalrate, proportional, and separate models on all-loci concatenated sequences? (y/n)\nNote that this function needs Treefinder.\n");
				}
				print("(default: y)\n");
				$response = <STDIN>;
				$response =~ s/\r?\n?$//;
				if ($response =~ /^\s*(?:yes|y)\s*$/i || $response =~ /^\s*$/) {
					$comparemix = 1;
					if ($output{'raxml'} && !$output{'treefinder'} && !$output{'mrbayes'}) {
						print("OK. Comparison among nonpartitioned, equalrate, and separate models is enabled.\n");
					}
					else {
						print("OK. Comparison among nonpartitioned, equalrate, proportional, and separate models is enabled.\n");
					}
					if ($output{'mrbayes'}) {
						print("However, gene specific rate model is not considered because these models are incompatible with Treefinder.\n\n");
					}
					else {
						print("\n");
					}
					last;
				}
				elsif ($response =~ /^\s*(?:no|n)\s*$/i) {
					$comparemix = 0;
					if ($output{'raxml'} && !$output{'treefinder'} && !$output{'mrbayes'}) {
						print("OK. Comparison among nonpartitioned, equalrate, and separate models is disabled.\n\n");
					}
					else {
						print("OK. Comparison among nonpartitioned, equalrate, proportional, and separate models is disabled.\n\n");
					}
					last;
				}
				else {
					print("Cannot understand your answer.\n\n");
				}
			}
		}
		else {
			$comparemix = 0;
			print("Because you did not enable the output option for Treefinder, MrBayes or RAxML, or because you did not give multiple files, comparison among nonpartitioned, proportional and separate models is forced to disable.\n\n");
		}
		# selection of likelihood optimizer
		if ($output{'raxml'}) {
			$lnlcalc = 'raxml';
			print("Because you enabled output option for RAxML, the likelihoods will be calculated by RAxML.\n\n");
		}
		elsif ($output{'treefinder'}) {
			$lnlcalc = 'tf';
			print("Because you enabled output option for Treefinder, the likelihoods will be calculated by Treefinder.\n\n");
		}
		elsif ($output{'mrbayes'}) {
			$lnlcalc = 'tf';
			print("Because you enabled output option for MrBayes, the likelihoods will be calculated by Treefinder.\n\n");
		}
		else {
			while ($loop) {
				my $response;
				if ($comparemix && ($output{'treefinder'} || $output{'mrbayes'})) {
					print("Which do you want to use the program for likelihood calculation? (codeml/tf)\n(default: tf)\n");
				}
				else {
					print("Which do you want to use the program for likelihood calculation? (codeml/tf)\n(default: codeml)\n");
				}
				$response = <STDIN>;
				$response =~ s/\r?\n?$//;
				if ($response =~ /^\s*tf\s*$/i || $response =~ /^\s*$/ && $comparemix && ($output{'treefinder'} || $output{'mrbayes'})) {
					$lnlcalc = 'tf';
					print("OK. The likelihoods will be calculated by TF.\n\n");
					last;
				}
				elsif ($response =~ /^\s*codeml\s*$/i || $response =~ /^\s*$/) {
					$lnlcalc = 'codeml';
					print("OK. The likelihoods will be calculated by CODEML.\n\n");
					last;
				}
				else {
					print("Cannot understand your answer.\n\n");
				}
			}
		}
		# composition parameter setting
		if ($output{'phyml'}) {
			$aafreq = 'Empirical';
			print("Because you enabled output option for PHYML, the parameters of amino-acid composition will not be optimized and the empirical values will be used.\n\n");
		}
		elsif ($lnlcalc ne 'codeml') {
			while ($loop) {
				my $response;
				print("Do you want to optimize the parameters of amino-acid composition? (y/n)\n(default: n)\n");
				$response = <STDIN>;
				$response =~ s/\r?\n?$//;
				if ($response =~ /^\s*(?:yes|y)\s*$/i) {
					$aafreq = 'Estimate';
					print("OK. The parameters will be optimized.\n\n");
					last;
				}
				elsif ($response =~ /^\s*(?:no|n)\s*$/i || $response =~ /^\s*$/) {
					$aafreq = 'Empirical';
					print("OK. The empirical values will be used.\n\n");
					last;
				}
				else {
					print("Cannot understand your answer.\n\n");
				}
			}
		}
		else {
			$aafreq = 'Empirical';
			print("Because you specify CODEML as likelihood calculator, the parameters of amino-acid frequencies will be fixed to observed values.\n\n");
		}
		# the number of gamma rate categories
		if ($output{'raxml'} && !$output{'treefinder'} && !$output{'mrbayes'}) {
			$ngammacat = 4;
			print("Because you enabled output option for RAxML, the number of rate categories of discrete gamma rate heterogeneity is set to 4.\n\n");
		}
		else {
			while ($loop) {
				my $response;
				print("How many rate categories of discrete gamma rate heterogeneity do you want to consider? (integer)\n(default: 8)\n");
				$response = <STDIN>;
				$response =~ s/\r?\n?$//;
				unless ($response) { $response = 8; }
				if ($response =~ /^(\d+)$/) {
					if ($1 < 2) {
						&errorMessage(__LINE__, 'The number of rate categories is not valid.');
					}
					else {
						$ngammacat = $1;
					}
					print("OK. The number of rate categories is set to $1.\n\n");
					last;
				}
				else {
					print("Cannot understand your answer.\n\n");
				}
			}
		}
		# Invariant setting
		if ($lnlcalc eq 'tf' && !$output{'raxml'}) {
			while ($loop) {
				my $response;
				print("Do you want to consider invariant model for among-site rate variation? (y/n)\n(default: n)\n");
				$response = <STDIN>;
				$response =~ s/\r?\n?$//;
				if ($response =~ /^\s*(?:yes|y)\s*$/i) {
					$invariant = 1;
					print("OK. Invariant model for among-site rate variation will be considered.\n\n");
					last;
				}
				elsif ($response =~ /^\s*(?:no|n)\s*$/i || $response =~ /^\s*$/) {
					$invariant = 0;
					print("OK. Invariant model for among-site rate variation will not be considered.\n\n");
					last;
				}
				else {
					print("Cannot understand your answer.\n\n");
				}
			}
		}
		else {
			$invariant = 0;
			print("Because you specify CODEML or RAxML as likelihood calculator, invariant model for among-site rate variation will not be considered.\n\n");
		}
		# N-GAM and Autocorrelated Gamma
		if ($lnlcalc eq 'codeml' && !$comparemix && $output{'mrbayes'}) {
			if ($coding || @fullpathname > 1) {
				while ($loop) {
					my $response;
					print("Do you want to consider N-GAM model for among-site rate variation? (y/n)\nNote that this model is very time-consuming.\n(default: n)\n");
					$response = <STDIN>;
					$response =~ s/\r?\n?$//;
					if ($response =~ /^\s*(?:yes|y)\s*$/i) {
						$n_gam = 1;
						print("OK. N-GAM model for among-site rate variation will be considered.\n\n");
						last;
					}
					elsif ($response =~ /^\s*(?:no|n)\s*$/i || $response =~ /^\s*$/) {
						$n_gam = 0;
						print("OK. N-GAM model for among-site rate variation will not be considered.\n\n");
						last;
					}
					else {
						print("Cannot understand your answer.\n\n");
					}
				}
			}
			while ($loop) {
				my $response;
				print("Do you want to consider autocorrelated discrete gamma model for among-site rate variation? (y/n)\nNote that this model is very time-consuming.\n(default: n)\n");
				$response = <STDIN>;
				$response =~ s/\r?\n?$//;
				if ($response =~ /^\s*(?:yes|y)\s*$/i) {
					$agamma = 1;
					print("OK. Autocorrelated discrete gamma model for among-site rate variation will be considered.\n\n");
					last;
				}
				elsif ($response =~ /^\s*(?:no|n)\s*$/i || $response =~ /^\s*$/) {
					$agamma = 0;
					print("OK. Autocorrelated discrete gamma model for among-site rate variation will not be considered.\n\n");
					last;
				}
				else {
					print("Cannot understand your answer.\n\n");
				}
			}
		}
		else {
			print("Because you did not specify CODEML as likelihood calculator, because you disabled the output option for MrBayes, or because you enabled comparison among nonpartitioned, proportional and separate models, N-GAM and autocorrelated discrete gamma models for among-site rate variation will not be considered.\n\n");
		}
		if (@fullpathname > 1 && !$comparemix) {
			while ($loop) {
				my $response;
				print("Do you want to use different tree topology for parameter optimization on each locus? (y/n)\n(default: n)\n");
				$response = <STDIN>;
				$response =~ s/\r?\n?$//;
				if ($response =~ /^\s*(?:yes|y)\s*$/i) {
					$difftreeeachpart = 1;
					print("OK. Different tree topology will be used for parameter optimization on each locus.\n\n");
					last;
				}
				elsif ($response =~ /^\s*(?:no|n)\s*$/i || $response =~ /^\s*$/) {
					$difftreeeachpart = 0;
					print("OK. A common tree topology will be used for parameter optimization on each locus.\n\n");
					last;
				}
				else {
					print("Cannot understand your answer.\n\n");
				}
			}
		}
		elsif (@fullpathname > 1 && $comparemix) {
			$difftreeeachpart = 0;
			print("Because you enabled comparison among nonpartitioned, proportional and separate models, a common tree topology will be used for parameter optimization on each locus.\n\n");
		}
		while ($loop) {
			my $response;
			print("If you want to give tree(s) for parameter optimization, specify an input file name.\nOtherwise, just press enter.\n");
			$response = <STDIN>;
			$response =~ s/\r?\n?$//;
			$response =~ s/^\s*(.+?)\s*$/$1/;
			$response =~ s/^"(.+)"$/$1/;
			if ($response =~ /^\s*$/) {
				print("OK. Tree(s) for parameter optimization will be made by neighbor-joining based on K83 distances.\n\n");
				last;
			}
			elsif (-e $response && !-d $response && !-z $response) {
				$treefile = $response;
				print("OK. Tree(s) in the input file will be used in parameter optimization.\n\n");
				last;
			}
			else {
				print("The input file does not exist.\n\n");
			}
		}
		while ($loop) {
			my $response;
			print("How many processes do you want to run simultaneously? (integer)\n(default: 1)\n");
			$response = <STDIN>;
			$response =~ s/\r?\n?$//;
			unless ($response) { $response = 1; }
			if ($response =~ /^\s*(\d+)\s*$/) {
				if ($1 == 0) {
					&errorMessage(__LINE__, 'The number of processes is not valid.');
				}
				else {
					$numthreads = $1;
				}
				print("OK. The number of processes is set to $1.\n\n");
				last;
			}
			else {
				print("Cannot understand your answer.\n\n");
			}
		}
		if ($lnlcalc eq 'raxml') {
			while ($loop) {
				my $response;
				print("Which instruction do your processor support? (SSE2/SSE3/AVX/AVX2)\n(default: SSE2)\n");
				$response = <STDIN>;
				$response =~ s/\r?\n?$//;
				unless ($response) { $response = 1; }
				if ($response =~ /^\s*sse2\s*$/i || $response =~ /^\s*$/) {
					if ($numthreads == 1) {
						$raxml = 'raxmlHPC';
					}
					else {
						$raxml = 'raxmlHPC-PTHREADS';
					}
					print("OK. The likelihoods will be calculated by \"$raxml\".\n\n");
					last;
				}
				elsif ($response =~ /^\s*sse3\s*$/i) {
					if ($numthreads == 1) {
						$raxml = 'raxmlHPC-SSE3';
					}
					else {
						$raxml = 'raxmlHPC-PTHREADS-SSE3';
					}
					print("OK. The likelihoods will be calculated by \"$raxml\".\n\n");
					last;
				}
				elsif ($response =~ /^\s*avx\s*$/i) {
					if ($numthreads == 1) {
						$raxml = 'raxmlHPC-AVX';
					}
					else {
						$raxml = 'raxmlHPC-PTHREADS-AVX';
					}
					print("OK. The likelihoods will be calculated by \"$raxml\".\n\n");
					last;
				}
				elsif ($response =~ /^\s*avx2\s*$/i) {
					if ($numthreads == 1) {
						$raxml = 'raxmlHPC-AVX2';
					}
					else {
						$raxml = 'raxmlHPC-PTHREADS-AVX2';
					}
					print("OK. The likelihoods will be calculated by \"$raxml\".\n\n");
					last;
				}
				else {
					print("Cannot understand your answer.\n\n");
				}
			}
		}
		while ($loop) {
			my $response;
			print("All configurations have been completed.\nJust press enter to run!\n");
			$response = <STDIN>;
			$response =~ s/\r?\n?$//;
			if ($response =~ /^\s*$/) {
				print("OK. Please wait a while.\n\n");
				last;
			}
			else {
				print("Cannot understand your answer.\n\n");
			}
		}
	}
	if ($treefile) {
		$treefile = File::Spec->rel2abs($treefile);
	}
	unless (defined($ngammacat)) {
		if ($output{'raxml'}) {
			$ngammacat = 4;
		}
		else {
			$ngammacat = 8;
		}
	}
	unless (defined($comparemix)) {
		if ($output{'phyml'}) {
			$comparemix = 0;
		}
		else {
			$comparemix = 1;
		}
	}
	# Validation
	if ($output{'raxml'} && $lnlcalc ne 'raxml') {
		&errorMessage(__LINE__, 'The likelihood calculator must be RAxML, if output option for RAxML is enabled.');
	}
	if ($output{'treefinder'} && $lnlcalc ne 'tf') {
		&errorMessage(__LINE__, 'The likelihood calculator must be Treefinder, if output option for Treefinder is enabled.');
	}
	if ($output{'mrbayes'} && $lnlcalc ne 'tf') {
		&errorMessage(__LINE__, 'The likelihood calculator must be Treefinder, if output option for MrBayes is enabled.');
	}
	if ($output{'raxml'} && $ngammacat != 4) {
		&errorMessage(__LINE__, 'The number of gamma rate categories must be set to 4, if output option for RAxML is enabled.');
	}
	if ($output{'phyml'} && $aafreq ne 'Empirical') {
		&errorMessage(__LINE__, 'The parameters of amino-acid frequencies must be fixed to the empirical values, if output option for PHYML is enabled.');
	}
	if ($comparemix && $output{'phyml'}) {
		&errorMessage(__LINE__, 'Comparison among nonpartitioned, equalrate, proportional, and separate models cannot be enabled, if output options for Treefinder, MrBayes, and RAxML are disabled.');
	}
	if (@partname && @inputfile > 1) {
		&errorMessage(__LINE__, 'Multiple input files and command line partition specification is exclusive.');
	}
	elsif (!@partname) {
		my %temphash;
		foreach (@inputfile) {
			$_ =~ s/ /_/g;
			$_ =~ /^(.+)\.[^\.]*$/;
			my $partname = $1;
			if ($partname eq 'whole') {
				&errorMessage(__LINE__, 'Partition name "whole" is reserved by Aminosan.');
			}
			unless ($temphash{$partname}) {
				$temphash{$partname} = 1;
				push(@partname, $partname);
			}
			else {
				&errorMessage(__LINE__, "Partition name \"$partname\" is doubly specified.");
			}
		}
	}
	# calcwhole option validation
	if (@partname < 2) {
		$calcwhole = 1;
		$comparemix = 0;
	}
	# difftreeeachpart option validation
	if ($difftreeeachpart && $comparemix) {
		&errorMessage(__LINE__, 'Comparison among nonpartitioned, equalrate, proportional, and separate models cannot be enabled, if different tree topology for likelihood optimization on each locus is enabled.');
	}
	# Are there unused sites?
	for (my $i = 1; $i <= keys(%site2part); $i ++) {
		unless ($site2part{$i}) {
			&errorMessage(__LINE__, 'Partition specification is not valid. There are unused sites.');
		}
	}
	# If output option is not specified...
	if (!%output) {
		&errorMessage(__LINE__, "Output option is not specified.");
	}
	elsif (scalar(keys(%output)) > 2) {
		&errorMessage(__LINE__, "Output option is not valid.");
	}
	if ($output{'mrbayes'}) {
		foreach (@modelformrbayes) {
			$model{$_} = 1;
		}
		foreach (@heterogeneityformrbayes) {
			$heterogeneity{$_} = 1;
		}
	}
	if ($output{'phyml'}) {
		foreach (@modelforphyml) {
			$model{$_} = 1;
		}
		foreach (@heterogeneityforphyml) {
			$heterogeneity{$_} = 1;
		}
	}
	if ($output{'raxml'}) {
		foreach (@modelforraxml) {
			$model{$_} = 1;
		}
		foreach (@heterogeneityforraxml) {
			$heterogeneity{$_} = 1;
		}
	}
	if ($output{'treefinder'}) {
		foreach (@modelfortf) {
			$model{$_} = 1;
		}
		foreach (@heterogeneityfortf) {
			$heterogeneity{$_} = 1;
		}
	}
	# validation
	if ($aafreq eq 'Estimate' && $lnlcalc eq 'codeml') {
		&errorMessage(__LINE__, 'Cannot optimize the parameters of amino-acid frequencies in CODEML mode.');
	}
	if ($invariant && $lnlcalc eq 'codeml') {
		&errorMessage(__LINE__, 'Cannot use invariant model for among-site rate variation in CODEML mode.');
	}
	if ($n_gam && $lnlcalc ne 'codeml') {
		&errorMessage(__LINE__, 'Cannot use N-GAM in TF/RAxML mode.');
	}
	if ($n_gam && @fullpathname < 2 && !@partname) {
		&errorMessage(__LINE__, 'Cannot use N-GAM in single partition mode.');
	}
	if ($agamma && $lnlcalc ne 'codeml') {
		&errorMessage(__LINE__, 'Cannot use autocorrelated discrete gamma in TF/RAxML mode.');
	}
	# if RAxML mode
	if ($lnlcalc eq 'raxml' && $numthreads >= 2) {
		$raxml .= " -T $numthreads";
		$numthreads = 1;
	}
	print("done.\n\n");
}

sub range2list {
	# Input: Site of range start, Site of range end, Skip number of sites
	# Output: List of sites which belong to the range
	my ($start, $end, $skip) = @_;
	my @num;
	if ($start != 0 && $skip != 0 && $start <= $end && $skip <= $end - $start) {
		for (my $i = $start; $i <= $end; $i += $skip) {
			push(@num, $i);
		}
	}
	else {
		&errorMessage(__LINE__, 'Partition specification is not valid.');
	}
	return(@num);
}

sub checkFiles {
	print("Checking commands and files...\n");
	# Check working directory
	my $datadir;
	{
		my $maindirname;
		{
			my ($volume, $directories, $file) = File::Spec->splitpath($maindir);
			$datadir = $volume . $directories;
			$maindirname = $file;
		}
		my $tempno;
		while (-e $maindir) {
			$tempno ++;
			unless (-e $datadir . "bkup$tempno.$maindirname") {
				my $loop = 1;
				while ($loop) {
					print("\"$maindir\" already exists. Can I rename it to \"$datadir" . "bkup$tempno.$maindirname\"?\n");
					my $response;
					$response = <STDIN>;
					$response =~ s/\r?\n?$//;
					if ($response =~ /^\s*(?:yes|y)\s*$/i) {
						unless (rename($maindir, $datadir . "bkup$tempno.$maindirname")) {
							&errorMessage(__LINE__, 'Cannot change the directory name.');
						}
						else {
							print("OK. Changed to \"$datadir" . "bkup$tempno.$maindirname\".\n");
							last;
						}
					}
					elsif ($response =~ /^\s*(?:no|n)\s*$/i) {
						&errorMessage(__LINE__, 'The script was aborted.');
					}
					else {
						print("Cannot understand your answer.\n\n");
					}
				}
			}
		}
	}
	# Make working directory
	my $aminosanpath;
	BEGIN {
		use File::Spec;
		my ($volume, $directories, $file) = File::Spec->splitpath(File::Spec->rel2abs($0));
		$aminosanpath = $volume . $directories;
	}
	use lib qq($aminosanpath);
	use Cwd;
	my $devnull = File::Spec->devnull();
	my $currentdir;
	{
		my ($volume, $directories, $file) = File::Spec->splitpath(File::Spec->catfile(Cwd->cwd(), 'tmp'));
		$currentdir = $volume . $directories;
	}
	unless (mkdir($maindir)) {
		&errorMessage(__LINE__, "Cannot make \"$maindir\" directory.");
	}
	unless (chdir($maindir)) {
		&errorMessage(__LINE__, 'Cannot change working directory.');
	}
	unless (mkdir('Logs')) {
		&errorMessage(__LINE__, "Cannot make \"$maindir/Logs\" directory.");
	}
	# Check ReadSeq
	if (system('java -jar "' . $currentdir . 'readseq.jar" -h 2> ' . $devnull . ' 1> ' . $devnull)) {
		if (system('java -jar "' . $datadir . 'readseq.jar" -h 2> ' . $devnull . ' 1> ' . $devnull)) {
			if (system('java -jar "' . $aminosanpath . 'readseq.jar" -h 2> ' . $devnull . ' 1> ' . $devnull)) {
				if (system("\"$currentdir$readseq\" -h 2> " . $devnull . ' 1> ' . $devnull)) {
					if (system("\"$datadir$readseq\" -h 2> " . $devnull . ' 1> ' . $devnull)) {
						if (system("\"$aminosanpath$readseq\" -h 2> " . $devnull . ' 1> ' . $devnull)) {
							if (system("$readseq -h 2> " . $devnull . ' 1> ' . $devnull)) {
								&errorMessage(__LINE__, 'Cannot find readseq. Please install readseq.');
							}
						}
						else {
							$readseq = "\"$aminosanpath$readseq\"";
						}
					}
					else {
						$readseq = "\"$datadir$readseq\"";
					}
				}
				else {
					$readseq = "\"$currentdir$readseq\"";
				}
			}
			else {
				$readseq = 'java -jar "' . $aminosanpath . 'readseq.jar"';
			}
		}
		else {
			$readseq = 'java -jar "' . $datadir . 'readseq.jar"';
		}
	}
	else {
		$readseq = 'java -jar "' . $currentdir . 'readseq.jar"';
	}
	# Check PAUP
	unless (open(NEXUS, '> NEXUS.nex')) {
		&errorMessage(__LINE__, 'Cannot make file.');
	}
	print(NEXUS "#NEXUS\nBegin PAUP;\nQuit;\nEnd;\n");
	close(NEXUS);
	if (system("\"$currentdir$paupcmd\" -u -n NEXUS.nex 2> " . $devnull . ' 1> ' . $devnull)) {
		if (system("\"$datadir$paupcmd\" -u -n NEXUS.nex 2> " . $devnull . ' 1> ' . $devnull)) {
			if (system("\"$aminosanpath$paupcmd\" -u -n NEXUS.nex 2> " . $devnull . ' 1> ' . $devnull)) {
				if (system("$paupcmd -u -n NEXUS.nex 2> " . $devnull . ' 1> ' . $devnull)) {
					print("Cannot find PAUP*.\n");
					undef($paupcmd);
				}
			}
			else {
				$paupcmd = "\"$aminosanpath$paupcmd\"";
			}
		}
		else {
			$paupcmd = "\"$datadir$paupcmd\"";
		}
	}
	else {
		$paupcmd = "\"$currentdir$paupcmd\"";
	}
	unlink('NEXUS.nex');
	# Check PHYLIP & CODEML/TF
	{
		unless (open(SEQ, '> tempseq.phy')) {
			&errorMessage(__LINE__, 'Cannot make file.');
		}
		print(SEQ " 3 12\nTax1      ACGTACGTACGT\nTax2      AGCTAGCTAGCT\nTax3      ATCGATCGATCG\n");
		close(SEQ);
		unless (open(CMD, '> command')) {
			&errorMessage(__LINE__, 'Cannot make file.');
		}
		print(CMD "tempseq.phy\nY\n");
		close(CMD);
		if (system("\"$currentdir$protdist\" < command 2> " . $devnull . ' 1> ' . $devnull)) {
			if (system("\"$datadir$protdist\" < command 2> " . $devnull . ' 1> ' . $devnull)) {
				if (system("\"$aminosanpath$protdist\" < command 2> " . $devnull . ' 1> ' . $devnull)) {
					if (system("$protdist < command 2> " . $devnull . ' 1> ' . $devnull)) {
						&errorMessage(__LINE__, "Cannot run $protdist. Please check $protdist.");
					}
				}
				else {
					$protdist = "\"$aminosanpath$protdist\"";
				}
			}
			else {
				$protdist = "\"$datadir$protdist\"";
			}
		}
		else {
			$protdist = "\"$currentdir$protdist\"";
		}
		rename('outfile','tempdist.phy');
		if (system("\"$currentdir$protpars\" < command 2> " . $devnull . ' 1> ' . $devnull)) {
			if (system("\"$datadir$protpars\" < command 2> " . $devnull . ' 1> ' . $devnull)) {
				if (system("\"$aminosanpath$protpars\" < command 2> " . $devnull . ' 1> ' . $devnull)) {
					if (system("$protpars < command 2> " . $devnull . ' 1> ' . $devnull)) {
						&errorMessage(__LINE__, "Cannot run $protpars. Please check $protpars.");
					}
					else {
						unlink('outtree');
						unlink('outfile');
					}
				}
				else {
					unlink('outtree');
					unlink('outfile');
					$protpars = "\"$aminosanpath$protpars\"";
				}
			}
			else {
				unlink('outtree');
				unlink('outfile');
				$protpars = "\"$datadir$protpars\"";
			}
		}
		else {
			unlink('outtree');
			unlink('outfile');
			$protpars = "\"$currentdir$protpars\"";
		}
		unless (open(CMD, '> command')) {
			&errorMessage(__LINE__, 'Cannot make file.');
		}
		print(CMD "tempdist.phy\nY\n");
		close(CMD);
		if (system("\"$currentdir$neighbor\" < command 2> " . $devnull . ' 1> ' . $devnull)) {
			if (system("\"$datadir$neighbor\" < command 2> " . $devnull . ' 1> ' . $devnull)) {
				if (system("\"$aminosanpath$neighbor\" < command 2> " . $devnull . ' 1> ' . $devnull)) {
					if (system("$neighbor < command 2> " . $devnull . ' 1> ' . $devnull)) {
						&errorMessage(__LINE__, "Cannot run $neighbor. Please check $neighbor.");
					}
				}
				else {
					$neighbor = "\"$aminosanpath$neighbor\"";
				}
			}
			else {
				$neighbor = "\"$datadir$neighbor\"";
			}
		}
		else {
			$neighbor = "\"$currentdir$neighbor\"";
		}
		unlink('outfile');
		unlink('command');
		unlink('tempdist.phy');
		if ($lnlcalc eq 'codeml') {
			unless (mkdir('dat')) {
				&errorMessage(__LINE__, "Cannot make \"$maindir/dat\" directory.");
			}
			foreach my $model (keys(%model)) {
				if ($model !~ /^Poisson/ && $model !~ /\+F$/) {
					my $datfilepath = File::Spec->catfile($maindir, 'dat', "$model.txt");
					$dat{$model} = "../dat/$model.txt";
					&makeAAModelFiles($model, $datfilepath);
				}
			}
			unless (open(CMD, '> codeml.ctl')) {
				&errorMessage(__LINE__, 'Cannot make file.');
			}
			print(CMD "seqfile = tempseq.phy\ntreefile = outtree\noutfile = outfile\nseqtype = 2\nmodel = 0\n");
			close(CMD);
			if (system("\"$currentdir$codeml\" 2> " . $devnull . ' 1> ' . $devnull)) {
				if (system("\"$datadir$codeml\" 2> " . $devnull . ' 1> ' . $devnull)) {
					if (system("\"$aminosanpath$codeml\" 2> " . $devnull . ' 1> ' . $devnull)) {
						if (system("$codeml 2> " . $devnull . ' 1> ' . $devnull)) {
							&errorMessage(__LINE__, "Cannot run $codeml. Please check $codeml.");
						}
					}
					else {
						$codeml = "\"$aminosanpath$codeml\"";
					}
				}
				else {
					$codeml = "\"$datadir$codeml\"";
				}
			}
			else {
				$codeml = "\"$currentdir$codeml\"";
			}
			unlink('outfile');
			unlink('lnf');
			unlink('rst');
			unlink('rst1');
			unlink('rub');
			unlink('codeml.ctl');
		}
		elsif ($lnlcalc eq 'tf' || ($comparemix && ($output{'treefinder'} || $output{'mrbayes'}))) {
			unless (open(CMD, '> quit.tl')) {
				&errorMessage(__LINE__, 'Cannot make file.');
			}
			print(CMD "Quit\n");
			close(CMD);
			if (system("\"$currentdir$tf\" quit.tl 2> " . $devnull . ' 1> ' . $devnull)) {
				if (system("\"$datadir$tf\" quit.tl 2> " . $devnull . ' 1> ' . $devnull)) {
					if (system("\"$aminosanpath$tf\" quit.tl 2> " . $devnull . ' 1> ' . $devnull)) {
						my $tfdir;
						{
							my ($volume, $directories, $file) = File::Spec->splitpath(File::Spec->catfile($ENV{'TFDIR'}, 'tmp'));
							$tfdir = $volume . $directories;
						}
						if (system("\"$tfdir$tf\" quit.tl 2> " . $devnull . ' 1> ' . $devnull)) {
							if (system("$tf quit.tl 2> " . $devnull . ' 1> ' . $devnull)) {
								&errorMessage(__LINE__, "Cannot run $tf. Please check $tf.");
							}
						}
						else {
							$tf = "\"$tfdir$tf\"";
						}
					}
					else {
						$tf = "\"$aminosanpath$tf\"";
					}
				}
				else {
					$tf = "\"$datadir$tf\"";
				}
			}
			else {
				$tf = "\"$currentdir$tf\"";
			}
			unlink('quit.tl');
		}
		elsif ($lnlcalc eq 'raxml' || ($comparemix && $output{'raxml'})) {
			if (system("\"$currentdir$raxml\" -v 2> " . $devnull . ' 1> ' . $devnull)) {
				if (system("\"$datadir$raxml\" -v 2> " . $devnull . ' 1> ' . $devnull)) {
					if (system("\"$aminosanpath$raxml\" -v 2> " . $devnull . ' 1> ' . $devnull)) {
						if (system("$raxml -v 2> " . $devnull . ' 1> ' . $devnull)) {
							&errorMessage(__LINE__, "Cannot run $raxml. Please check $raxml.");
						}
					}
					else {
						$raxml = "\"$aminosanpath$raxml\"";
					}
				}
				else {
					$raxml = "\"$datadir$raxml\"";
				}
			}
			else {
				$raxml = "\"$currentdir$raxml\"";
			}
		}
		unlink('outtree');
		unlink('tempseq.phy');
	}
	unless (chdir('..')) {
		&errorMessage(__LINE__, 'Cannot change working directory.');
	}
	print("done.\n\n");
}

sub convertSequences {
	print("Converting sequence files...\n");
	unless (chdir($maindir)) {
		&errorMessage(__LINE__, 'Cannot change working directory.');
	}
	push(@allpart, 'whole');
	my %temptaxa;
	for (my $i = 0; $i < @inputfile; $i ++) {
		# Preprocess input file
		my $nexusornot;
		my $tempntax;
		my $tempnchar;
		unless (open(OUTPUT, "> $inputfile[$i]")) {
			&errorMessage(__LINE__, "Cannot make \"$maindir/$inputfile[$i]\".");
		}
		unless (open(INPUT, "< $fullpathname[$i]")) {
			&errorMessage(__LINE__, "Cannot open \"$fullpathname[$i]\".");
		}
		while (<INPUT>) {
			s/\r?\n?$//;
			if (/^#NEXUS\s*$/i) {
				print(OUTPUT "$_\n");
				$nexusornot = 1;
			}
			if ($nexusornot == 1 && (/Begin\s+Data\s*;/i || /Begin\s+Characters\s*;/i)) {
				$nexusornot = 2;
			}
			if ($nexusornot == 2 && /NTax=(\d+)/i) {
				$tempntax = $1;
			}
			if ($nexusornot == 2 && /NChar=(\d+)/i) {
				$tempnchar = $1;
			}
			if ($nexusornot == 2 && /End\s*;/i) {
				print(OUTPUT "$_\n");
				$nexusornot = 1;
			}
			if ($nexusornot != 1) {
				print(OUTPUT "$_\n");
			}
		}
		close(INPUT);
		if ($nexusornot && $tempntax && $tempnchar && $paupcmd) {
			print(OUTPUT "Begin PAUP;\n\tExport File=$inputfile[$i]" . "_temp.nex Format=NEXUS;\n\tQuit;\nEnd;\n");
		}
		close(OUTPUT);
		my $outpart;
		if (@inputfile == 1) {
			$outpart = 'whole';
		}
		else {
			$outpart = $partname[$i];
			push(@allpart, $partname[$i]);
		}
		if ($nexusornot && $tempntax && $tempnchar && $paupcmd) {
			if (system("$paupcmd $inputfile[$i] > Logs/convert_$inputfile[$i].txt")) {
				&errorMessage(__LINE__, "Cannot run $paupcmd. Please check $paupcmd.");
			}
			unless (open(TEMP, "< $inputfile[$i]" . "_temp.nex")) {
				&errorMessage(__LINE__, "Cannot open \"$maindir/$inputfile[$i]" . '_temp.nex".');
			}
			unless (open(OUTPUT, "> data_$outpart.nex")) {
				&errorMessage(__LINE__, "Cannot make \"$maindir/data_$outpart.nex\".");
			}
			while (<TEMP>) {
				s/(datatype=dna)[^\r\n]+/$1 gap=- missing=?;/i;
				print(OUTPUT $_);
			}
			close(OUTPUT);
			close(TEMP);
			unlink("$inputfile[$i]" . "_temp.nex");
		}
		else {
			# Convert to FASTA by ReadSeq
			if (system("$readseq -a -C -f8 -o=$inputfile[$i].fas $inputfile[$i]")) {
				&errorMessage(__LINE__, 'Cannot convert sequence file.');
			}
			# Convert ReadSeq output FASTA to NEXUS
			unless (open(FASTA, "< $inputfile[$i].fas")) {
				&errorMessage(__LINE__, "Cannot open \"$maindir/$inputfile[$i].fas\".");
			}
			$tempntax = 0;
			while (<FASTA>) {
				if (/^>[^\s,]+.*?,? (\d+) .+\n$/) {
					$tempntax ++;
					unless ($tempnchar) {
						$tempnchar = $1;
					}
					else {
						if ($tempnchar != $1) {
							&errorMessage(__LINE__, 'Sequence lengths are not equal.');
						}
					}
				}
			}
			close(FASTA);
			if ($tempntax < 3) {
				&errorMessage(__LINE__, 'The number of taxon is too small or file parsing error.');
			}
			unless (open(NEXUS, "> data_$outpart.nex")) {
				&errorMessage(__LINE__, "Cannot make \"$maindir/data_$outpart.nex\".");
			}
			print(NEXUS '#NEXUS' . "\n\nBegin Data;\n\tDimensions NTax=$tempntax NChar=$tempnchar;\n\tFormat DataType=PROTEIN Gap=- Missing=?;\n\tMatrix");
			unless (open(FASTA, "< $inputfile[$i].fas")) {
				&errorMessage(__LINE__, "Cannot open \"$maindir/$inputfile[$i].fas\".");
			}
			while (<FASTA>) {
				s/^>([^\s,]+).*?,? \d+ .+\r?\n?$/\n$1 /;
				s/^([^>][^\r\n]*?)\r?\n?$/$1/;
				print(NEXUS $_);
			}
			close(FASTA);
			print(NEXUS "\n\t;\nEnd;\n");
			close(NEXUS);
		}
		# Output PHYLIP format
		{
			unless (open(PHYLIP, "> data_$outpart.phy")) {
				&errorMessage(__LINE__, "Cannot make \"$maindir/data_$outpart.phy\".");
			}
			print(PHYLIP " $tempntax $tempnchar\n");
			unless (open(NEXUS, "< data_$outpart.nex")) {
				&errorMessage(__LINE__, "Cannot open \"$maindir/data_$outpart.nex\".");
			}
			my $line = 0;
			my @temptaxa;
			while (<NEXUS>) {
				$line ++;
				if ($line < 7) {
					next;
				}
				elsif ($line >= 7 && $line < 7 + $tempntax) {
					if (/^(\S+)\s+(\S+)\r?\n?$/) {
						if (!exists($temptaxa{$1})) {
							push(@temptaxa, $1);
							$temptaxa{$1} = sprintf("Tax%-5d", $line - 6);
						}
						print(PHYLIP $temptaxa{$1} . "  $2\n");
					}
					else {
						&errorMessage(__LINE__, "\"$maindir/data_$outpart.nex\" is not valid.");
					}
				}
				else {
					last;
				}
			}
			close(NEXUS);
			close(PHYLIP);
			# Output taxon list
			unless (-e "taxalist.txt") {
				unless (open(TXT, "> taxalist.txt")) {
					&errorMessage(__LINE__, "Cannot make \"$maindir/taxalist.txt\".");
				}
				for (my $i = 0; $i < $tempntax; $i ++) {
					print(TXT 'Tax' . ($i + 1) . "\t" . $temptaxa[$i] . "\n");
				}
				close(TXT);
			}
		}
		unless ($ntax) {
			$ntax = $tempntax;
		}
		if ($ntax != $tempntax) {
			&errorMessage(__LINE__, 'Invalid input files. The number of taxa is not equal.');
		}
		if ($outpart eq 'whole' && @partname == 1 && !$part2site{$partname[0]}) {
			push(@{$part2site{$partname[0]}}, &range2list(1, $tempnchar, 1));
			$nchar = $tempnchar;
		}
		elsif ($outpart eq 'whole') {
			$nchar = $tempnchar;
		}
		else {
			push(@{$part2site{$outpart}}, &range2list($nchar + 1, $nchar + $tempnchar, 1));
			$nchar += $tempnchar;
		}
	}
	unless (chdir('..')) {
		&errorMessage(__LINE__, 'Cannot change working directory.');
	}
	print("done.\n\n");
}

sub splitSequences {
	print("Splitting sequences...\n");
	unless (chdir($maindir)) {
		&errorMessage(__LINE__, 'Cannot change working directory.');
	}
	# Read sequence file and output to separated files
	unless (open(NEXUS, "< data_whole.nex")) {
		&errorMessage(__LINE__, "Cannot open \"$maindir/data_whole.nex\".");
	}
	my $line = 0;
	while (<NEXUS>) {
		$line ++;
		if ($line < 7) {
			next;
		}
		elsif ($line >= 7 && $line < 7 + $ntax) {
			if (/^(\S+)\s+(\S+)\r?\n?$/) {
				push(@taxa, $1);
				for (my $i = 0; $i < $nchar; $i ++) {
					push(@{$seqs{$1}}, uc(substr($2, $i, 1)));
				}
			}
			else {
				&errorMessage(__LINE__, "\"$maindir/data_whole.nex\" is not valid.");
			}
		}
		else {
			last;
		}
	}
	close(NEXUS);
	&calcTaxLength();
	if (@partname > 1) {
		foreach my $partname (@partname) {
			push(@allpart, $partname);
			unless (open(OUT, "> data_$partname.nex")) {
				&errorMessage(__LINE__, "Cannot make \"$maindir/data_$partname.nex\".");
			}
			print(OUT '#NEXUS' . "\n\nBegin Data;\n\tDimensions NTax=$ntax NChar=" . scalar(@{$part2site{$partname}}) . ";\n\tFormat DataType=PROTEIN Gap=- Missing=?;\n\tMatrix\n");
			foreach my $taxon (@taxa) {
				printf(OUT "%-*s ", $taxlength, $taxon);
				foreach my $site (@{$part2site{$partname}}) {
					print(OUT $seqs{$taxon}[$site - 1]);
				}
				print(OUT "\n");
			}
			print(OUT "\t;\nEnd;\n");
			close(OUT);
			{
				unless (open(OUT, "> data_$partname.phy")) {
					&errorMessage(__LINE__, "Cannot make \"$maindir/data_$partname.phy\".");
				}
				print(OUT " $ntax " . scalar(@{$part2site{$partname}}) . "\n");
				for (my $i = 0; $i < @taxa; $i ++) {
					printf(OUT "Tax%-5d  ", $i + 1);
					foreach my $site (@{$part2site{$partname}}) {
						print(OUT $seqs{$taxa[$i]}[$site - 1]);
					}
					print(OUT "\n");
				}
				close(OUT);
			}
		}
	}
	unless (chdir('..')) {
		&errorMessage(__LINE__, 'Cannot change working directory.');
	}
	print("done.\n\n");
}

sub combineSequences {
	print("Combining sequences...\n");
	unless (chdir($maindir)) {
		&errorMessage(__LINE__, 'Cannot change working directory.');
	}
	my %taxa;
	for (my $i = 0; $i < @partname; $i ++) {
		unless (open(NEXUS, "< data_$partname[$i].nex")) {
			&errorMessage(__LINE__, "Cannot open \"$maindir/data_$partname[$i].nex\".");
		}
		my $line = 0;
		while (<NEXUS>) {
			$line ++;
			if ($line < 7) {
				next;
			}
			elsif ($line >= 7 && $line < 7 + $ntax) {
				if (/^(\S+)\s+(\S+)\r?\n?$/) {
					if ($i == 0 && !exists($taxa{$1})) {
						push(@taxa, $1);
						$taxa{$1} = 1;
					}
					elsif ($i == 0 && exists($taxa{$1})) {
						&errorMessage(__LINE__, 'Duplicated taxon.');
					}
					elsif (!exists($taxa{$1})) {
						&errorMessage(__LINE__, 'Input files have different taxa set.');
					}
					for (my $j = 0; $j < scalar(@{$part2site{$partname[$i]}}); $j ++) {
						push(@{$seqs{$1}}, uc(substr($2, $j, 1)));
					}
				}
				else {
					&errorMessage(__LINE__, "\"$maindir/data_$partname[$i].nex\" is not valid.");
				}
			}
			else {
				last;
			}
		}
		close(NEXUS);
	}
	&calcTaxLength();
	unless (open(OUT, "> data_whole.nex")) {
		&errorMessage(__LINE__, "Cannot make \"$maindir/data_whole.nex\".");
	}
	print(OUT '#NEXUS' . "\n\nBegin Data;\n\tDimensions NTax=$ntax NChar=$nchar;\n\tFormat DataType=PROTEIN Gap=- Missing=?;\n\tMatrix\n");
	foreach my $taxon (@taxa) {
		printf(OUT "%-*s ", $taxlength, $taxon);
		for (my $i = 0; $i < $nchar; $i ++) {
			print(OUT $seqs{$taxon}[$i]);
		}
		print(OUT "\n");
	}
	print(OUT "\t;\nEnd;\n");
	close(OUT);
	{
		unless (open(OUT, "> data_whole.phy")) {
			&errorMessage(__LINE__, "Cannot make \"$maindir/data_whole.phy\".");
		}
		print(OUT " $ntax $nchar\n");
		for (my $i = 0; $i < @taxa; $i ++) {
			printf(OUT "Tax%-5d  ", $i + 1);
			for (my $j = 0; $j < $nchar; $j ++) {
				print(OUT $seqs{$taxa[$i]}[$j]);
			}
			print(OUT "\n");
		}
		close(OUT);
	}
	unless (chdir('..')) {
		&errorMessage(__LINE__, 'Cannot change working directory.');
	}
	print("done.\n\n");
}

sub calcTaxLength {
	foreach my $taxon (@taxa) {
		if (length($taxon) > $taxlength) {
			$taxlength = length($taxon);
		}
	}
}

sub setPartitions {
	unless (chdir($maindir)) {
		&errorMessage(__LINE__, 'Cannot change working directory.');
	}
	# CODEML mode
	if ($calcwhole && $lnlcalc eq 'codeml') {
		# Convert PHYLIP to PAML for genes
		if (@partname > 1) {
			unless (open(PAML, "> data_whole_gene.phy")) {
				&errorMessage(__LINE__, "Cannot make \"$maindir/data_whole_gene.phy\".");
			}
			print(PAML " $ntax $nchar G\n");
			print(PAML 'G ' . scalar(@partname));
			foreach my $partname (@partname) {
				print(PAML ' ' . scalar(@{$part2site{$partname}}));
			}
			print(PAML "\n");
			for (my $i = 0; $i < @taxa; $i ++) {
				printf(PAML "Tax%-5d  ", $i + 1);
				for (my $j = 0; $j < $nchar; $j ++) {
					print(PAML $seqs{$taxa[$i]}[$j]);
				}
				print(PAML "\n");
			}
			close(PAML);
		}
	}
	unless (chdir('..')) {
		&errorMessage(__LINE__, 'Cannot change working directory.');
	}
}

sub outputConfigurations {
	print("Outputting configuration file...\n");
	unless (chdir($maindir)) {
		&errorMessage(__LINE__, 'Cannot change working directory.');
	}
	unless (open(OUTPUT, '> configuration.txt')) {
		&errorMessage(__LINE__, "Cannot make \"$maindir/configuration.txt\".");
	}
	print(OUTPUT "Start date and time: " . localtime() . "\nAminosan version: $buildno\nInput file(s)\n\t" . join("\n\t", @inputfile) . "\nTarget software(s)\n\t" . join("\n\t", keys(%output)) . "\nLikelihoods optimizer: $lnlcalc\nPartitioning options\n\tConsidering nonpartitioning of loci: ");
	if ($calcwhole) {
		print(OUTPUT "Enabled\n");
	}
	else {
		print(OUTPUT "Disabled\n");
	}
	print(OUTPUT "\tConsidering partitioning of loci: ");
	if (@partname > 1) {
		print(OUTPUT "Enabled\n");
	}
	else {
		print(OUTPUT "Disabled\n");
	}
	print(OUTPUT "Model options\n\tParameters of amino-acid frequencies were ");
	if ($aafreq eq 'Estimate') {
		print(OUTPUT "optimized.\n");
	}
	else {
		print(OUTPUT "given by data but not optimized.\n");
	}
	print(OUTPUT "\tConsidering discrete gamma model for among-site rate variation: Enabled\n\tThe number of gamma rate categories: $ngammacat\n\tConsidering invariant model for among-site rate variation: ");
	if ($invariant) {
		print(OUTPUT "Enabled\n");
	}
	else {
		print(OUTPUT "Disabled\n");
	}
	print(OUTPUT "\tConsidering N-GAM model for among-site rate variation: ");
	if ($n_gam) {
		print(OUTPUT "Enabled\n");
	}
	else {
		print(OUTPUT "Disabled\n");
	}
	print(OUTPUT "\tConsidering autocorrelated discrete gamma model for among-site rate variation: ");
	if ($agamma) {
		print(OUTPUT "Enabled\n");
	}
	else {
		print(OUTPUT "Disabled\n");
	}
	print(OUTPUT "\tComparison among nonpartitioned, proportional, and separate models: ");
	if ($comparemix) {
		print(OUTPUT "Enabled\n");
	}
	else {
		print(OUTPUT "Disabled\n");
	}
	print(OUTPUT "Tree options\n\tTree topologies were ");
	if ($treefile) {
		print(OUTPUT "given by \"$treefile\".\n");
	}
	elsif ($algorithm eq 'NJK83') {
		print(OUTPUT "made from K83 distances by neighbor-joining method.\n");
	}
	print(OUTPUT "\tDifferent trees for different loci: ");
	if ($difftreeeachpart) {
		print(OUTPUT "Enabled\n");
	}
	else {
		print(OUTPUT "Disabled\n");
	}
	print(OUTPUT "Partition(s)\n\twhole: all sites\n");
	if (@partname > 1) {
		foreach my $partname (@partname) {
			print(OUTPUT "\t$partname: @{$part2site{$partname}}\n");
		}
	}
	close(OUTPUT);
	unless (chdir('..')) {
		&errorMessage(__LINE__, 'Cannot change working directory.');
	}
	print("done.\n\n");
}

sub setCandidateModels {
	if ($lnlcalc eq 'tf') {
		delete($heterogeneity{'AGamma'});
		delete($heterogeneity{'Gene'});
		delete($heterogeneity{'Gene1Gamma'});
		delete($heterogeneity{'GeneNGamma'});
		delete($heterogeneity{'Gene1AGamma'});
		delete($heterogeneity{'GeneNAGamma'});
	}
	elsif ($lnlcalc eq 'codeml') {
		delete($heterogeneity{'Invariant'});
		delete($heterogeneity{'GammaInvariant'});
	}
	elsif ($lnlcalc eq 'raxml') {
		foreach my $heterogeneity (keys(%heterogeneity)) {
			if ($heterogeneity ne 'Gamma') {
				delete($heterogeneity{$heterogeneity});
			}
		}
	}
	else {
		&errorMessage(__LINE__, 'Invalid likelihood calculation program.');
	}
	# If output option for MrBayes and comparemix is enabled
	if ($output{'mrbayes'} && $comparemix) {
		delete($heterogeneity{'AGamma'});
		delete($heterogeneity{'Gene'});
		delete($heterogeneity{'Gene1Gamma'});
		delete($heterogeneity{'GeneNGamma'});
		delete($heterogeneity{'Gene1AGamma'});
		delete($heterogeneity{'GeneNAGamma'});
	}
	# If invariant is disabled
	unless ($invariant) {
		delete($heterogeneity{'Invariant'});
		delete($heterogeneity{'GammaInvariant'});
	}
	# If N-GAM is disabled
	unless ($n_gam) {
		delete($heterogeneity{'GeneNGamma'});
		delete($heterogeneity{'GeneNAGamma'});
	}
	# If AGamma is disabled
	unless ($agamma) {
		delete($heterogeneity{'AGamma'});
		delete($heterogeneity{'Gene1AGamma'});
		delete($heterogeneity{'GeneNAGamma'});
	}
	foreach my $heterogeneity (keys(%heterogeneity)) {
		if ($heterogeneity =~ /^Gene/ && @partname < 2) {
			delete($heterogeneity{$heterogeneity});
		}
	}
	# Candidate models of each partition
	foreach my $partname (@allpart) {
		if ($partname ne 'whole' || $calcwhole) {
			foreach my $model (keys(%model)) {
				if ($partname =~ /_Poisson$/i || $partname eq 'whole' && @partname < 2 && $partname[0] =~ /_Poisson$/i) {
					if ($model =~ /^Poisson/) {
						$models{$partname}{$model} = 1;
					}
					else {
						next;
					}
				}
				elsif ($partname =~ /_BLOSUM62$/i || $partname eq 'whole' && @partname < 2 && $partname[0] =~ /_BLOSUM62$/i) {
					if ($model =~ /^BLOSUM62/) {
						$models{$partname}{$model} = 1;
					}
					else {
						next;
					}
				}
				elsif ($partname =~ /_cpREV$/i || $partname eq 'whole' && @partname < 2 && $partname[0] =~ /_cpREV$/i) {
					if ($model =~ /^cpREV/) {
						$models{$partname}{$model} = 1;
					}
					else {
						next;
					}
				}
				elsif ($partname =~ /_Dayhoff$/i || $partname eq 'whole' && @partname < 2 && $partname[0] =~ /_Dayhoff$/i) {
					if ($model =~ /^Dayhoff/) {
						$models{$partname}{$model} = 1;
					}
					else {
						next;
					}
				}
				elsif ($partname =~ /_JTT$/i || $partname eq 'whole' && @partname < 2 && $partname[0] =~ /_JTT$/i) {
					if ($model =~ /^JTT/) {
						$models{$partname}{$model} = 1;
					}
					else {
						next;
					}
				}
				elsif ($partname =~ /_LG$/i || $partname eq 'whole' && @partname < 2 && $partname[0] =~ /_LG$/i) {
					if ($model =~ /^LG/) {
						$models{$partname}{$model} = 1;
					}
					else {
						next;
					}
				}
				elsif ($partname =~ /_mtArt$/i || $partname eq 'whole' && @partname < 2 && $partname[0] =~ /_mtArt$/i) {
					if ($model =~ /^mtArt/) {
						$models{$partname}{$model} = 1;
					}
					else {
						next;
					}
				}
				elsif ($partname =~ /_mtMam$/i || $partname eq 'whole' && @partname < 2 && $partname[0] =~ /_mtMam$/i) {
					if ($model =~ /^mtMam/) {
						$models{$partname}{$model} = 1;
					}
					else {
						next;
					}
				}
				elsif ($partname =~ /_mtREV$/i || $partname eq 'whole' && @partname < 2 && $partname[0] =~ /_mtREV$/i) {
					if ($model =~ /^mtREV/) {
						$models{$partname}{$model} = 1;
					}
					else {
						next;
					}
				}
				elsif ($partname =~ /_mtZoa$/i || $partname eq 'whole' && @partname < 2 && $partname[0] =~ /_mtZoa$/i) {
					if ($model =~ /^mtZoa/) {
						$models{$partname}{$model} = 1;
					}
					else {
						next;
					}
				}
				elsif ($partname =~ /_PMB$/i || $partname eq 'whole' && @partname < 2 && $partname[0] =~ /_PMB$/i) {
					if ($model =~ /^PMB/) {
						$models{$partname}{$model} = 1;
					}
					else {
						next;
					}
				}
				elsif ($partname =~ /_rtREV$/i || $partname eq 'whole' && @partname < 2 && $partname[0] =~ /_rtREV$/i) {
					if ($model =~ /^rtREV/) {
						$models{$partname}{$model} = 1;
					}
					else {
						next;
					}
				}
				elsif ($partname =~ /_HIVb$/i || $partname eq 'whole' && @partname < 2 && $partname[0] =~ /_HIVb$/i) {
					if ($model =~ /^HIVb/) {
						$models{$partname}{$model} = 1;
					}
					else {
						next;
					}
				}
				elsif ($partname =~ /_HIVw$/i || $partname eq 'whole' && @partname < 2 && $partname[0] =~ /_HIVw$/i) {
					if ($model =~ /^HIVw/) {
						$models{$partname}{$model} = 1;
					}
					else {
						next;
					}
				}
				elsif ($partname =~ /_VT$/i || $partname eq 'whole' && @partname < 2 && $partname[0] =~ /_VT$/i) {
					if ($model =~ /^VT/) {
						$models{$partname}{$model} = 1;
					}
					else {
						next;
					}
				}
				elsif ($partname =~ /_WAG$/i || $partname eq 'whole' && @partname < 2 && $partname[0] =~ /_WAG$/i) {
					if ($model =~ /^WAG/) {
						$models{$partname}{$model} = 1;
					}
					else {
						next;
					}
				}
				elsif ($partname =~ /_FLU$/i || $partname eq 'whole' && @partname < 2 && $partname[0] =~ /_FLU$/i) {
					if ($model =~ /^FLU/) {
						$models{$partname}{$model} = 1;
					}
					else {
						next;
					}
				}
				elsif ($partname =~ /_stmtREV$/i || $partname eq 'whole' && @partname < 2 && $partname[0] =~ /_stmtREV$/i) {
					if ($model =~ /^stmtREV/) {
						$models{$partname}{$model} = 1;
					}
					else {
						next;
					}
				}
				elsif ($partname =~ /_LG4M$/i || $partname eq 'whole' && @partname < 2 && $partname[0] =~ /_LG4M$/i) {
					if ($model =~ /^LG4M/) {
						$models{$partname}{$model} = 1;
					}
					else {
						next;
					}
				}
				elsif ($partname =~ /_LG4X$/i || $partname eq 'whole' && @partname < 2 && $partname[0] =~ /_LG4X$/i) {
					if ($model =~ /^LG4X/) {
						$models{$partname}{$model} = 1;
					}
					else {
						next;
					}
				}
				elsif ($partname =~ /_nc$/i || $partname eq 'whole' && @partname < 2 && $partname[0] =~ /_nc$/i) {
					if ($model =~ /^(?:BLOSUM62|Dayhoff|JTT|LG|PMB|VT|WAG|LG4M|LG4X)/) {
						$models{$partname}{$model} = 1;
					}
					else {
						next;
					}
				}
				elsif ($partname =~ /_mt$/i || $partname eq 'whole' && @partname < 2 && $partname[0] =~ /_mt$/i) {
					if ($model =~ /^(?:mtArt|mtMam|mtREV|mtZoa|stmtREV)/) {
						$models{$partname}{$model} = 1;
					}
					else {
						next;
					}
				}
				elsif ($partname =~ /_cp$/i || $partname eq 'whole' && @partname < 2 && $partname[0] =~ /_cp$/i) {
					if ($model =~ /^cpREV/) {
						$models{$partname}{$model} = 1;
					}
					else {
						next;
					}
				}
				elsif ($partname =~ /_rt$/i || $partname eq 'whole' && @partname < 2 && $partname[0] =~ /_rt$/i) {
					if ($model =~ /^(?:rtREV|HIVb|HIVw|FLU)/) {
						$models{$partname}{$model} = 1;
					}
					else {
						next;
					}
				}
				elsif ($partname ne 'whole' || $partname eq 'whole' && @partname < 2) {
					$models{$partname}{$model} = 1;
				}
			}
			foreach my $heterogeneity (keys(%heterogeneity)) {
				if (
					($partname ne 'whole' && $heterogeneity !~ /^Gene/) || 
					($heterogeneity ne 'AGamma' && $heterogeneity !~ /^Gene/) || 
					($partname eq 'whole' && $heterogeneity ne 'AGamma' && $heterogeneity !~ /^Gene/) || 
					($partname eq 'whole' && $heterogeneity eq 'AGamma' && @partname < 2) || 
					($partname eq 'whole' && $heterogeneity eq 'Gene' && @partname > 1) || 
					($partname eq 'whole' && $heterogeneity =~ /Gene(?:1|N)Gamma/ && @partname > 1) || 
					($partname eq 'whole' && $heterogeneity =~ /Gene(?:1|N)AGamma/ && @partname > 1)
				) {
					$heterogeneities{$partname}{$heterogeneity} = 1;
				}
			}
		}
	}
	if (@partname > 1 && $calcwhole) {
		foreach my $partname (@allpart) {
			if ($partname ne 'whole') {
				foreach my $model (keys(%{$models{$partname}})) {
					$models{'whole'}{$model} = 1;
				}
			}
		}
	}
}

sub runChiSquareTest {
	print("Testing homogeneity of amino acid composition across taxa...\n");
	unless (chdir($maindir)) {
		&errorMessage(__LINE__, 'Cannot change working directory.');
	}
	unless (mkdir('Chisq')) {
		&errorMessage(__LINE__, "Cannot make \"$maindir/Chisq\" directory.");
	}
	use Statistics::ChisqIndep;
	foreach my $partname (@allpart) {
		my @obs;
		my @sites;
		my @temptaxa;
		if ($partname eq 'whole') {
			@sites = (1..$nchar);
		}
		else {
			@sites = @{$part2site{$partname}};
		}
		foreach my $taxon (@taxa) {
			my @taxobs = (0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
			foreach my $site (@sites) {
				if ($seqs{$taxon}[$site - 1] =~ /^A$/i) {
					$taxobs[0] ++;
				}
				elsif ($seqs{$taxon}[$site - 1] =~ /^R$/i) {
					$taxobs[1] ++;
				}
				elsif ($seqs{$taxon}[$site - 1] =~ /^N$/i) {
					$taxobs[2] ++;
				}
				elsif ($seqs{$taxon}[$site - 1] =~ /^D$/i) {
					$taxobs[3] ++;
				}
				elsif ($seqs{$taxon}[$site - 1] =~ /^C$/i) {
					$taxobs[4] ++;
				}
				elsif ($seqs{$taxon}[$site - 1] =~ /^Q$/i) {
					$taxobs[5] ++;
				}
				elsif ($seqs{$taxon}[$site - 1] =~ /^E$/i) {
					$taxobs[6] ++;
				}
				elsif ($seqs{$taxon}[$site - 1] =~ /^G$/i) {
					$taxobs[7] ++;
				}
				elsif ($seqs{$taxon}[$site - 1] =~ /^H$/i) {
					$taxobs[8] ++;
				}
				elsif ($seqs{$taxon}[$site - 1] =~ /^I$/i) {
					$taxobs[9] ++;
				}
				elsif ($seqs{$taxon}[$site - 1] =~ /^L$/i) {
					$taxobs[10] ++;
				}
				elsif ($seqs{$taxon}[$site - 1] =~ /^K$/i) {
					$taxobs[11] ++;
				}
				elsif ($seqs{$taxon}[$site - 1] =~ /^M$/i) {
					$taxobs[12] ++;
				}
				elsif ($seqs{$taxon}[$site - 1] =~ /^F$/i) {
					$taxobs[13] ++;
				}
				elsif ($seqs{$taxon}[$site - 1] =~ /^P$/i) {
					$taxobs[14] ++;
				}
				elsif ($seqs{$taxon}[$site - 1] =~ /^S$/i) {
					$taxobs[15] ++;
				}
				elsif ($seqs{$taxon}[$site - 1] =~ /^T$/i) {
					$taxobs[16] ++;
				}
				elsif ($seqs{$taxon}[$site - 1] =~ /^W$/i) {
					$taxobs[17] ++;
				}
				elsif ($seqs{$taxon}[$site - 1] =~ /^Y$/i) {
					$taxobs[18] ++;
				}
				elsif ($seqs{$taxon}[$site - 1] =~ /^V$/i) {
					$taxobs[19] ++;
				}
			}
			if ($taxobs[0] + $taxobs[1] + $taxobs[2] + $taxobs[3] + $taxobs[4] + $taxobs[5] + $taxobs[6] + $taxobs[7] + $taxobs[8] + $taxobs[9] + $taxobs[10] + $taxobs[11] + $taxobs[12] + $taxobs[13] + $taxobs[14] + $taxobs[15] + $taxobs[16] + $taxobs[17] + $taxobs[18] + $taxobs[19] > 0) {
				push(@obs, \@taxobs);
				push(@temptaxa, $taxon);
			}
		}
		my @aminoacid = ('         A', '         R', '         N', '         D', '         C', '         Q', '         E', '         G', '         H', '         I', '         L', '         K', '         M', '         F', '         P', '         S', '         T', '         W', '         Y', '         V');
		for (my $i = 19; $i >= 0; $i --) {
			my $sum = 0;
			for (my $j = 0; $j < scalar(@obs); $j ++) {
				$sum += $obs[$j][$i];
			}
			if ($sum == 0) {
				for (my $j = 0; $j < scalar(@obs); $j ++) {
					splice(@{$obs[$j]}, $i, 1);
					splice(@aminoacid, $i, 1);
				}
			}
		}
		my $chi = new Statistics::ChisqIndep;
		$chi->load_data(\@obs);
		if ($chi->{valid}) {
			unless (open(RESULT, "> Chisq/chisq_$partname.txt")) {
				&errorMessage(__LINE__, "Cannot make \"$maindir/Chisq/chisq_$partname.txt\".");
			}
			#output the contingency table
			my $obs = $chi->{obs}; # observed values
			my $exp = $chi->{expected}; # expected values
			my $rtotals = $chi->{rtotals}; # row totals
			my $ctotals = $chi->{ctotals}; # column totals
			my $p_value = $chi->{p_value}; # p value
			print(RESULT "Type of Amino Acids: " . $chi->{cols} . "\n");
			print(RESULT "Number of Taxa: " . $chi->{rows} . "\n"); 
			print(RESULT "Degree of Freedom: " . $chi->{df} . "\n");
			print(RESULT "Total Count: " . $chi->{total} . "\n");
			print(RESULT "Chi-square Statistic: " . $chi->{chisq_statistic} . "\n");
			print(RESULT "p-value: " . $p_value . "\n");
			print(RESULT "\n");
			printf(RESULT "%*s ", $taxlength, ' ');
			foreach (@aminoacid) {
				print(RESULT "\t$_");
			}
			print(RESULT "\t    rtotal\n");
			my $warning1 = 0;
			my $warning2 = 0;
			for (my $i = 0; $i < $chi->{rows}; $i ++) {
				printf(RESULT "%-*s \t", $taxlength, $temptaxa[$i]);
				for (my $j = 0; $j < $chi->{cols}; $j ++) {
					printf(RESULT "%10d\t", $obs->[$i]->[$j]);
				}
				printf(RESULT "%10d\n", $rtotals->[$i]);
				printf(RESULT "%*s ", $taxlength, ' ');
				for (my $j = 0; $j < $chi->{cols}; $j ++) {
					printf(RESULT "\t%10f", $exp->[$i]->[$j]);
					if ($exp->[$i]->[$j] < 1) {
						$warning1 ++;
					}
					elsif ($exp->[$i]->[$j] < 5) {
						$warning2 ++;
					}
				}
				print(RESULT "\n"); 
			}
			printf(RESULT "%-*s \t", $taxlength, 'ctotal');
			for (my $j = 0; $j < $chi->{cols}; $j ++) {
				printf(RESULT "%10d\t", $ctotals->[$j]);
			}
			#output total counts
			printf(RESULT "%10d\n", $chi->{total});
			if ($warning1) {
				print(RESULT "Because expected values of one or more cells are lower than 1, p-value may be unreliable (Cochran, 1954).\n");
			}
			if ($warning2 >= (scalar(@obs) * scalar(@{$obs[0]})) / 5) {
				print(RESULT "Because expected values of over 20 % cells are lower than 5, p-value may be unreliable (Cochran, 1954).\n");
			}
			close(RESULT);
		}
		else {
			&errorMessage(__LINE__, 'Cannot run chi-square test.');
		}
	}
	no Statistics::ChisqIndep;
	unless (chdir('..')) {
		&errorMessage(__LINE__, 'Cannot change working directory.');
	}
	print("done.\n\n");
}

sub setModelParameters {
	print("Counting model parameters...\n");
	foreach my $partname (@allpart) {
		foreach my $model (keys(%{$models{$partname}})) {
			foreach my $heterogeneity (keys(%{$heterogeneities{$partname}})) {
				# the number of branch lengths
				$nparam{$partname}{$model . '_' . $heterogeneity} = ($ntax * 2) - 3;
				# the number of parameters of amino-acid substitution rate matrix
				if ($model eq 'Poisson+F') {
					$nparam{$partname}{$model . '_' . $heterogeneity} += 19;
				}
				elsif ($model eq 'BLOSUM62+F') {
					$nparam{$partname}{$model . '_' . $heterogeneity} += 19;
				}
				elsif ($model eq 'cpREV+F') {
					$nparam{$partname}{$model . '_' . $heterogeneity} += 19;
				}
				elsif ($model eq 'Dayhoff+F') {
					$nparam{$partname}{$model . '_' . $heterogeneity} += 19;
				}
				elsif ($model eq 'JTT+F') {
					$nparam{$partname}{$model . '_' . $heterogeneity} += 19;
				}
				elsif ($model eq 'LG+F') {
					$nparam{$partname}{$model . '_' . $heterogeneity} += 19;
				}
				elsif ($model eq 'mtArt+F') {
					$nparam{$partname}{$model . '_' . $heterogeneity} += 19;
				}
				elsif ($model eq 'mtMam+F') {
					$nparam{$partname}{$model . '_' . $heterogeneity} += 19;
				}
				elsif ($model eq 'mtREV+F') {
					$nparam{$partname}{$model . '_' . $heterogeneity} += 19;
				}
				elsif ($model eq 'mtZoa+F') {
					$nparam{$partname}{$model . '_' . $heterogeneity} += 19;
				}
				elsif ($model eq 'PMB+F') {
					$nparam{$partname}{$model . '_' . $heterogeneity} += 19;
				}
				elsif ($model eq 'rtREV+F') {
					$nparam{$partname}{$model . '_' . $heterogeneity} += 19;
				}
				elsif ($model eq 'HIVb+F') {
					$nparam{$partname}{$model . '_' . $heterogeneity} += 19;
				}
				elsif ($model eq 'HIVw+F') {
					$nparam{$partname}{$model . '_' . $heterogeneity} += 19;
				}
				elsif ($model eq 'VT+F') {
					$nparam{$partname}{$model . '_' . $heterogeneity} += 19;
				}
				elsif ($model eq 'WAG+F') {
					$nparam{$partname}{$model . '_' . $heterogeneity} += 19;
				}
				elsif ($model eq 'FLU+F') {
					$nparam{$partname}{$model . '_' . $heterogeneity} += 19;
				}
				elsif ($model eq 'stmtREV+F') {
					$nparam{$partname}{$model . '_' . $heterogeneity} += 19;
				}
				elsif ($model eq 'LG4M') {
					$nparam{$partname}{$model . '_' . $heterogeneity} += 1;
				}
				elsif ($model eq 'LG4M+F') {
					$nparam{$partname}{$model . '_' . $heterogeneity} += 20;
				}
				elsif ($model eq 'LG4X') {
					$nparam{$partname}{$model . '_' . $heterogeneity} += 6;
				}
				elsif ($model eq 'LG4X+F') {
					$nparam{$partname}{$model . '_' . $heterogeneity} += 25;
				}
				# the number of parameters of site rate heterogeneity
				if ($heterogeneity eq 'Homogeneous') {
					$nparam{$partname}{$model . '_' . $heterogeneity} += 0;
				}
				elsif ($heterogeneity eq 'Gamma') {
					$nparam{$partname}{$model . '_' . $heterogeneity} += 1;
				}
				elsif ($heterogeneity eq 'Invariant') {
					$nparam{$partname}{$model . '_' . $heterogeneity} += 1;
				}
				elsif ($heterogeneity eq 'GammaInvariant') {
					$nparam{$partname}{$model . '_' . $heterogeneity} += 2;
				}
				elsif ($heterogeneity eq 'AGamma' && ($partname ne 'whole' || $partname eq 'whole' && @partname < 2)) {
					$nparam{$partname}{$model . '_' . $heterogeneity} += 2;
				}
				elsif ($heterogeneity eq 'Gene' && $partname eq 'whole' && @partname > 1) {
					$nparam{$partname}{$model . '_' . $heterogeneity} += @partname - 1;
				}
				elsif ($heterogeneity eq 'Gene1Gamma' && $partname eq 'whole' && @partname > 1) {
					$nparam{$partname}{$model . '_' . $heterogeneity} += @partname;
				}
				elsif ($heterogeneity eq 'GeneNGamma' && $partname eq 'whole' && @partname > 1) {
					$nparam{$partname}{$model . '_' . $heterogeneity} += (@partname * 2) - 1;
				}
				elsif ($heterogeneity eq 'Gene1AGamma' && $partname eq 'whole' && @partname > 1) {
					$nparam{$partname}{$model . '_' . $heterogeneity} += @partname + 1;
				}
				elsif ($heterogeneity eq 'GeneNAGamma' && $partname eq 'whole' && @partname > 1) {
					$nparam{$partname}{$model . '_' . $heterogeneity} += @partname * 2;
				}
				else {
					&errorMessage(__LINE__, "Irregular model \"$model" . "_$heterogeneity\" at $partname.");
				}
				if (($model eq 'LG4M' || $model eq 'LG4M+F') && $heterogeneity eq 'Gamma') {
					$nparam{$partname}{$model . '_' . $heterogeneity} -= 1;
				}
			}
		}
	}
#	foreach (keys(%nparam)) {
#		print("$_ : $nparam{$_}\n");
#	}
	print("done.\n\n");
}

sub makeStartingTree {
	if ($treefile) {
		print("Reading tree topology for parameter optimization...\n");
	}
	else {
		print("Generating tree topology for parameter optimization...\n");
	}
	unless (chdir($maindir)) {
		&errorMessage(__LINE__, 'Cannot change working directory.');
	}
	if ($treefile) {
		unless (open(INFILE, "< $treefile")) {
			&errorMessage(__LINE__, "Cannot read \"$treefile\".");
		}
		my $format;
		while (<INFILE>) {
			if (/^#NEXUS/i) {
				$format = 'NEXUS';
			}
			else {
				$format = 'Newick'
			}
			last;
		}
		close(INFILE);
		unless (open(INFILE, "< $treefile")) {
			&errorMessage(__LINE__, "Cannot read \"$treefile\".");
		}
		my @trees;
		if ($format eq 'NEXUS') {
			my $treesblock = 0;
			my $translate = 0;
			my %table;
			while (<INFILE>) {
				s/\[.*\]//g;
				if ($treesblock != 1 && /^\s*Begin\s+Trees\s*;/i) {
					$treesblock = 1;
				}
				elsif ($treesblock == 1 && /^\s*End\s*;/i) {
					last;
				}
				elsif ($treesblock == 1 && $translate == 1) {
					if (/^\s*(\S+)\s+([^,;\s]+)\s*([,;])\s*\r?\n?$/) {
						$table{$1} = $2;
						if ($3 eq ';') {
							$translate = 0;
						}
					}
				}
				elsif ($treesblock == 1 && $translate == 0 && !%table && /^\s*Translate/i) {
					$translate = 1;
				}
				elsif ($treesblock == 1 && $translate == 0) {
					if (/(\(.+\))/) {
						my $tree = $1;
						foreach my $from (keys(%table)) {
							$tree =~ s/([\(,])$from([:,\)])/$1$table{$from}$2/;
						}
						push(@trees, $tree);
					}
				}
			}
		}
		elsif ($format eq 'Newick') {
			local $/ = ';';
			while (<INFILE>) {
				if (/(\(.+\))/) {
					my $tree = $1;
					$tree =~ s/\s//g;
					push(@trees, $tree);
				}
			}
		}
		close(INFILE);
		if (scalar(@trees) == 0) {
			&errorMessage(__LINE__, 'Specified tree file does not contain tree.');
		}
		my @temppartname;
		if ($difftreeeachpart && @partname > 1) {
			if (scalar(@trees) < scalar(@partname)) {
				&errorMessage(__LINE__, 'The number of trees of specified tree file was not enough.');
			}
			@temppartname = ('whole', @partname);
		}
		else {
			push(@temppartname, 'whole');
		}
		for (my $i = 0; $i < scalar(@temppartname); $i ++) {
			my $partname = $temppartname[$i];
			my $tree = $trees[$i];
			if ($tree !~ /:/) {
				$tree =~ s/([\(,][^\(\):,]+)([,\)])/$1:0.1$2/g;
			}
			foreach my $taxon (@taxa) {
				if ($tree !~ /$taxon/) {
					&errorMessage(__LINE__, 'Specified tree file is invalid.');
				}
			}
			unless (open(TREE, "> starttree_$partname.tre")) {
				&errorMessage(__LINE__, "Cannot make \"$maindir/starttree_$partname.tre\".");
			}
			print(TREE "#NEXUS\n\nBegin Trees;\n");
			print(TREE "\tTree tree_$partname = ");
			print(TREE $tree . ";\n");
			print(TREE "End;\n");
			close(TREE);
			for (my $i = 0; $i < @taxa; $i ++) {
				my $from = $taxa[$i];
				my $to = 'Tax' . ($i + 1);
				unless ($tree =~ s/([\(,])$from([:,\)])/$1$to$2/) {
					&errorMessage(__LINE__, 'Specified tree file is invalid.');
				}
			}
			unless (open(TREE, "> starttree_$partname.nwk")) {
				&errorMessage(__LINE__, "Cannot make \"$maindir/starttree_$partname.nwk\".");
			}
			print(TREE $tree . ";\n");
			close(TREE);
		}
	}
	elsif ($difftreeeachpart && @partname > 1) {
		my $child = 0;
		$| = 1;
		$? = 0;
		foreach my $partname ('whole', @partname) {
			if (my $pid = fork()) {
				$child ++;
				if ($child == $numthreads) {
					if (wait == -1) {
						$child = 0;
					}
					else {
						$child --;
					}
				}
				if ($?) {
					&errorMessage(__LINE__, 'Tree building error.');
				}
				next;
			}
			else {
				&starttreecmd($partname);
				exit;
			}
		}
		while (wait != -1) {
			if ($?) {
				&errorMessage(__LINE__, 'Tree building error.');
			}
		}
	}
	else {
		&starttreecmd('whole');
	}
	unless (chdir('..')) {
		&errorMessage(__LINE__, 'Cannot change working directory.');
	}
	print("done.\n\n");
}

sub starttreecmd {
	my $partname = shift;
	{
		# CODEML/TF/RAxML mode
		# make distance matrix by protdist
		unless (mkdir("maketree_$partname")) {
			&errorMessage(__LINE__, "Cannot make \"maketree_$partname\" directory.");
		}
		unless (chdir("maketree_$partname")) {
			&errorMessage(__LINE__, 'Cannot change working directory.');
		}
		unless (open(CMD, "> ../Logs/maketree_$partname.protdist")) {
			&errorMessage(__LINE__, "Cannot make \"$maindir/Logs/maketree_$partname.protdist\".");
		}
		print(CMD "../data_$partname.phy\nI\n2\n");
		if ($algorithm eq 'NJK83') {
			print(CMD "P\nP\nP\nY\n");
		}
		else {
			&errorMessage(__LINE__, 'Cannot recognize algorithm.');
		}
		close(CMD);
		if (system("$protdist < ../Logs/maketree_$partname.protdist > ../Logs/maketree_$partname" . '_protdist.txt')) {
			&errorMessage(__LINE__, "Cannot run $protdist. Please check $protdist.");
		}
		unless (rename('outfile', 'infile')) {
			&errorMessage(__LINE__, "Cannot move \"$maindir/maketree_$partname/outfile\".");
		}
		# make NJ tree by neighbor
		unless (open(CMD, "> ../Logs/maketree_$partname.neighbor")) {
			&errorMessage(__LINE__, "Cannot make \"$maindir/Logs/maketree_$partname.neighbor\".");
		}
		{
			my $seed = substr(time(), -9);
			if ($seed % 2 == 0) {
				$seed ++;
			}
			print(CMD "J\n$seed\n2\n3\nY\n");
		}
		close(CMD);
		if (system("$neighbor < ../Logs/maketree_$partname.neighbor > ../Logs/maketree_$partname" . '_neighbor.txt')) {
			&errorMessage(__LINE__, "Cannot run $neighbor. Please check $neighbor.");
		}
		unless (rename('outfile', "../Logs/maketree_$partname.txt")) {
			&errorMessage(__LINE__, "Cannot move \"$maindir/outfile\".");
		}
		unless (rename('infile', "../Logs/$partname.dist")) {
			&errorMessage(__LINE__, "Cannot move \"$maindir/infile\".");
		}
		unless (open(IN, '< outtree')) {
			&errorMessage(__LINE__, "Cannot open \"$maindir/outtree\".");
		}
		unless (open(OUT, "> ../starttree_$partname.nwk")) {
			&errorMessage(__LINE__, "Cannot make \"$maindir/starttree_$partname.nwk\".");
		}
		while (<IN>) {
			s/-\d+(?:\.\d+)?/0/g;
			s/(.+)\r?\n?/$1/;
			s/;$/;\n/;
			print(OUT);
		}
		close(OUT);
		close(IN);
		unlink('outtree');
		unless (chdir('..')) {
			&errorMessage(__LINE__, 'Cannot change working directory.');
		}
		unless (rmdir("maketree_$partname")) {
			&errorMessage(__LINE__, 'Cannot remove temporary directory.');
		}
		unless (open(IN, "< starttree_$partname.nwk")) {
			&errorMessage(__LINE__, "Cannot open \"$maindir/starttree_$partname.nwk\".");
		}
		unless (open(OUT, "> starttree_$partname.tre")) {
			&errorMessage(__LINE__, "Cannot make \"$maindir/starttree_$partname.tre\".");
		}
		print(OUT "#NEXUS\n\nBegin Trees;\n\tTree tree_$partname = ");
		while (<IN>) {
			for (my $i = 0; $i < @taxa; $i ++) {
				my $from = 'Tax' . ($i + 1);
				my $to = $taxa[$i];
				unless (s/([\(,])$from([:,\)])/$1$to$2/) {
					&errorMessage(__LINE__, 'Specified tree file is invalid.');
				}
			}
			print(OUT);
			last;
		}
		print(OUT "End;\n");
		close(OUT);
		close(IN);
	}
}

sub setSampleSizes {
	print("Counting sample sizes...\n");
	unless (chdir($maindir)) {
		&errorMessage(__LINE__, 'Cannot change working directory.');
	}
	# sample size 0 : pscore
	# sample size 1 : sum of (the number of character states - 1)
	# sample size 2 : sum of the number of character states
	# sample size 3 : the number of sites = alignment length
	# sample size 4 : the number of variable characters
	# sample size 5 : (sample size 3) x the number of OTUs
	my $child = 0;
	$| = 1;
	$? = 0;
	foreach my $partname (@allpart) {
		if (my $pid = fork()) {
			$child ++;
			if ($child == $numthreads) {
				if (wait == -1) {
					$child = 0;
				}
				else {
					$child --;
				}
			}
			if ($?) {
				&errorMessage(__LINE__, 'Sample size counting error.');
			}
			next;
		}
		else {
			&calcpscore($partname);
			exit;
		}
	}
	while (wait != -1) {
		if ($?) {
			&errorMessage(__LINE__, 'Sample size counting error.');
		}
	}
#	foreach my $partname (@allpart) {
#		&calcpscore($partname);
#	}
	foreach my $partname (@allpart) {
		unless (open(SCORE, "< Logs/pscore_$partname.txt")) {
			&errorMessage(__LINE__, "Cannot open \"$maindir/Logs/pscore_$partname.txt\".");
		}
		while (<SCORE>) {
			# CODEML/TF/RAxML mode
			if (/^requires a total of\s*(\d+)/) {
				$samplesize{$partname}[0] = $1;
				if ($1 == 0) {
					delete($models{$partname});
					delete($heterogeneities{$partname});
					if ($lnlcalc eq 'raxml') {
						$models{$partname}{'BLOSUM62'} = 1;
						$heterogeneities{$partname}{'Gamma'} = 1;
					}
					else {
						$models{$partname}{'Poisson'} = 1;
						$heterogeneities{$partname}{'Homogeneous'} = 1;
					}
				}
				last;
			}
		}
		close(SCORE);
	}
#	my @numstate;
#	for (my $i = 0; $i < $nchar; $i ++) {
#		my %state;
#		foreach my $taxon (@taxa) {
#			if ($seqs{$taxon}[$i] =~ /^[ARNDCQEGHILKMFPSTWYVBZJX]$/i) {
#				$state{$seqs{$taxon}[$i]} = 1;
#			}
#			elsif ($seqs{$taxon}[$i] =~ /^-$/) {
#				$state{'X'} = 1;
#			}
#			elsif ($seqs{$taxon}[$i] =~ /^\?$/) {
#				$state{'X'} = 1;
#			}
#		}
#		if (exists($state{'B'}) && (exists($state{'N'}) || exists($state{'D'}))) {
#			delete($state{'B'});
#		}
#		if (exists($state{'Z'}) && (exists($state{'Q'}) || exists($state{'E'}))) {
#			delete($state{'Z'});
#		}
#		if (exists($state{'J'}) && (exists($state{'I'}) || exists($state{'L'}))) {
#			delete($state{'J'});
#		}
#		
#		elsif (exists($state{'A'}) || exists($state{'C'}) || exists($state{'G'}) || exists($state{'T'}) || exists($state{'N'})) {
#			$numstate[$i] = 1;
#		}
#		$samplesize{'whole'}[2] += $numstate[$i];
#		unless ($numstate[$i]) {
#			&errorMessage(__LINE__, 'The number of state of ' . ($i + 1) . 'th column is unable to count.');
#		}
#		elsif ($numstate[$i] > 1) {
#			$samplesize{'whole'}[4] ++;
#		}
#	}
	undef(%seqs);
	$samplesize{'whole'}[1] = 0;
	$samplesize{'whole'}[2] = 0;
	$samplesize{'whole'}[3] = $nchar;
#	$samplesize{'whole'}[1] = $samplesize{'whole'}[2] - $samplesize{'whole'}[3];
	$samplesize{'whole'}[5] = $nchar * $ntax;
	unless (defined($samplesize{'whole'}[4])) {
		$samplesize{'whole'}[4] = 0;
	}
	if (@partname > 1) {
		foreach my $partname (@partname) {
#			foreach my $site (@{$part2site{$partname}}) {
#				$samplesize{$partname}[2] += $numstate[$site - 1];
#				if ($numstate[$site - 1] > 1) {
#					$samplesize{$partname}[4] ++;
#				}
#			}
			$samplesize{$partname}[1] = 0;
			$samplesize{$partname}[2] = 0;
			$samplesize{$partname}[3] = scalar(@{$part2site{$partname}});
#			$samplesize{$partname}[1] = $samplesize{$partname}[2] - $samplesize{$partname}[3];
			$samplesize{$partname}[5] = scalar(@{$part2site{$partname}}) * $ntax;
			unless (defined($samplesize{$partname}[4])) {
				$samplesize{$partname}[4] = 0;
			}
		}
	}
	foreach my $partname (@allpart) {
#		print("$partname : @{$samplesize{$partname}}\n");
		foreach (0 .. 5) {
			unless (defined($samplesize{$partname}[$_])) {
				&errorMessage(__LINE__, 'Sample size counting error.');
			}
		}
	}
	unless (chdir('..')) {
		&errorMessage(__LINE__, 'Cannot change working directory.');
	}
	print("done.\n\n");
}

sub calcpscore {
	my $partname = shift;
	{
		# CODEML/TF/RAxML mode
		if ($difftreeeachpart) {
			$treefile = "starttree_$partname.nwk";
		}
		else {
			$treefile = "starttree_whole.nwk";
		}
		unless (open(OUTTREE, "> temptree_$partname.nwk")) {
			&errorMessage(__LINE__, "Cannot make \"$maindir/temptree_$partname.nwk\".");
		}
		unless (open(INTREE, "< $treefile")) {
			&errorMessage(__LINE__, "Cannot read \"$maindir/$treefile\".");
		}
		while (<INTREE>) {
			s/^(\()(.+)(,[^,]+\);)/$1\($2\)$3/;
			print(OUTTREE $_);
		}
		close(INTREE);
		close(OUTTREE);
		unless (open(CMD, "> Logs/pscore_$partname.protpars")) {
			&errorMessage(__LINE__, "Cannot make \"$maindir/Logs/pscore_$partname.protpars\".");
		}
		print(CMD "../data_$partname.phy\nU\nI\n2\n6\nY\n../temptree_$partname.nwk\n");
		close(CMD);
		unless (mkdir("pscore_$partname")) {
			&errorMessage(__LINE__, "Cannot make \"pscore_$partname\" directory.");
		}
		unless (chdir("pscore_$partname")) {
			&errorMessage(__LINE__, 'Cannot change working directory.');
		}
		if (system("$protpars < ../Logs/pscore_$partname.protpars > ../Logs/pscore_$partname" . '_protpars.txt')) {
			&errorMessage(__LINE__, "Cannot run $protpars. Please check $protpars.");
		}
		unless (rename('outfile', "../Logs/pscore_$partname.txt")) {
			&errorMessage(__LINE__, "Cannot move \"$maindir/pscore_$partname/outfile\".");
		}
		unless (chdir('..')) {
			&errorMessage(__LINE__, 'Cannot change working directory.');
		}
		unless (rmdir("pscore_$partname")) {
			&errorMessage(__LINE__, 'Cannot remove temporary directory.');
		}
		unlink("temptree_$partname.nwk");
	}
}

sub analyzeEachPartition {
	print("Analyzing each partition under the models...\n");
	unless (chdir($maindir)) {
		&errorMessage(__LINE__, 'Cannot change working directory.');
	}
	unless (mkdir('Scores')) {
		&errorMessage(__LINE__, "Cannot make \"$maindir/Scores\" directory.");
	}
	my $child = 0;
	$| = 1;
	$? = 0;
	foreach my $partname (@allpart) {
		foreach my $model (keys(%{$models{$partname}})) {
			foreach my $heterogeneity (keys(%{$heterogeneities{$partname}})) {
				if ($partname ne 'whole' || $calcwhole) {
					if (my $pid = fork()) {
						$child ++;
						if ($child == $numthreads) {
							if (wait == -1) {
								$child = 0;
							}
							else {
								$child --;
							}
						}
						if ($?) {
							&errorMessage(__LINE__, 'Likelihood calculation error.');
						}
						next;
					}
					else {
						&calcL($partname, $model, $heterogeneity);
						exit;
					}
				}
			}
		}
	}
	while (wait != -1) {
		if ($?) {
			&errorMessage(__LINE__, 'Likelihood calculation error.');
		}
	}
	unless (chdir('..')) {
		&errorMessage(__LINE__, 'Cannot change working directory.');
	}
	print("done.\n\n");
}

sub calcL {
	my ($partname, $model, $heterogeneity) = @_;
	print("Calculating likelihood under $model" . '_' . "$heterogeneity model on $partname partition...\n");
	if ($lnlcalc eq 'tf') {
		# TF mode
		unless (open(CMD, '> Logs/' . $partname . '_' . $model . '_' . $heterogeneity . '.tl')) {
			&errorMessage(__LINE__, "Cannot make \"$maindir/Logs/$partname" . '_' . $model . '_' . "$heterogeneity.tl\".");
		}
		my $tempmodel = &makeTFModel($model . '_' . $heterogeneity);
		print(CMD "report:=ReconstructPhylogeny[\n\t\"data_$partname.phy\",\n\tSubstitutionModel->$tempmodel,\n");
		print(CMD "\tTree->\"starttree_");
		if ($difftreeeachpart) {
			print(CMD $partname);
		}
		else {
			print(CMD 'whole');
		}
		print(CMD ".nwk\",\n");
		print(CMD "\tWithEdgeSupport->False,\n\tAcceptFlatness->True,\n\tVerbose->False\n],\nOprec[\n\t20,\n\tSave[\n\t\treport|1|Likelihood,\n\t\t\"Scores/$partname\_$model\_$heterogeneity.txt\"\n\t]\n]\n");
		close(CMD);
		if (system("$tf Logs/" . $partname . '_' . $model . '_' . $heterogeneity . '.tl')) {
			&errorMessage(__LINE__, "Cannot run $tf. Please check $tf.");
		}
	}
	elsif ($lnlcalc eq 'codeml') {
		# CODEML mode
		unless (open(CMD, '> Logs/' . $partname . '_' . $model . '_' . $heterogeneity . '.paml')) {
			&errorMessage(__LINE__, "Cannot make \"$maindir/Logs/$partname" . '_' . $model . '_' . "$heterogeneity.paml\".");
		}
		if ($heterogeneity =~ /^Gene/) {
			print(CMD "seqfile = ../data_whole_gene.phy\n");
		}
		else {
			print(CMD "seqfile = ../data_$partname.phy\n");
		}
		print(CMD "treefile = ../starttree_");
		if ($difftreeeachpart) {
			print(CMD $partname);
		}
		else {
			print(CMD 'whole');
		}
		print(CMD ".nwk\n");
		print(CMD "outfile = ../Scores/" . $partname . '_' . $model . '_' . $heterogeneity . ".txt\n");
		print(CMD "noisy = 0\n");
		print(CMD "verbose = 0\n");
		print(CMD "Mgene = 0\n");
		print(CMD "clock = 0\n");
		print(CMD "getSE = 0\n");
		print(CMD "RateAncestor = 0\n");
		if ($heterogeneity =~ /AGamma/) {
			print(CMD "method = 0\n");
		}
		else {
			print(CMD "method = 1\n");
		}
		print(CMD "Small_Diff = 1e-6\n");
		print(CMD "fix_blength = 0\n");
		if ($model eq 'Poisson') {
			print(CMD "model = 0\n");
		}
		elsif ($model eq 'Poisson+F') {
			print(CMD "model = 1\n");
		}
		elsif ($model =~ /^(.+)\+F$/) {
			print(CMD "model = 3\n");
			print(CMD "aaRatefile = $dat{$1}\n");
		}
		else {
			print(CMD "model = 2\n");
			print(CMD "aaRatefile = $dat{$model}\n");
		}
		if ($heterogeneity eq 'Homogeneous' || $heterogeneity eq 'Gene') {
			print(CMD "fix_alpha = 1\nalpha = 0\nMalpha = 0\nfix_rho = 1\nrho = 0\n");
		}
		elsif ($heterogeneity eq 'Gamma' || $heterogeneity eq 'Gene1Gamma') {
			print(CMD "fix_alpha = 0\nalpha = 0\nMalpha = 0\nfix_rho = 1\nrho = 0\nncatG = $ngammacat\n");
		}
		elsif ($heterogeneity eq 'AGamma' || $heterogeneity eq 'Gene1AGamma') {
			print(CMD "fix_alpha = 0\nalpha = 0\nMalpha = 0\nfix_rho = 0\nrho = 0\nncatG = $ngammacat\n");
		}
		elsif ($heterogeneity eq 'GeneNGamma') {
			print(CMD "fix_alpha = 0\nalpha = 0\nMalpha = 1\nfix_rho = 1\nrho = 0\nncatG = $ngammacat\n");
		}
		elsif ($heterogeneity eq 'GeneNAGamma') {
			print(CMD "fix_alpha = 0\nalpha = 0\nMalpha = 1\nfix_rho = 0\nrho = 0\nncatG = $ngammacat\n");
		}
		close(CMD);
		unless (mkdir($partname . '_' . $model . '_' . $heterogeneity)) {
			&errorMessage(__LINE__, "Cannot make \"$partname" . '_' . $model . '_' . "$heterogeneity\" directory.");
		}
		unless (chdir($partname . '_' . $model . '_' . $heterogeneity)) {
			&errorMessage(__LINE__, 'Cannot change working directory.');
		}
		if (system("$codeml ../Logs/" . $partname . '_' . $model . '_' . $heterogeneity . '.paml > ../Logs/' . $partname . '_' . $model . '_' . $heterogeneity . '.txt')) {
			&errorMessage(__LINE__, "Cannot run $codeml. Please check $codeml.");
		}
		unlink('lnf', 'rates', 'rst', 'rst1', 'rub');
		unless (chdir('..')) {
			&errorMessage(__LINE__, 'Cannot change working directory.');
		}
		unless (rmdir($partname . '_' . $model . '_' . $heterogeneity)) {
			&errorMessage(__LINE__, 'Cannot remove temporary directory.');
		}
	}
	elsif ($lnlcalc eq 'raxml') {
		# RAxML mode
		my $treefile = "starttree_";
		if ($difftreeeachpart) {
			if ($partname =~ /^(.+_P)[1-3]$/i) {
				$treefile .= $1;
			}
			else {
				$treefile .= $partname;
			}
		}
		else {
			$treefile .= 'whole';
		}
		$treefile .= ".nwk";
		my $modelname = &makeRAxMLModel($model, $heterogeneity);
		if (system("$raxml --no-seq-check --no-bfgs -m PROTGAMMA$modelname -f e -n " . $partname . '_' . $model . '_' . $heterogeneity . " -t $treefile -s data_$partname.phy > Logs/" . $partname . '_' . $model . '_' . $heterogeneity . ".txt")) {
			&errorMessage(__LINE__, "Cannot run $raxml. Please check $raxml.");
		}
		unless (rename("RAxML_info." . $partname . '_' . $model . '_' . $heterogeneity, "Scores/" . $partname . '_' . $model . '_' . $heterogeneity . ".txt")) {
			&errorMessage(__LINE__, "Cannot move \"$maindir/RAxML_info." . $partname . '_' . $model . '_' . $heterogeneity . "\".");
		}
		foreach my $file (glob("RAxML_*." . $partname . '_' . $model . '_' . $heterogeneity)) {
			unlink($file);
		}
	}
}

sub calculateCriteria {
	print("Calculating AIC, AICc, and BIC...\n");
	unless (chdir($maindir)) {
		&errorMessage(__LINE__, 'Cannot change working directory.');
	}
	foreach my $partname (@allpart) {
		foreach my $model (keys(%{$models{$partname}})) {
			foreach my $heterogeneity (keys(%{$heterogeneities{$partname}})) {
				if (open(LNL, '< Scores/' . $partname . '_' . $model . '_' . $heterogeneity . '.txt')) {
					while (<LNL>) {
						if (($lnlcalc eq 'codeml' && /^lnL\([^\(\)]+\):.*?-(\d+(?:\.\d+)?)/) || ($lnlcalc eq 'tf' && /-(\d+(?:\.\d+)?)/) || ($lnlcalc eq 'raxml' && /^Final +GAMMA +likelihood: +-(\d+(?:\.\d+)?)/)) {
							$LnL{$partname}{$model . '_' . $heterogeneity} = $1;
							last;
						}
					}
					close(LNL);
					$criteria{'AIC'}{$partname}{$model . '_' . $heterogeneity} = &calcAIC($LnL{$partname}{$model . '_' . $heterogeneity}, $nparam{$partname}{$model . '_' . $heterogeneity});
					if ($samplesize{$partname}[0] - $nparam{$partname}{$model . '_' . $heterogeneity} - 1 > 0) {
						$criteria{'AICc1'}{$partname}{$model . '_' . $heterogeneity} = &calcAICc($LnL{$partname}{$model . '_' . $heterogeneity}, $nparam{$partname}{$model . '_' . $heterogeneity}, $samplesize{$partname}[0]);
					}
					#if ($samplesize{$partname}[1] - $nparam{$partname}{$model . '_' . $heterogeneity} - 1 > 0) {
					#	$criteria{'AICc2'}{$partname}{$model . '_' . $heterogeneity} = &calcAICc($LnL{$partname}{$model . '_' . $heterogeneity}, $nparam{$partname}{$model . '_' . $heterogeneity}, $samplesize{$partname}[1]);
					#}
					#if ($samplesize{$partname}[2] - $nparam{$partname}{$model . '_' . $heterogeneity} - 1 > 0) {
					#	$criteria{'AICc3'}{$partname}{$model . '_' . $heterogeneity} = &calcAICc($LnL{$partname}{$model . '_' . $heterogeneity}, $nparam{$partname}{$model . '_' . $heterogeneity}, $samplesize{$partname}[2]);
					#}
					if ($samplesize{$partname}[3] - $nparam{$partname}{$model . '_' . $heterogeneity} - 1 > 0) {
						$criteria{'AICc4'}{$partname}{$model . '_' . $heterogeneity} = &calcAICc($LnL{$partname}{$model . '_' . $heterogeneity}, $nparam{$partname}{$model . '_' . $heterogeneity}, $samplesize{$partname}[3]);
					}
					#if ($samplesize{$partname}[4] - $nparam{$partname}{$model . '_' . $heterogeneity} - 1 > 0) {
					#	$criteria{'AICc5'}{$partname}{$model . '_' . $heterogeneity} = &calcAICc($LnL{$partname}{$model . '_' . $heterogeneity}, $nparam{$partname}{$model . '_' . $heterogeneity}, $samplesize{$partname}[4]);
					#}
					if ($samplesize{$partname}[5] - $nparam{$partname}{$model . '_' . $heterogeneity} - 1 > 0) {
						$criteria{'AICc6'}{$partname}{$model . '_' . $heterogeneity} = &calcAICc($LnL{$partname}{$model . '_' . $heterogeneity}, $nparam{$partname}{$model . '_' . $heterogeneity}, $samplesize{$partname}[5]);
					}
					if (defined($samplesize{$partname}[0])) {
						$criteria{'BIC1'}{$partname}{$model . '_' . $heterogeneity} = &calcBIC($LnL{$partname}{$model . '_' . $heterogeneity}, $nparam{$partname}{$model . '_' . $heterogeneity}, $samplesize{$partname}[0]);
					}
					#if (defined($samplesize{$partname}[1])) {
					#	$criteria{'BIC2'}{$partname}{$model . '_' . $heterogeneity} = &calcBIC($LnL{$partname}{$model . '_' . $heterogeneity}, $nparam{$partname}{$model . '_' . $heterogeneity}, $samplesize{$partname}[1]);
					#}
					#if (defined($samplesize{$partname}[2])) {
					#	$criteria{'BIC3'}{$partname}{$model . '_' . $heterogeneity} = &calcBIC($LnL{$partname}{$model . '_' . $heterogeneity}, $nparam{$partname}{$model . '_' . $heterogeneity}, $samplesize{$partname}[2]);
					#}
					if (defined($samplesize{$partname}[3])) {
						$criteria{'BIC4'}{$partname}{$model . '_' . $heterogeneity} = &calcBIC($LnL{$partname}{$model . '_' . $heterogeneity}, $nparam{$partname}{$model . '_' . $heterogeneity}, $samplesize{$partname}[3]);
					}
					#if (defined($samplesize{$partname}[4])) {
					#	$criteria{'BIC5'}{$partname}{$model . '_' . $heterogeneity} = &calcBIC($LnL{$partname}{$model . '_' . $heterogeneity}, $nparam{$partname}{$model . '_' . $heterogeneity}, $samplesize{$partname}[4]);
					#}
					if (defined($samplesize{$partname}[5])) {
						$criteria{'BIC6'}{$partname}{$model . '_' . $heterogeneity} = &calcBIC($LnL{$partname}{$model . '_' . $heterogeneity}, $nparam{$partname}{$model . '_' . $heterogeneity}, $samplesize{$partname}[5]);
					}
				}
			}
		}
	}
	&calcweight();
	unless (chdir('..')) {
		&errorMessage(__LINE__, 'Cannot change working directory.');
	}
	print("done.\n\n");
}

sub calcAIC {
	my $AIC = 2 * $_[0] + 2 * $_[1];
	return($AIC);
}

sub calcAICc {
	my $AICc;
	$AICc = 2 * $_[0] + 2 * $_[1] * $_[2] / ($_[2] - $_[1] - 1);
	return($AICc);
}

sub calcBIC {
	my $BIC;
	if ($_[2] > 0) {
		$BIC = 2 * $_[0] + $_[1] * log($_[2]);
	}
	else {
		$BIC = 0;
	}
	return($BIC);
}

sub calcweight {
	foreach my $partname (@allpart) {
		foreach my $criterion (sort(keys(%criteria))) {
			my $num = 0;
			my $minvalue;
			my $sum = 0;
			foreach my $model (sort({$criteria{$criterion}{$partname}{$a} <=> $criteria{$criterion}{$partname}{$b}} keys(%{$criteria{$criterion}{$partname}}))) {
				if ($num == 0) {
					$minvalue = $criteria{$criterion}{$partname}{$model};
					$sum = 1;
				}
				else {
					$sum += exp(($criteria{$criterion}{$partname}{$model} - $minvalue) / (-2));
				}
				$num ++;
			}
			foreach my $model (keys(%{$criteria{$criterion}{$partname}})) {
				$weight{$criterion}{$partname}{$model} = exp(($criteria{$criterion}{$partname}{$model} - $minvalue) / (-2)) / $sum;
			}
		}
	}
}

sub outputFiles {
	print("Outputting results...\n");
	unless (chdir($maindir)) {
		&errorMessage(__LINE__, 'Cannot change working directory.');
	}
	# output results
	&outputResults();
	# output for MrBayes
	if ($output{'mrbayes'}) {
		&outforMrBayes();
	}
	# output for Treefinder
	if ($output{'treefinder'} || ($output{'mrbayes'} && $comparemix)) {
		&outforTreefinder();
	}
	# output for PHYML
	if ($output{'phyml'}) {
		&outforPHYML();
	}
	# output for RAxML
	if ($output{'raxml'}) {
		&outforRAxML();
	}
	unless (chdir('..')) {
		&errorMessage(__LINE__, 'Cannot change working directory.');
	}
	print("done.\n\n");
}

sub outputResults {
	unless (mkdir('Results')) {
		&errorMessage(__LINE__, "Cannot make \"$maindir/Results\" directory.");
	}
	foreach my $partname (@allpart) {
		foreach my $criterion (sort(keys(%criteria))) {
			if (scalar(keys(%{$criteria{$criterion}{$partname}}))) {
				unless (open(OUT, "> Results/$partname" . "_$criterion.txt")) {
					&errorMessage(__LINE__, "Cannot make \"$maindir/Resilts/$partname" . '_' . "$criterion.txt\".");
				}
				print(OUT 'model                     ' . "\t");
				printf(OUT "% 19s \t", $criterion);
				print(OUT 'weight ' . "\t");
				print(OUT '               -LnL ' . "\t" . 'nparam');
				if ($criterion =~ /\d$/) {
					print(OUT "\t" . 'sample size' );
				}
				print(OUT "\n");
				foreach my $model (sort({$criteria{$criterion}{$partname}{$a} <=> $criteria{$criterion}{$partname}{$b}} keys(%{$criteria{$criterion}{$partname}}))) {
					printf(OUT "%-26s\t", $model);
					printf(OUT "% 20.12e\t", $criteria{$criterion}{$partname}{$model});
					printf(OUT "%.5f\t", $weight{$criterion}{$partname}{$model});
					printf(OUT "% 20.12e\t", $LnL{$partname}{$model});
					printf(OUT "% 6d", $nparam{$partname}{$model});
					if ($criterion =~ /(\d)$/) {
						printf(OUT "\t% 11d", $samplesize{$partname}[$1 - 1]);
					}
					print(OUT "\n");
				}
				close(OUT);
			}
		}
	}
}

sub outforMrBayes {
	unless (mkdir('MrBayes')) {
		&errorMessage(__LINE__, "Cannot make \"$maindir/MrBayes\" directory.");
	}
	my @temppartname;
	if (@partname > 1) {
		@temppartname = @partname;
	}
	foreach my $criterion (sort(keys(%criteria))) {
		foreach my $partname ('whole', @temppartname) {
			if ($partname eq 'whole' || scalar(keys(%{$criteria{$criterion}{$partname}}))) {
				&makeMrBayesNexus($criterion, $partname);
			}
		}
	}
}

sub makeMrBayesNexus {
	my ($criterion, $partname) = @_;
	my $outfile;
	my $partcriterion = 1;
	if ($partname eq 'whole') {
		foreach my $temppartname (@partname) {
			if (!scalar(keys(%{$criteria{$criterion}{$temppartname}}))) {
				$partcriterion = 0;
			}
		}
	}
	# nonpartitioned model
	if (scalar(keys(%{$criteria{$criterion}{$partname}}))) {
		$outfile = $partname . '_' . $criterion . '_nonpartitioned.nex';
	}
	if ($outfile) {
		open(OUT, "> MrBayes/$outfile");
		print(OUT &dataNexus($partname));
		print(OUT "\nBegin MrBayes;\n");
		print(OUT &makeMrBayesCommand($criterion, 0, $partname));
		print(OUT "End;\n");
		close(OUT);
		undef($outfile);
	}
	# proportional model
	if ($partname eq 'whole' && @partname > 1 && $partcriterion == 1) {
		$outfile = $partname . '_' . $criterion . '_proportional.nex';
		open(OUT, "> MrBayes/$outfile");
		print(OUT &dataNexus($partname));
		print(OUT "\nBegin MrBayes;\n");
		my $partno = 0;
		my @temppartname;
		foreach my $temppartname (@partname) {
			$partno ++;
			push(@temppartname, $temppartname);
			print(OUT "\tCharSet $temppartname=$part2site{$temppartname}[0]-$part2site{$temppartname}[-1];\n");
		}
		print(OUT "\tPartition Aminosan=" . $partno . ':' . join(',', @temppartname) . ';' . "\n");
		print(OUT "\tSet Partition=Aminosan;\n");
		print(OUT "\tUnlink StateFreq=(all) Shape=(all) PInvar=(all) Correlation=(all);\n");
		print(OUT "\tPrSet ApplyTo=(all) RatePr=Variable;\n");
		print(OUT &makeMrBayesCommand($criterion, 1, @partname));
		print(OUT "End;\n");
		close(OUT);
		undef($outfile);
	}
	# equalrate model
	if ($partname eq 'whole' && @partname > 1 && $partcriterion == 1) {
		$outfile = $partname . '_' . $criterion . '_equalrate.nex';
		open(OUT, "> MrBayes/$outfile");
		print(OUT &dataNexus($partname));
		print(OUT "\nBegin MrBayes;\n");
		my $partno = 0;
		my @temppartname;
		foreach my $temppartname (@partname) {
			$partno ++;
			push(@temppartname, $temppartname);
			print(OUT "\tCharSet $temppartname=$part2site{$temppartname}[0]-$part2site{$temppartname}[-1];\n");
		}
		print(OUT "\tPartition Aminosan=" . $partno . ':' . join(',', @temppartname) . ';' . "\n");
		print(OUT "\tSet Partition=Aminosan;\n");
		print(OUT "\tUnlink StateFreq=(all) Shape=(all) PInvar=(all) Correlation=(all);\n");
		print(OUT "\tPrSet ApplyTo=(all) RatePr=Fixed;\n");
		print(OUT &makeMrBayesCommand($criterion, 1, @partname));
		print(OUT "End;\n");
		close(OUT);
		undef($outfile);
	}
	# separate model
	if ($partname eq 'whole' && @partname > 1 && $partcriterion == 1) {
		$outfile = $partname . '_' . $criterion . '_separate.nex';
		open(OUT, "> MrBayes/$outfile");
		print(OUT &dataNexus($partname));
		print(OUT "\nBegin MrBayes;\n");
		my $partno = 0;
		my @temppartname;
		foreach my $temppartname (@partname) {
			$partno ++;
			push(@temppartname, $temppartname);
			print(OUT "\tCharSet $temppartname=$part2site{$temppartname}[0]-$part2site{$temppartname}[-1];\n");
		}
		print(OUT "\tPartition Aminosan=" . $partno . ':' . join(',', @temppartname) . ';' . "\n");
		print(OUT "\tSet Partition=Aminosan;\n");
		print(OUT "\tUnlink StateFreq=(all) Shape=(all) PInvar=(all) Correlation=(all) BrLens=(all);\n");
		print(OUT &makeMrBayesCommand($criterion, 1, @partname));
		print(OUT "End;\n");
		close(OUT);
		undef($outfile);
	}
}

sub makeMrBayesCommand {
	my %modelMrBayes;
	foreach my $model (@modelformrbayes) {
		foreach my $heterogeneity (@heterogeneityformrbayes) {
			$modelMrBayes{$model . '_' . $heterogeneity} = 1;
		}
	}
	my $partno;
	my $criterion = shift(@_);
	my $repeat = shift(@_);
	my $cmd;
	if (@_ > 1) {
		$partno = 1;
	}
	foreach my $partname (@_) {
		$cmd .= '[' . $partname . ']' . "\n";
		my $numpart;
		my $notbest;
		foreach my $model (sort({$criteria{$criterion}{$partname}{$a} <=> $criteria{$criterion}{$partname}{$b}} keys(%{$criteria{$criterion}{$partname}}))) {
			if ($partname eq 'whole' && $model =~ /Gene/) {
				$numpart = @partname;
			}
			else {
				$numpart = 1;
			}
			$cmd .= '[ ' . sprintf("%-26s ", $model);
			$cmd .= sprintf("% 12.5e", $criteria{$criterion}{$partname}{$model});
			$cmd .= ' ]';
			if ($modelMrBayes{$model}) {
				unless ($notbest) {
					$cmd .= '  ';
				}
				else {
					$cmd .= '[ ';
				}
				$cmd .= &makeMrBayesLSet($model, $partno, $numpart, $partname);
				if ($repeat) {
					$cmd .= "\n";
					last;
				}
				unless ($notbest) {
					$notbest = 1;
				}
				else {
					$cmd .=' ]';
				}
			}
			$cmd .= "\n";
		}
		if ($partno) {
			$partno += $numpart;
		}
	}
	return($cmd);
}

sub makeMrBayesLSet {
	my ($model, $heterogeneity) = split(/_/, $_[0]);
	my $partno = $_[1];
	my $numpart = $_[2];
	my $partname = $_[3];
	my $mblset;
	my $mbprset;
	my $unlinkshape;
	if ($model eq 'Poisson') {
		$mbprset .= 'AAModelPr=Fixed(Poisson)';
	}
	elsif ($model eq 'Poisson+F') {
		$mbprset .= 'AAModelPr=Fixed(EqualIn)';
		if ($aafreq eq 'Estimate') {
			$mbprset .= ' StateFreqPr=Dirichlet(1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1)';
		}
		elsif ($aafreq eq 'Empirical') {
			$mbprset .= ' StateFreqPr=Fixed(Empirical)';
		}
	}
	elsif ($model eq 'BLOSUM62') {
		$mbprset .= 'AAModelPr=Fixed(BLOSUM)';
	}
	elsif ($model eq 'BLOSUM62+F') {
		$mbprset .= 'AAModelPr=Fixed(BLOSUMF)';
		if ($aafreq eq 'Estimate') {
			$mbprset .= ' StateFreqPr=Dirichlet(1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1)';
		}
		elsif ($aafreq eq 'Empirical') {
			$mbprset .= ' StateFreqPr=Fixed(Empirical)';
		}
	}
	elsif ($model eq 'cpREV') {
		$mbprset .= 'AAModelPr=Fixed(cpREV)';
	}
	elsif ($model eq 'cpREV+F') {
		$mbprset .= 'AAModelPr=Fixed(cpREVF)';
		if ($aafreq eq 'Estimate') {
			$mbprset .= ' StateFreqPr=Dirichlet(1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1)';
		}
		elsif ($aafreq eq 'Empirical') {
			$mbprset .= ' StateFreqPr=Fixed(Empirical)';
		}
	}
	elsif ($model eq 'Dayhoff') {
		$mbprset .= 'AAModelPr=Fixed(Dayhoff)';
	}
	elsif ($model eq 'Dayhoff+F') {
		$mbprset .= 'AAModelPr=Fixed(DayhoffF)';
		if ($aafreq eq 'Estimate') {
			$mbprset .= ' StateFreqPr=Dirichlet(1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1)';
		}
		elsif ($aafreq eq 'Empirical') {
			$mbprset .= ' StateFreqPr=Fixed(Empirical)';
		}
	}
	elsif ($model eq 'JTT') {
		$mbprset .= 'AAModelPr=Fixed(Jones)';
	}
	elsif ($model eq 'JTT+F') {
		$mbprset .= 'AAModelPr=Fixed(JonesF)';
		if ($aafreq eq 'Estimate') {
			$mbprset .= ' StateFreqPr=Dirichlet(1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1)';
		}
		elsif ($aafreq eq 'Empirical') {
			$mbprset .= ' StateFreqPr=Fixed(Empirical)';
		}
	}
	elsif ($model eq 'LG') {
		$mbprset .= 'AAModelPr=Fixed(LG)';
	}
	elsif ($model eq 'LG+F') {
		$mbprset .= 'AAModelPr=Fixed(LGF)';
		if ($aafreq eq 'Estimate') {
			$mbprset .= ' StateFreqPr=Dirichlet(1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1)';
		}
		elsif ($aafreq eq 'Empirical') {
			$mbprset .= ' StateFreqPr=Fixed(Empirical)';
		}
	}
	elsif ($model eq 'mtArt') {
		$mbprset .= 'AAModelPr=Fixed(mtArt)';
	}
	elsif ($model eq 'mtArt+F') {
		$mbprset .= 'AAModelPr=Fixed(mtArtF)';
		if ($aafreq eq 'Estimate') {
			$mbprset .= ' StateFreqPr=Dirichlet(1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1)';
		}
		elsif ($aafreq eq 'Empirical') {
			$mbprset .= ' StateFreqPr=Fixed(Empirical)';
		}
	}
	elsif ($model eq 'mtMam') {
		$mbprset .= 'AAModelPr=Fixed(mtMam)';
	}
	elsif ($model eq 'mtMam+F') {
		$mbprset .= 'AAModelPr=Fixed(mtMamF)';
		if ($aafreq eq 'Estimate') {
			$mbprset .= ' StateFreqPr=Dirichlet(1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1)';
		}
		elsif ($aafreq eq 'Empirical') {
			$mbprset .= ' StateFreqPr=Fixed(Empirical)';
		}
	}
	elsif ($model eq 'mtREV') {
		$mbprset .= 'AAModelPr=Fixed(mtREV)';
	}
	elsif ($model eq 'mtREV+F') {
		$mbprset .= 'AAModelPr=Fixed(mtREVF)';
		if ($aafreq eq 'Estimate') {
			$mbprset .= ' StateFreqPr=Dirichlet(1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1)';
		}
		elsif ($aafreq eq 'Empirical') {
			$mbprset .= ' StateFreqPr=Fixed(Empirical)';
		}
	}
	elsif ($model eq 'mtZoa') {
		$mbprset .= 'AAModelPr=Fixed(mtZoa)';
	}
	elsif ($model eq 'mtZoa+F') {
		$mbprset .= 'AAModelPr=Fixed(mtZoaF)';
		if ($aafreq eq 'Estimate') {
			$mbprset .= ' StateFreqPr=Dirichlet(1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1)';
		}
		elsif ($aafreq eq 'Empirical') {
			$mbprset .= ' StateFreqPr=Fixed(Empirical)';
		}
	}
	elsif ($model eq 'PMB') {
		$mbprset .= 'AAModelPr=Fixed(PMB)';
	}
	elsif ($model eq 'PMB+F') {
		$mbprset .= 'AAModelPr=Fixed(PMBF)';
		if ($aafreq eq 'Estimate') {
			$mbprset .= ' StateFreqPr=Dirichlet(1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1)';
		}
		elsif ($aafreq eq 'Empirical') {
			$mbprset .= ' StateFreqPr=Fixed(Empirical)';
		}
	}
	elsif ($model eq 'rtREV') {
		$mbprset .= 'AAModelPr=Fixed(rtREV)';
	}
	elsif ($model eq 'rtREV+F') {
		$mbprset .= 'AAModelPr=Fixed(rtREVF)';
		if ($aafreq eq 'Estimate') {
			$mbprset .= ' StateFreqPr=Dirichlet(1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1)';
		}
		elsif ($aafreq eq 'Empirical') {
			$mbprset .= ' StateFreqPr=Fixed(Empirical)';
		}
	}
	elsif ($model eq 'HIVb') {
		$mbprset .= 'AAModelPr=Fixed(HIVb)';
	}
	elsif ($model eq 'HIVb+F') {
		$mbprset .= 'AAModelPr=Fixed(HIVbF)';
		if ($aafreq eq 'Estimate') {
			$mbprset .= ' StateFreqPr=Dirichlet(1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1)';
		}
		elsif ($aafreq eq 'Empirical') {
			$mbprset .= ' StateFreqPr=Fixed(Empirical)';
		}
	}
	elsif ($model eq 'HIVw') {
		$mbprset .= 'AAModelPr=Fixed(HIVw)';
	}
	elsif ($model eq 'HIVw+F') {
		$mbprset .= 'AAModelPr=Fixed(HIVwF)';
		if ($aafreq eq 'Estimate') {
			$mbprset .= ' StateFreqPr=Dirichlet(1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1)';
		}
		elsif ($aafreq eq 'Empirical') {
			$mbprset .= ' StateFreqPr=Fixed(Empirical)';
		}
	}
	elsif ($model eq 'VT') {
		$mbprset .= 'AAModelPr=Fixed(VT)';
	}
	elsif ($model eq 'VT+F') {
		$mbprset .= 'AAModelPr=Fixed(VTF)';
		if ($aafreq eq 'Estimate') {
			$mbprset .= ' StateFreqPr=Dirichlet(1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1)';
		}
		elsif ($aafreq eq 'Empirical') {
			$mbprset .= ' StateFreqPr=Fixed(Empirical)';
		}
	}
	elsif ($model eq 'WAG') {
		$mbprset .= 'AAModelPr=Fixed(WAG)';
	}
	elsif ($model eq 'WAG+F') {
		$mbprset .= 'AAModelPr=Fixed(WAGF)';
		if ($aafreq eq 'Estimate') {
			$mbprset .= ' StateFreqPr=Dirichlet(1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1)';
		}
		elsif ($aafreq eq 'Empirical') {
			$mbprset .= ' StateFreqPr=Fixed(Empirical)';
		}
	}
	elsif ($model eq 'FLU') {
		$mbprset .= 'AAModelPr=Fixed(FLU)';
	}
	elsif ($model eq 'FLU+F') {
		$mbprset .= 'AAModelPr=Fixed(FLUF)';
		if ($aafreq eq 'Estimate') {
			$mbprset .= ' StateFreqPr=Dirichlet(1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1)';
		}
		elsif ($aafreq eq 'Empirical') {
			$mbprset .= ' StateFreqPr=Fixed(Empirical)';
		}
	}
	elsif ($model eq 'stmtREV') {
		$mbprset .= 'AAModelPr=Fixed(stmtREV)';
	}
	elsif ($model eq 'stmtREV+F') {
		$mbprset .= 'AAModelPr=Fixed(stmtREVF)';
		if ($aafreq eq 'Estimate') {
			$mbprset .= ' StateFreqPr=Dirichlet(1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1)';
		}
		elsif ($aafreq eq 'Empirical') {
			$mbprset .= ' StateFreqPr=Fixed(Empirical)';
		}
	}
	elsif ($model eq 'SYM20') {
		$mbprset .= 'AAModelPr=Fixed(GTR) StateFreqPr=Fixed(Equal)';
	}
	elsif ($model eq 'GTR20') {
		$mbprset .= 'AAModelPr=Fixed(GTR)';
		if ($aafreq eq 'Estimate') {
			$mbprset .= ' StateFreqPr=Dirichlet(1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1)';
		}
		elsif ($aafreq eq 'Empirical') {
			$mbprset .= ' StateFreqPr=Fixed(Empirical)';
		}
	}
	if ($heterogeneity eq 'Homogeneous') {
		$mblset .= 'Rates=Equal';
	}
	elsif ($heterogeneity eq 'Gamma') {
		$mblset .= 'Rates=Gamma NGammaCat=' . $ngammacat;
	}
	elsif ($heterogeneity eq 'Invariant') {
		$mblset .= 'Rates=PropInv';
	}
	elsif ($heterogeneity eq 'GammaInvariant') {
		$mblset .= 'Rates=InvGamma NGammaCat=' . $ngammacat;
	}
	elsif ($heterogeneity eq 'AGamma') {
		$mblset .= ' Rates=ADGamma NGammaCat=' . $ngammacat;
	}
	elsif ($heterogeneity eq 'Gene') {
		$mblset .= ' Rates=Equal';
		$mbprset .= ' RatePr=Variable';
	}
	elsif ($heterogeneity eq 'Gene1Gamma') {
		$mblset .= ' Rates=Gamma NGammaCat=' . $ngammacat;
		$mbprset .= ' RatePr=Variable';
	}
	elsif ($heterogeneity eq 'GeneNGamma') {
		$mblset .= ' Rates=Gamma NGammaCat=' . $ngammacat;
		$mbprset .= ' RatePr=Variable';
		$unlinkshape = 1;
	}
	elsif ($heterogeneity eq 'Gene1AGamma') {
		$mblset .= ' Rates=ADGamma NGammaCat=' . $ngammacat;
		$mbprset .= ' RatePr=Variable';
	}
	elsif ($heterogeneity eq 'GeneNAGamma') {
		$mblset .= ' Rates=ADGamma NGammaCat=' . $ngammacat;
		$mbprset .= ' RatePr=Variable';
		$unlinkshape = 1;
	}
	my $applyto;
	if ($partno) {
		my @applyto;
		for (my $i = 0; $i < $numpart; $i ++) {
			push(@applyto, $partno + $i);
		}
		$applyto = join(',', @applyto);
	}
	else {
		$applyto = 'all';
	}
	my $cmd;
	if ($partname eq 'whole' && $heterogeneity =~ /^Gene/ && !$partno) {
		$cmd = &whole_gene('MrBayes');
	}
	$cmd =~ s/\t//g;
	$cmd =~ s/\n/ /g;
	$cmd .= "LSet ApplyTo=($applyto) $mblset; PrSet ApplyTo=($applyto) $mbprset;";
	if ($unlinkshape) {
		$cmd .= " Unlink Shape=($applyto) Correlation=($applyto);";
	}
	return($cmd);
}

sub dataNexus {
	my $partname = shift(@_);
	my $data;
	unless (open(IN, "< data_$partname.nex")) {
		&errorMessage(__LINE__, "Cannot open \"$maindir/data_$partname.nex\".");
	}
	while (<IN>) {
		$data .= $_;
	}
	close(IN);
	return($data);
}

sub whole_gene {
	my $mode= shift(@_);
	my $cmd;
	if ($mode eq 'MrBayes') {
		foreach my $partname (@partname) {
			$cmd .= "\tCharSet $partname=$part2site{$partname}[0]-$part2site{$partname}[-1];\n";
		}
		$cmd .= "\tPartition Aminosan=" . scalar(@partname) . ':' . join(',', @partname) . ";\n";
		$cmd .= "\tSet Partition=Aminosan;\n";
	}
	elsif ($mode eq 'Treefinder') {
		my $figure = length(scalar(@partname));
		my @partitions;
		my $num = 1;
		foreach my $partname (@partname) {
			for (my $i = 0; $i < @{$part2site{$partname}}; $i ++) {
				my @num = split('', sprintf("%0*d", $figure, $num));
				for (my $j = 0; $j < @num; $j ++) {
					$partitions[$j] .= $num[$j];
				}
			}
			$num ++;
		}
		for (my $i = 0; $i < @partitions; $i ++) {
			$cmd .= '"part' . ($i + 1) . '" ' . $partitions[$i] . "\n";
		}
	}
	elsif ($mode eq 'RAxML') {
		foreach my $partname (@partname) {
			$cmd .= "AAMODEL, $partname=$part2site{$partname}[0]-$part2site{$partname}[-1]\n";
		}
	}
	return($cmd);
}

sub outforTreefinder {
	unless (mkdir('Treefinder')) {
		&errorMessage(__LINE__, "Cannot make \"$maindir/Treefinder\" directory.");
	}
	my @temppartname;
	if (@partname > 1) {
		@temppartname = @partname;
	}
	foreach my $criterion (sort(keys(%criteria))) {
		foreach my $partname ('whole', @temppartname) {
			if ($partname eq 'whole' || scalar(keys(%{$criteria{$criterion}{$partname}}))) {
				&makeTFFiles($criterion, $partname);
			}
		}
	}
}

sub makeTFFiles {
	my ($criterion, $partname) = @_;
	my $partcriterion = 1;
	if ($partname eq 'whole') {
		foreach my $temppartname (@partname) {
			if (!scalar(keys(%{$criteria{$criterion}{$temppartname}}))) {
				$partcriterion = 0;
			}
		}
	}
	my @suffixes;
	# nonpartitioned model
	if ($partname ne 'whole' || scalar(keys(%{$criteria{$criterion}{$partname}}))) {
		my $suffix = '_nonpartitioned';
		push(@suffixes, $suffix);
		&makeTLFiles(0, "$partname$suffix.tf", $partname . '_' . $criterion . $suffix);
		if ($criterion eq 'AIC') {
			open(OUT, "> Treefinder/$partname$suffix.tf");
			print(OUT &dataTF($partname));
			close(OUT);
		}
		open(OUT, '> Treefinder/' . $partname . '_' . $criterion . $suffix . '.model');
		print(OUT &makeTFModelExpression($criterion, $partname));
		close(OUT);
	}
	# proportional / separate model
	if ($partname eq 'whole' && @partname > 1 && $partcriterion == 1) {
		foreach my $mixedmodel ('proportional', 'separate', 'equalrate') {
			my $suffix1 = '_' . $mixedmodel;
			my $suffix2 = '_partitioned';
			push(@suffixes, $suffix1);
			&makeTLFiles(1, "$partname$suffix2.tf", $partname . '_' . $criterion . $suffix1);
			if ($mixedmodel eq 'proportional') {
				if ($criterion eq 'AIC') {
					open(OUT, "> Treefinder/$partname$suffix2.tf");
					print(OUT &whole_gene('Treefinder'));
					print(OUT &dataTF($partname));
					close(OUT);
				}
				open(OUT, '> Treefinder/' . $partname . '_' . $criterion . $suffix2 . '.model');
				print(OUT &makeTFModelExpression($criterion, @partname));
				close(OUT);
			}
			open(OUT, '> Treefinder/' . $partname . '_' . $criterion . $suffix1 . '.rates');
			if ($mixedmodel eq 'separate') {
				my @temprates;
				for (my $i = 1; $i <= @partname; $i ++) {
					push(@temprates, "$i:1.");
				}
				print(OUT "{" . join(',', @temprates) . "}\n");
			}
			elsif ($mixedmodel eq 'proportional') {
				print(OUT "Optimum\n");
			}
			elsif ($mixedmodel eq 'equalrate') {
				my @temprates;
				for (my $i = 1; $i <= @partname; $i ++) {
					push(@temprates, "1:1.");
				}
				print(OUT "{" . join(',', @temprates) . "}\n");
			}
			close(OUT);
		}
	}
	if (scalar(@suffixes) > 1 && ($criterion eq 'AIC' || $criterion eq 'AICc4' || $criterion eq 'BIC4')) {
		# Output TL scripts to compare nonpartitioned and partitioned models
		unless (open(TL, '> Treefinder/' . $partname . '_' . $criterion . '_comparemodels.tl')) {
			&errorMessage(__LINE__, "Cannot make \"$maindir/Treefinder/$partname\_$criterion\_comparemodels.tl\".");
		}
		my @reports;
		for (my $i = 0; $i < scalar(@suffixes); $i ++) {
			my $suffix1 = $suffixes[$i];
			my $suffix2 = $suffix1;
			$suffix2 =~ s/_(?:proportional|separate|equalrate)$/_partitioned/;
			my $datafile = "$partname$suffix2.tf";
			my $prefix1 = $partname . '_' . $criterion . $suffix1;
			my $prefix2 = $partname . '_' . $criterion . $suffix2;
			my $partrates;
			if ($suffix1 ne '_nonpartitioned') {
				$partrates = "\tPartitionRates->Load[\n\t\t\"$prefix1.rates\"\n\t],\n";
			}
			my $report = 'report' . ($i + 1);
			push(@reports, "$report");
			my $tree;
			if ($difftreeeachpart) {
				$tree = "\tTree->\"../starttree_$partname.tre\",";
			}
			else {
				$tree = "\tTree->\"../starttree_whole.tre\",";
			}
			print(TL <<"_END");
$report:=ReconstructPhylogeny[
	\"$datafile\",
	SubstitutionModel->Load[
		\"$prefix2.model\"
	],
$partrates$tree
	WithEdgeSupport->False,
	AcceptFlatness->True
],
_END
		}
		my $reports = join(',', @reports);
		my $sortedby;
		if ($criterion eq 'AIC') {
			$sortedby = '-AIC';
		}
		elsif ($criterion eq 'AICc4') {
			$sortedby = '-AICc';
		}
		elsif ($criterion eq 'BIC4') {
			$sortedby = '-BIC';
		}
		print(TL <<"_END");
Oprec[
	20,
	SaveReport[
		AsReport[
			SortHypotheses[
				JoinReports[
					$reports
				],
				\"$sortedby\"
			]
		],
		\"$partname\_$criterion\_comparemodels.log\"
	]
]
_END
		close(TL);
	}
	if ($partname eq 'whole' && scalar(@suffixes) > 1) {
		@{$suffixes{$criterion}} = @suffixes;
	}
}

sub makeTFModelExpression {
	my %modelTF;
	foreach my $model (@modelfortf) {
		foreach my $heterogeneity (@heterogeneityfortf) {
			$modelTF{$model . '_' . $heterogeneity} = 1;
		}
	}
	my $criterion = shift(@_);
	my $numpart = scalar(@_);
	my $cmd;
	if ($numpart > 1) {
		$cmd .= "{\n";
	}
	my $partno = 0;
	foreach my $partname (@_) {
		$partno ++;
		$cmd .= '% ' . $partname . "\n";
		my $notbest;
		foreach my $model (sort({$criteria{$criterion}{$partname}{$a} <=> $criteria{$criterion}{$partname}{$b}} keys(%{$criteria{$criterion}{$partname}}))) {
			$cmd .= "\t% " . sprintf("%-26s ", $model);
			$cmd .= sprintf("% 12.5e\n", $criteria{$criterion}{$partname}{$model});
			if ($modelTF{$model}) {
				unless ($notbest) {
					$cmd .= "\t\t  ";
				}
				else {
					$cmd .= "\t\t% ";
				}
				$cmd .= &makeTFModel($model);
				unless ($notbest) {
					$notbest = 1;
				}
				$cmd .= "\n";
			}
		}
		if ($partno < $numpart) {
			$cmd .= "\t,\n";
		}
	}
	if ($numpart > 1) {
		$cmd .= "}\n";
	}
	return($cmd);	
}

sub makeTFModel {
	my $model = shift(@_);
	my $cmd;
	if ($model =~ /^Poisson\+F/) {
		$cmd .= 'GTR20[{1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1},';
		if ($aafreq eq 'Empirical') {
			$cmd .= 'Empirical]';
		}
		elsif ($aafreq eq 'Estimate') {
			$cmd .= 'Optimum]';
		}
	}
	elsif ($model =~ /^Poisson/) {
		$cmd .= 'GTR20[{1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1},{1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1}]';
	}
	elsif ($model =~ /^BLOSUM62\+F/) {
		$cmd .= 'BLOSUM[,';
		if ($aafreq eq 'Empirical') {
			$cmd .= 'Empirical]';
		}
		elsif ($aafreq eq 'Estimate') {
			$cmd .= 'Optimum]';
		}
	}
	elsif ($model =~ /^BLOSUM62/) {
		$cmd .= 'BLOSUM[,]';
	}
	elsif ($model =~ /^cpREV\+F/) {
		$cmd .= 'cpREV[,';
		if ($aafreq eq 'Empirical') {
			$cmd .= 'Empirical]';
		}
		elsif ($aafreq eq 'Estimate') {
			$cmd .= 'Optimum]';
		}
	}
	elsif ($model =~ /^cpREV/) {
		$cmd .= 'cpREV[,]';
	}
	elsif ($model =~ /^Dayhoff\+F/) {
		$cmd .= 'Dayhoff[,';
		if ($aafreq eq 'Empirical') {
			$cmd .= 'Empirical]';
		}
		elsif ($aafreq eq 'Estimate') {
			$cmd .= 'Optimum]';
		}
	}
	elsif ($model =~ /^Dayhoff/) {
		$cmd .= 'Dayhoff[,]';
	}
	elsif ($model =~ /^JTT\+F/) {
		$cmd .= 'JTT[,';
		if ($aafreq eq 'Empirical') {
			$cmd .= 'Empirical]';
		}
		elsif ($aafreq eq 'Estimate') {
			$cmd .= 'Optimum]';
		}
	}
	elsif ($model =~ /^JTT/) {
		$cmd .= 'JTT[,]';
	}
	elsif ($model =~ /^LG\+F/) {
		$cmd .= 'LG[,';
		if ($aafreq eq 'Empirical') {
			$cmd .= 'Empirical]';
		}
		elsif ($aafreq eq 'Estimate') {
			$cmd .= 'Optimum]';
		}
	}
	elsif ($model =~ /^LG/) {
		$cmd .= 'LG[,]';
	}
	elsif ($model =~ /^mtArt\+F/) {
		$cmd .= 'mtArt[,';
		if ($aafreq eq 'Empirical') {
			$cmd .= 'Empirical]';
		}
		elsif ($aafreq eq 'Estimate') {
			$cmd .= 'Optimum]';
		}
	}
	elsif ($model =~ /^mtArt/) {
		$cmd .= 'mtArt[,]';
	}
	elsif ($model =~ /^mtMam\+F/) {
		$cmd .= 'mtMam[,';
		if ($aafreq eq 'Empirical') {
			$cmd .= 'Empirical]';
		}
		elsif ($aafreq eq 'Estimate') {
			$cmd .= 'Optimum]';
		}
	}
	elsif ($model =~ /^mtMam/) {
		$cmd .= 'mtMam[,]';
	}
	elsif ($model =~ /^mtREV\+F/) {
		$cmd .= 'mtREV[,';
		if ($aafreq eq 'Empirical') {
			$cmd .= 'Empirical]';
		}
		elsif ($aafreq eq 'Estimate') {
			$cmd .= 'Optimum]';
		}
	}
	elsif ($model =~ /^mtREV/) {
		$cmd .= 'mtREV[,]';
	}
	elsif ($model =~ /^mtZoa\+F/) {
		$cmd .= 'mtZOA[,';
		if ($aafreq eq 'Empirical') {
			$cmd .= 'Empirical]';
		}
		elsif ($aafreq eq 'Estimate') {
			$cmd .= 'Optimum]';
		}
	}
	elsif ($model =~ /^mtZoa/) {
		$cmd .= 'mtZOA[,]';
	}
	elsif ($model =~ /^PMB\+F/) {
		$cmd .= 'PMB[,';
		if ($aafreq eq 'Empirical') {
			$cmd .= 'Empirical]';
		}
		elsif ($aafreq eq 'Estimate') {
			$cmd .= 'Optimum]';
		}
	}
	elsif ($model =~ /^PMB/) {
		$cmd .= 'PMB[,]';
	}
	elsif ($model =~ /^rtREV\+F/) {
		$cmd .= 'rtREV[,';
		if ($aafreq eq 'Empirical') {
			$cmd .= 'Empirical]';
		}
		elsif ($aafreq eq 'Estimate') {
			$cmd .= 'Optimum]';
		}
	}
	elsif ($model =~ /^rtREV/) {
		$cmd .= 'rtREV[,]';
	}
	elsif ($model =~ /^HIVb\+F/) {
		$cmd .= 'betHIV[,';
		if ($aafreq eq 'Empirical') {
			$cmd .= 'Empirical]';
		}
		elsif ($aafreq eq 'Estimate') {
			$cmd .= 'Optimum]';
		}
	}
	elsif ($model =~ /^HIVb/) {
		$cmd .= 'betHIV[,]';
	}
	elsif ($model =~ /^HIVw\+F/) {
		$cmd .= 'witHIV[,';
		if ($aafreq eq 'Empirical') {
			$cmd .= 'Empirical]';
		}
		elsif ($aafreq eq 'Estimate') {
			$cmd .= 'Optimum]';
		}
	}
	elsif ($model =~ /^HIVw/) {
		$cmd .= 'witHIV[,]';
	}
	elsif ($model =~ /^VT\+F/) {
		$cmd .= 'VT[,';
		if ($aafreq eq 'Empirical') {
			$cmd .= 'Empirical]';
		}
		elsif ($aafreq eq 'Estimate') {
			$cmd .= 'Optimum]';
		}
	}
	elsif ($model =~ /^VT/) {
		$cmd .= 'VT[,]';
	}
	elsif ($model =~ /^WAG\+F/) {
		$cmd .= 'WAG[,';
		if ($aafreq eq 'Empirical') {
			$cmd .= 'Empirical]';
		}
		elsif ($aafreq eq 'Estimate') {
			$cmd .= 'Optimum]';
		}
	}
	elsif ($model =~ /^WAG/) {
		$cmd .= 'WAG[,]';
	}
	elsif ($model =~ /^FLU\+F/) {
		$cmd .= 'FLU[,';
		if ($aafreq eq 'Empirical') {
			$cmd .= 'Empirical]';
		}
		elsif ($aafreq eq 'Estimate') {
			$cmd .= 'Optimum]';
		}
	}
	elsif ($model =~ /^FLU/) {
		$cmd .= 'FLU[,]';
	}
	elsif ($model =~ /^stmtREV\+F/) {
		$cmd .= 'stmtREV[,';
		if ($aafreq eq 'Empirical') {
			$cmd .= 'Empirical]';
		}
		elsif ($aafreq eq 'Estimate') {
			$cmd .= 'Optimum]';
		}
	}
	elsif ($model =~ /^stmtREV/) {
		$cmd .= 'stmtREV[,]';
	}
	if ($model =~ /Gamma$/) {
		$cmd .= ':G[Optimum]:' . $ngammacat;
	}
	elsif ($model =~ /GammaInvariant$/) {
		$cmd .= ':GI[Optimum]:' . $ngammacat;
	}
	elsif ($model =~ /Invariant$/) {
		$cmd .= ':I[Optimum]';
	}
	return($cmd);
}

sub dataTF {
	my $partname = shift(@_);
	my $data;
	unless (open(IN, "< data_$partname.nex")) {
		&errorMessage(__LINE__, "Cannot open \"$maindir/data_$partname.nex\".");
	}
	my $line = 0;
	while (<IN>) {
		$line ++;
		if ($line < 7) {
			next;
		}
		elsif ($line >= 7 && $line < 7 + $ntax) {
			s/^(\S+)/"$1"/;
			$data .= $_;
		}
		else {
			last;
		}
	}
	close(IN);
	return($data);
}

sub makeTLFiles {
	my $multilocus = shift(@_);
	my $datafile = shift(@_);
	my $prefix1 = shift(@_);
	my $prefix2 = $prefix1;
	$prefix2 =~ s/_(?:proportional|separate|equalrate)$/_partitioned/;
	my $partrates;
	my $save;
	my $bootstrap;
	if ($multilocus) {
		$partrates = "\tPartitionRates->Load[\n\t\t\"$prefix1.rates\"\n\t],\n";
		$save = "\n\tSave[\n\t\treport|1|PartitionRates,\n\t\t\"$prefix1\_optimum.rates\"\n\t],";
		$bootstrap = "\tPartitionRates->Load[\n\t\t\"$prefix1\_optimum.rates\"\n\t],\n";
	}
	open(TL, '> Treefinder/' . $prefix1 . '_singlesearch.tl');
	print(TL <<"_END");
report:=ReconstructPhylogeny[
	\"$datafile\",
	SubstitutionModel->Load[
		\"$prefix2.model\"
	],
$partrates	WithEdgeSupport->False,
	SearchDepth->2,
	AcceptFlatness->True
],
Oprec[
	20,
	SaveReport[
		AsReport[
			report
		],
		\"$prefix1\_treesearch.log\"
	],
	Save[
		report|1|SubstitutionModel,
		\"$prefix1\_optimum.model\"
	],$save
	SaveTree[
		AsTree[
			report|1|Phylogeny
		],
		\"$prefix1\_optimum.nwk\",
		Format->\"NEWICK\"
	]
]
_END
	close(TL);
	open(TL, '> Treefinder/' . $prefix1 . '_shotgunsearch.tl');
	my $nstarts = $ntax - 3;
	my $nsteps = int((($ntax - 3) / 2) + 0.5);
	if ($nsteps < 5) {
		$nsteps = 5;
	}
	print(TL <<"_END");
SaveTreeList[
	GenerateStartTrees[
		\"$prefix1\_optimum.nwk\",
		NStarts->$nstarts,
		NSteps->$nsteps,
		NTries->10
	],
	\"$prefix1\_starts.nwk\",
	Format->\"NEWICK\"
],
newreport:=ReconstructPhylogeny[
	\"$datafile\",
	SubstitutionModel->Load[
		\"$prefix2.model\"
	],
$partrates	StartTrees->\"$prefix1\_starts.nwk\",
	WithEdgeSupport->False,
	SearchDepth->2,
	AcceptFlatness->True
],
pastreport:=LoadReport[
	\"$prefix1\_treesearch.log\"
],
Spell[\"$prefix1\\n\"],
If[
	pastreport|1|Likelihood>=newreport|1|Likelihood,
	Spell[\"There is no improvement.\\n\"],
	If[
		Oprec[20,ToString[pastreport|1|Likelihood]]==Oprec[20,ToString[newreport|1|Likelihood]],
		Spell[\"There is no improvement.\\n\"],
		Spell[\"The likelihood was improved.\\n\"]
	]
],
report:=SortHypotheses[
	JoinReports[
		newreport,pastreport
	],
	"Likelihood"
],
Oprec[
	20,
	SaveReport[
		AsReport[
			report
		],
		\"$prefix1\_treesearch.log\"
	],
	Save[
		report|1|SubstitutionModel,
		\"$prefix1\_optimum.model\"
	],$save
	SaveTree[
		AsTree[
			report|1|Phylogeny
		],
		\"$prefix1\_optimum.nwk\",
		Format->\"NEWICK\"
	]
]
_END
	close(TL);
	open(TL, '> Treefinder/' . $prefix1 . '_bootstrap.tl');
	print(TL <<"_END");
BootstrapAnalysis[
	\"$datafile\",
	SubstitutionModel->Load[
		\"$prefix1\_optimum.model\"
	],
$bootstrap	StartTrees->\"$prefix1\_optimum.nwk\",
	SearchDepth->2,
	NReplicates->100,
	ConsensusLevel->50,
	SampleFile->\"$prefix1\_bootstrap.log\"
],
boottreelist:=LoadTreeList[
	\"$prefix1\_bootstrap.log\"
],
SaveTreeList[
	boottreelist,
	\"$prefix1\_bootstrap.nwk\",
	Format->\"NEWICK\"
],
SaveTree[
	BuildConsensusTree[
		boottreelist,
		WithEdgeLengthIntervals->False
	],
	\"$prefix1\_consensus.nwk\",
	Format->\"NEWICK\"
]
_END
	close(TL);
}

sub outforPHYML {
	unless (mkdir('PHYML')) {
		&errorMessage(__LINE__, "Cannot make \"$maindir/PHYML\" directory.");
	}
	my @temppartname;
	if (@partname > 1) {
		@temppartname = @partname;
	}
	foreach my $model (@modelforphyml) {
		if ($model !~ /\+F$/) {
			&makeAAModelFiles($model, "PHYML/$model.txt");
		}
	}
	foreach my $criterion (sort(keys(%criteria))) {
		foreach my $partname ('whole', @temppartname) {
			if (scalar(keys(%{$criteria{$criterion}{$partname}}))) {
				&makePHYMLFiles($criterion, $partname);
			}
		}
	}
}

sub makePHYMLFiles {
	my ($criterion, $partname) = @_;
	my %modelPHYML;
	foreach my $model (@modelforphyml) {
		foreach my $heterogeneity (@heterogeneityforphyml) {
			$modelPHYML{$model . '_' . $heterogeneity} = 1;
		}
	}
	if ($criterion eq 'AIC') {
		unless (open(OUT, "> PHYML/$partname.phy")) {
			&errorMessage(__LINE__, "Cannot make \"$maindir/PHYML/$partname.phy\".");
		}
		my ($partnchar, $partseq) = &dataPHYLIP($partname);
		print(OUT " $ntax $partnchar\n$partseq");
		close(OUT);
	}
	my $extension;
	my $commentout;
	if ($^O eq 'MSWin32') {
		$extension = '.bat';
		$commentout = '::';
	}
	else {
		$extension = '.sh';
		$commentout = '#';
	}
	foreach my $analysis ('singlesearch', 'shotgunsearch', 'bootstrap', 'shotgunbootstrap') {
		unless (open(OUT, '> PHYML/' . $partname . '_' . $criterion . '_' . $analysis . $extension)) {
			&errorMessage(__LINE__, "Cannot make \"$maindir/PHYML/$partname\_$criterion\_$analysis$extension\".");
		}
		my $notbest;
		foreach my $model (sort({$criteria{$criterion}{$partname}{$a} <=> $criteria{$criterion}{$partname}{$b}} keys(%{$criteria{$criterion}{$partname}}))) {
			print(OUT $commentout . ' ');
			printf(OUT "%-26s ", $model);
			printf(OUT "% 12.5e", $criteria{$criterion}{$partname}{$model});
			print(OUT "\n");
			if ($modelPHYML{$model}) {
				if ($notbest) {
					print(OUT $commentout . ' ');
				}
				print(OUT "phyml --run_id $partname\_$criterion\_$analysis --input $partname.phy --datatype aa --sequential --search BEST -o tlr");
				if ($analysis =~ /shotgun/) {
					my $nstarts = $ntax - 3;
					if ($nstarts < 10) {
						$nstarts = 10;
					}
					print(OUT " --n_rand_starts $nstarts");
				}
				if ($analysis =~ /bootstrap/) {
					print(OUT " --bootstrap 100");
				}
				print(OUT &makePHYMLModel(split(/_/, $model)));
				unless ($notbest) {
					$notbest = 1;
				}
			}
			print(OUT "\n");
		}
		close(OUT);
	}
}

sub makePHYMLModel {
	my ($model, $heterogeneity) = @_;
	my $aamodelfile = $model;
	$aamodelfile =~ s/\+F$//;
	$aamodelfile .= '.txt';
	my $cmd = " --model custom --aa_rate_file $aamodelfile";
	if ($model =~ /\+F$/) {
		$cmd .= ' -f e';
	}
	else {
		$cmd .= ' -f m';
	}
	if ($heterogeneity eq 'Homogeneous') {
		$cmd .= ' --nclasses 1';
	}
	elsif ($heterogeneity eq 'Gamma') {
		$cmd .= " --alpha e --nclasses $ngammacat";
	}
	elsif ($heterogeneity eq 'Invariant') {
		$cmd .= ' --nclasses 1 -v e';
	}
	elsif ($heterogeneity eq 'GammaInvariant') {
		$cmd .= " -v e --alpha e --nclasses $ngammacat";
	}
	return($cmd);
}

sub dataPHYLIP {
	my $partname = shift(@_);
	my $partnchar;
	my $data;
	unless (open(IN, "< data_$partname.nex")) {
		&errorMessage(__LINE__, "Cannot open \"$maindir/data_$partname.nex\".");
	}
	my $line = 0;
	while (<IN>) {
		$line ++;
		if ($line < 7) {
			if (/NChar=(\d+)/i) {
				$partnchar = $1;
			}
			next;
		}
		elsif ($line >= 7 && $line < 7 + $ntax) {
			$data .= $_;
		}
		else {
			last;
		}
	}
	close(IN);
	return($partnchar, $data);
}

sub outforRAxML {
	unless (mkdir('RAxML')) {
		&errorMessage(__LINE__, "Cannot make \"$maindir/RAxML\" directory.");
	}
	my @temppartname;
	if (@partname > 1) {
		@temppartname = @partname;
	}
	foreach my $partname ('whole', @temppartname) {
		unless (open(OUT, "> RAxML/$partname.phy")) {
			&errorMessage(__LINE__, "Cannot make \"$maindir/RAxML/$partname.phy\".");
		}
		my ($partnchar, $partseq) = &dataPHYLIP($partname);
		print(OUT " $ntax $partnchar\n$partseq");
		close(OUT);
	}
	foreach my $criterion (sort(keys(%criteria))) {
		foreach my $partname ('whole', @temppartname) {
			if ($partname eq 'whole' || scalar(keys(%{$criteria{$criterion}{$partname}}))) {
				&makeRAxMLFiles($criterion, $partname);
			}
		}
	}
}

sub makeRAxMLFiles {
	my ($criterion, $partname) = @_;
	my $partcriterion = 1;
	if ($partname eq 'whole') {
		foreach my $temppartname (@partname) {
			if (!scalar(keys(%{$criteria{$criterion}{$temppartname}}))) {
				$partcriterion = 0;
			}
		}
	}
	my @suffixes;
	# nonpartitioned model
	if ($partname ne 'whole' || scalar(keys(%{$criteria{$criterion}{$partname}}))) {
		my $suffix = '_nonpartitioned';
		push(@suffixes, $suffix);
		&makeRAxMLCommands(0, "$partname.phy", $partname, $criterion, $suffix, $suffix);
	}
	if ($partname eq 'whole' && @partname > 1 && $partcriterion == 1) {
		# separate model
		foreach my $mixedmodel ('separate') {
			my $suffix1 = '_' . $mixedmodel;
			my $suffix2 = '_partitioned';
			push(@suffixes, $suffix1);
			my $partitionexpression = &whole_gene('RAxML');
			foreach my $temppartname (@partname) {
				foreach my $model (sort({$criteria{$criterion}{$temppartname}{$a} <=> $criteria{$criterion}{$temppartname}{$b}} keys(%{$criteria{$criterion}{$temppartname}}))) {
					my $converted = &makeRAxMLModel(split(/_/, $model));
					$partitionexpression =~ s/AAMODEL, $temppartname=/$converted, $temppartname=/;
					last;
				}
			}
			open(OUT, "> RAxML/$partname\_$criterion$suffix2.partition");
			print(OUT $partitionexpression);
			close(OUT);
			&makeRAxMLCommands(2, "$partname.phy", $partname, $criterion, $suffix1, $suffix2);
		}
		# Gamma within partition and equal rate among partition (EqualRate)
		foreach my $mixedmodel ('equalrate') {
			my $suffix1;
			my $suffix2;
			$suffix1 = '_' . $mixedmodel;
			$suffix2 = '_partitioned';
			push(@suffixes, $suffix1);
			&makeRAxMLCommands(1, "$partname.phy", $partname, $criterion, $suffix1, $suffix2);
		}
	}
	if ($partname eq 'whole' && scalar(@suffixes) > 1) {
		@{$suffixes{$criterion}} = @suffixes;
	}
}

sub makeRAxMLCommands {
	my $multilocus = shift(@_);
	my $datafile = shift(@_);
	my $partname = shift(@_);
	my $criterion = shift(@_);
	my $suffix1 = shift(@_);
	my $suffix2 = shift(@_);
	my $prefix1 = $partname . '_' . $criterion . $suffix1;
	my $prefix2 = $partname . '_' . $criterion . $suffix2;
	my $extension;
	my $commentout;
	if ($^O eq 'MSWin32') {
		$extension = '.bat';
		$commentout = '::';
	}
	else {
		$extension = '.sh';
		$commentout = '#';
	}
	my %modelRAxML;
	foreach my $model (@modelforraxml) {
		foreach my $heterogeneity (@heterogeneityforraxml) {
			$modelRAxML{$model . '_' . $heterogeneity} = 1;
		}
	}
	foreach my $analysis ('singlesearch', 'shotgunsearch', 'bootstrap') {
		my $foption;
		my $nstarts;
		if ($analysis =~ /shotgun/) {
			$nstarts = int((($ntax - 3) / 4) + 0.5);
			if ($nstarts < 10) {
				$nstarts = 10;
			}
		}
		if ($analysis eq 'singlesearch') {
			$foption = '-f d -p 1234';
		}
		elsif ($analysis eq 'shotgunsearch') {
			$foption = "-f d -p 1234 -N $nstarts";
		}
		elsif ($analysis eq 'bootstrap') {
			$foption = '-f d -p 1234 -b 5678 -N 100';
		}
		#elsif ($analysis eq 'shotgunbootstrap') {
		#	$foption = "-f d -p 1234 -b 5678 -N 100 -u $nstarts";
		#}
		unless (open(OUT, '> RAxML/' . $prefix1 . '_' . $analysis . $extension)) {
			&errorMessage(__LINE__, "Cannot make \"$maindir/RAxML/$prefix1\_$analysis$extension\".");
		}
		if ($multilocus == 1) {
			print(OUT "$raxml --no-seq-check --no-bfgs -n $prefix1\_$analysis -s $datafile $foption -m PROTGAMMAWAG -q $prefix2.partition\n");
		}
		elsif ($multilocus == 2) {
			print(OUT "$raxml --no-seq-check --no-bfgs -n $prefix1\_$analysis -s $datafile $foption -m PROTGAMMAWAG -q $prefix2.partition -M\n");
		}
		else {
			my $notbest;
			foreach my $model (sort({$criteria{$criterion}{$partname}{$a} <=> $criteria{$criterion}{$partname}{$b}} keys(%{$criteria{$criterion}{$partname}}))) {
				print(OUT $commentout . ' ');
				printf(OUT "%-26s ", $model);
				printf(OUT "% 12.5e", $criteria{$criterion}{$partname}{$model});
				print(OUT "\n");
				if ($modelRAxML{$model}) {
					if ($notbest) {
						print(OUT $commentout . ' ');
					}
					print(OUT "$raxml --no-seq-check --no-bfgs -n $prefix1\_$analysis -s $datafile $foption -m PROTGAMMA");
					print(OUT &makeRAxMLModel(split(/_/, $model)));
					unless ($notbest) {
						$notbest = 1;
					}
				}
				print(OUT "\n");
			}
		}
		close(OUT);
	}
}

sub makeRAxMLModel {
	my ($model, $heterogeneity) = @_;
	my $converted;
	if ($model =~ /^BLOSUM62/) {
		$converted = 'BLOSUM62';
	}
	elsif ($model =~ /^cpREV/) {
		$converted = 'CPREV';
	}
	elsif ($model =~ /^Dayhoff/) {
		$converted = 'DCMUT';
	}
	elsif ($model =~ /^JTT/) {
		$converted = 'JTTDCMUT';
	}
	elsif ($model =~ /^LG4M/) {
		$converted = 'LG4M';
	}
	elsif ($model =~ /^LG4X/) {
		$converted = 'LG4X';
	}
	elsif ($model =~ /^LG/) {
		$converted = 'LG';
	}
	elsif ($model =~ /^mtArt/) {
		$converted = 'MTART';
	}
	elsif ($model =~ /^mtMam/) {
		$converted = 'MTMAM';
	}
	elsif ($model =~ /^mtREV/) {
		$converted = 'MTREV';
	}
	elsif ($model =~ /^mtZoa/) {
		$converted = 'MTZOA';
	}
	elsif ($model =~ /^PMB/) {
		$converted = 'PMB';
	}
	elsif ($model =~ /^rtREV/) {
		$converted = 'RTREV';
	}
	elsif ($model =~ /^HIVb/) {
		$converted = 'HIVB';
	}
	elsif ($model =~ /^HIVw/) {
		$converted = 'HIVW';
	}
	elsif ($model =~ /^VT/) {
		$converted = 'VT';
	}
	elsif ($model =~ /^WAG/) {
		$converted = 'WAG';
	}
	elsif ($model =~ /^FLU/) {
		$converted = 'FLU';
	}
	elsif ($model =~ /^stmtREV/) {
		$converted = 'STMTREV';
	}
	if ($model =~ /\+F$/) {
		if ($aafreq eq 'Estimate') {
			$converted .= 'X';
		}
		else {
			$converted .= 'F';
		}
	}
	return($converted);
}

sub compareMixedModels {
	if ($output{'raxml'}) {
		print("Comparing nonpartitioned, equalrate, and separate models...\n");
	}
	else {
		print("Comparing nonpartitioned, equalrate, proportional, and separate models...\n");
	}
	unless (chdir($maindir)) {
		&errorMessage(__LINE__, 'Cannot change working directory.');
	}
	# Optimize likelihoods of nonpartitioned and partitioned models
	my $child = 0;
	$| = 1;
	$? = 0;
	foreach my $criterion (sort(keys(%suffixes))) {
		my @suffixes = @{$suffixes{$criterion}};
		for (my $i = 0; $i < scalar(@suffixes); $i ++) {
			if (my $pid = fork()) {
				$child ++;
				if ($child == $numthreads) {
					if (wait == -1) {
						$child = 0;
					}
					else {
						$child --;
					}
				}
				if ($?) {
					&errorMessage(__LINE__, 'Likelihood calculation error.');
				}
				next;
			}
			else {
				if (!$output{'raxml'} && $lnlcalc ne 'tf' || !$output{'raxml'} && $suffixes[$i] =~ /proportional/ || !$output{'raxml'} && $suffixes[$i] =~ /equalrate/) {
					&calcLofMixedModelsbyTF($criterion, 'whole', $suffixes[$i]);
				}
				elsif ($output{'raxml'} && $suffixes[$i] =~ /equalrate/) {
					&calcLofMixedModelsbyRAxML($criterion, 'whole', $suffixes[$i]);
				}
				exit;
			}
		}
	}
	while (wait != -1) {
		if ($?) {
			&errorMessage(__LINE__, 'Likelihood calculation error.');
		}
	}
	print("done.\n\n");
	print("Outputting results...\n");
	# Calculate criterion
	foreach my $criterion (sort(keys(%suffixes))) {
		my @suffixes = @{$suffixes{$criterion}};
		my %criterion;
		my %logLik;
		my %numParams;
		for (my $i = 0; $i < scalar(@suffixes); $i ++) {
			my $suffix1 = $suffixes[$i];
			my $prefix1 = 'whole_' . $criterion . $suffix1;
			my $model = $suffix1;
			$model =~ s/^_//;
			$model =~ s/proportional/Proportional/;
			$model =~ s/separate/Separate/;
			$model =~ s/nonpartitioned/Nonpartitioned/;
			$model =~ s/equalrate/EqualRate/;
			if (!$output{'raxml'} && $lnlcalc ne 'tf' || !$output{'raxml'} && $model =~ /Proportional/ || !$output{'raxml'} && $model =~ /EqualRate/) {
				unless (open(LNL, '< Scores/' . $prefix1 . '.txt')) {
					&errorMessage(__LINE__, "Cannot open \"$maindir/Scores/$prefix1.txt\".");
				}
				while (<LNL>) {
					if (/Likelihood->-(\d+(?:\.\d+)?)/) {
						$logLik{$model} = $1;
					}
					if (/NParameters->(\d+)/) {
						$numParams{$model} = $1;
					}
				}
				close(LNL);
			}
			elsif ($output{'raxml'} && $model =~ /EqualRate/) {
				unless (open(LNL, '< Scores/' . $prefix1 . '.txt')) {
					&errorMessage(__LINE__, "Cannot open \"$maindir/Scores/$prefix1.txt\".");
				}
				while (<LNL>) {
					if (/^Final +GAMMA +likelihood: +-(\d+(?:\.\d+)?)/) {
						$logLik{$model} = $1;
					}
					if (/^Using (\d+) distinct models/) {
						$numParams{$model} = (1 * $1) + (2 * $ntax) - 3;
					}
					if (/^Using empirical base frequencies/ || /^Using ML estimate of base frequencies/) {
						$numParams{$model} += 19;
					}
				}
				close(LNL);
			}
			elsif ($model eq 'Separate') {
				foreach my $partname (@partname) {
					foreach my $tempmodel (sort({$criteria{$criterion}{$partname}{$a} <=> $criteria{$criterion}{$partname}{$b}} keys(%{$criteria{$criterion}{$partname}}))) {
						if ($criteria{$criterion}{$partname}{$tempmodel} > 0) {
							$logLik{$model} += $LnL{$partname}{$tempmodel};
							$numParams{$model} += $nparam{$partname}{$tempmodel};
						}
						last;
					}
				}
			}
			elsif ($model eq 'Nonpartitioned') {
				foreach my $tempmodel (sort({$criteria{$criterion}{'whole'}{$a} <=> $criteria{$criterion}{'whole'}{$b}} keys(%{$criteria{$criterion}{'whole'}}))) {
					if ($criteria{$criterion}{'whole'}{$tempmodel} > 0) {
						$criterion{$model} = $criteria{$criterion}{'whole'}{$tempmodel};
						$logLik{$model} = $LnL{'whole'}{$tempmodel};
						$numParams{$model} = $nparam{'whole'}{$tempmodel};
					}
					last;
				}
			}
			else {
				print("ERROR!: line " . __LINE__ . "\nUnknown model.\n");
			}
			unless ($criterion{$model}) {
				if ($criterion eq 'AIC') {
					$criterion{$model} = &calcAIC($logLik{$model}, $numParams{$model});
				}
				elsif ($criterion eq 'AICc1' && $samplesize{'whole'}[0] - $numParams{$model} - 1 > 0) {
					$criterion{$model} = &calcAICc($logLik{$model}, $numParams{$model}, $samplesize{'whole'}[0]);
				}
				#elsif ($criterion eq 'AICc2' && $samplesize{'whole'}[1] - $numParams{$model} - 1 > 0) {
				#	$criterion{$model} = &calcAICc($logLik{$model}, $numParams{$model}, $samplesize{'whole'}[1]);
				#}
				#elsif ($criterion eq 'AICc3' && $samplesize{'whole'}[2] - $numParams{$model} - 1 > 0) {
				#	$criterion{$model} = &calcAICc($logLik{$model}, $numParams{$model}, $samplesize{'whole'}[2]);
				#}
				elsif ($criterion eq 'AICc4' && $samplesize{'whole'}[3] - $numParams{$model} - 1 > 0) {
					$criterion{$model} = &calcAICc($logLik{$model}, $numParams{$model}, $samplesize{'whole'}[3]);
				}
				#elsif ($criterion eq 'AICc5' && $samplesize{'whole'}[4] - $numParams{$model} - 1 > 0) {
				#	$criterion{$model} = &calcAICc($logLik{$model}, $numParams{$model}, $samplesize{'whole'}[4]);
				#}
				elsif ($criterion eq 'AICc6' && $samplesize{'whole'}[5] - $numParams{$model} - 1 > 0) {
					$criterion{$model} = &calcAICc($logLik{$model}, $numParams{$model}, $samplesize{'whole'}[5]);
				}
				elsif ($criterion eq 'BIC1') {
					$criterion{$model} = &calcBIC($logLik{$model}, $numParams{$model}, $samplesize{'whole'}[0]);
				}
				#elsif ($criterion eq 'BIC2') {
				#	$criterion{$model} = &calcBIC($logLik{$model}, $numParams{$model}, $samplesize{'whole'}[1]);
				#}
				#elsif ($criterion eq 'BIC3') {
				#	$criterion{$model} = &calcBIC($logLik{$model}, $numParams{$model}, $samplesize{'whole'}[2]);
				#}
				elsif ($criterion eq 'BIC4') {
					$criterion{$model} = &calcBIC($logLik{$model}, $numParams{$model}, $samplesize{'whole'}[3]);
				}
				#elsif ($criterion eq 'BIC5') {
				#	$criterion{$model} = &calcBIC($logLik{$model}, $numParams{$model}, $samplesize{'whole'}[4]);
				#}
				elsif ($criterion eq 'BIC6') {
					$criterion{$model} = &calcBIC($logLik{$model}, $numParams{$model}, $samplesize{'whole'}[5]);
				}
			}
		}
		# Output results
		unless (open(OUT, "> Results/whole\_$criterion\_comparemix.txt")) {
			&errorMessage(__LINE__, "Cannot make \"$maindir/Resilts/whole\_$criterion\_comparemix.txt\".");
		}
		print(OUT 'model                   ' . "\t");
		printf(OUT "% 19s \t", $criterion);
		print(OUT '               -LnL ' . "\t" . 'nparam');
		if ($criterion =~ /\d$/) {
			print(OUT "\t" . 'sample size' );
		}
		print(OUT "\n");
		foreach my $model (sort({$criterion{$a} <=> $criterion{$b}} keys(%criterion))) {
			printf(OUT "%-24s\t", $model);
			printf(OUT "% 20.12e\t", $criterion{$model});
			printf(OUT "% 20.12e\t", $logLik{$model});
			printf(OUT "% 6d", $numParams{$model});
			if ($criterion =~ /(\d)$/) {
				printf(OUT "\t% 11d", $samplesize{'whole'}[$1 - 1]);
			}
			print(OUT "\n");
		}
		close(OUT);
	}
	#if (!$output{'treefinder'} && $output{'mrbayes'}) {
	#	while (glob("Treefinder/*")) {
	#		unlink($_);
	#	}
	#	rmdir("Treefinder");
	#}
	unless (chdir('..')) {
		&errorMessage(__LINE__, 'Cannot change working directory.');
	}
	print("done.\n\n");
}

sub calcLofMixedModelsbyTF {
	my ($criterion, $partname, $suffix1) = @_;
	my $suffix2 = $suffix1;
	$suffix2 =~ s/_(?:proportional|separate|equalrate)$/_partitioned/;
	my $model = $suffix1;
	$model =~ s/^_//;
	$model =~ s/proportional/Proportional/;
	$model =~ s/separate/Separate/;
	$model =~ s/nonpartitioned/Nonpartitioned/;
	$model =~ s/equalrate/EqualRate/;
	print("Calculating likelihood under $model model which selected by $criterion on $partname partition...\n");
	my $datafile = "$partname$suffix2.tf";
	my $prefix1 = $partname . '_' . $criterion . $suffix1;
	my $prefix2 = $partname . '_' . $criterion . $suffix2;
	# redundancy check
	my $modelexpression = $model;
	unless (open(MODEL, "< Treefinder/$prefix2.model")) {
		&errorMessage(__LINE__, "Cannot read \"$maindir/Treefinder/$prefix2.model\".");
	}
	while (<MODEL>) {
		s/\r?\n?//;
		s/\%.+//;
		s/\s+//g;
		$modelexpression .= $_;
	}
	close(MODEL);
	use Digest::MD5 qw(md5_hex);
	$modelexpression = md5_hex($modelexpression);
	no Digest::MD5;
	# if it has already calculated
	if (-e "$modelexpression.temp") {
		unless (open(TEMP, "< $modelexpression.temp")) {
			&errorMessage(__LINE__, "Cannot read \"$maindir/$modelexpression.temp\".");
		}
		unless (flock(TEMP, 1)) {
			&errorMessage(__LINE__, "Cannot lock \"$maindir/$modelexpression.temp\".");
		}
		my $sourcefile = readline(TEMP);
		close(TEMP);
		unless (open(LNL1, "< $sourcefile")) {
			&errorMessage(__LINE__, "Cannot read \"$maindir/$sourcefile\".");
		}
		unless (flock(LNL1, 1)) {
			&errorMessage(__LINE__, "Cannot lock \"$maindir/$sourcefile\".");
		}
		unless (open(LNL2, "> Scores/$prefix1.txt")) {
			&errorMessage(__LINE__, "Cannot make \"$maindir/Scores/$prefix1.txt\".");
		}
		while (<LNL1>) {
			print(LNL2 $_);
		}
		close(LNL2);
		close(LNL1);
	}
	# if it has not calculated yet
	else {
		unless (open(TEMP, "> $modelexpression.temp")) {
			&errorMessage(__LINE__, "Cannot make \"$maindir/$modelexpression.temp\".");
		}
		unless (flock(TEMP, 2)) {
			&errorMessage(__LINE__, "Cannot lock \"$maindir/$modelexpression.temp\".");
		}
		print(TEMP "Scores/$prefix1.txt");
		my $partrates;
		if ($model ne 'Nonpartitioned') {
			$partrates = "\tPartitionRates->Load[\n\t\t\"Treefinder/$prefix1.rates\"\n\t],\n";
		}
		my $tree;
		if ($difftreeeachpart) {
			$tree = "\tTree->\"starttree_$partname.tre\",";
		}
		else {
			$tree = "\tTree->\"starttree_whole.tre\",";
		}
		unless (open(TL, '> Logs/' . $prefix1 . '.tl')) {
			&errorMessage(__LINE__, "Cannot make \"$maindir/Logs/$prefix1.tl\".");
		}
		print(TL <<"_END");
report:=ReconstructPhylogeny[
	\"Treefinder/$datafile\",
	SubstitutionModel->Load[
		\"Treefinder/$prefix2.model\"
	],
$partrates$tree
	WithEdgeSupport->False,
	Verbose->False,
	AcceptFlatness->True
],
Oprec[
	20,
	SaveReport[
		AsReport[
			report
		],
		\"Scores/$prefix1.txt\"
	]
]
_END
		close(TL);
		if (system("$tf Logs/" . $prefix1 . '.tl')) {
			&errorMessage(__LINE__, "Cannot run $tf. Please check $tf.");
		}
		close(TEMP);
	}
}

sub calcLofMixedModelsbyRAxML {
	my ($criterion, $partname, $suffix1) = @_;
	my $suffix2 = $suffix1;
	$suffix2 =~ s/_equalrate$/_partitioned/;
	my $model = $suffix1;
	$model =~ s/^_//;
	$model =~ s/nonpartitioned/Nonpartitioned/;
	$model =~ s/equalrate/EqualRate/;
	print("Calculating likelihood under $model model which selected by $criterion on $partname partition...\n");
	my $datafile = "$partname$suffix2.tf";
	my $prefix1 = $partname . '_' . $criterion . $suffix1;
	my $prefix2 = $partname . '_' . $criterion . $suffix2;
	# redundancy check
	my $modelexpression = $model;
	unless (open(MODEL, "< RAxML/$prefix2.partition")) {
		&errorMessage(__LINE__, "Cannot read \"$maindir/RAxML/$prefix2.partition\".");
	}
	while (<MODEL>) {
		s/\r?\n?//;
		s/\s+//g;
		$modelexpression .= $_;
	}
	close(MODEL);
	use Digest::MD5 qw(md5_hex);
	$modelexpression = md5_hex($modelexpression);
	no Digest::MD5;
	# if it has already calculated
	if (-e "$modelexpression.temp") {
		unless (open(TEMP, "< $modelexpression.temp")) {
			&errorMessage(__LINE__, "Cannot read \"$maindir/$modelexpression.temp\".");
		}
		unless (flock(TEMP, 1)) {
			&errorMessage(__LINE__, "Cannot lock \"$maindir/$modelexpression.temp\".");
		}
		my $sourcefile = readline(TEMP);
		close(TEMP);
		unless (open(LNL1, "< $sourcefile")) {
			&errorMessage(__LINE__, "Cannot read \"$maindir/$sourcefile\".");
		}
		unless (flock(LNL1, 1)) {
			&errorMessage(__LINE__, "Cannot lock \"$maindir/$sourcefile\".");
		}
		unless (open(LNL2, "> Scores/$prefix1.txt")) {
			&errorMessage(__LINE__, "Cannot make \"$maindir/Scores/$prefix1.txt\".");
		}
		while (<LNL1>) {
			print(LNL2 $_);
		}
		close(LNL2);
		close(LNL1);
	}
	# if it has not calculated yet
	else {
		unless (open(TEMP, "> $modelexpression.temp")) {
			&errorMessage(__LINE__, "Cannot make \"$maindir/$modelexpression.temp\".");
		}
		unless (flock(TEMP, 2)) {
			&errorMessage(__LINE__, "Cannot lock \"$maindir/$modelexpression.temp\".");
		}
		print(TEMP "Scores/$prefix1.txt");
		my $partition;
		if ($model ne 'Nonpartitioned') {
			$partition = "-q RAxML/$prefix2.partition";
		}
		my $tree;
		if ($difftreeeachpart) {
			$tree = "starttree_$partname.nwk";
		}
		else {
			$tree = "starttree_whole.nwk";
		}
		if (system("$raxml --no-seq-check --no-bfgs -f e -m PROTGAMMAWAG -n $prefix1 -t $tree -s data_$partname.phy $partition > Logs/$prefix1.txt")) {
			&errorMessage(__LINE__, "Cannot run $raxml. Please check $raxml.");
		}
		unless (rename("RAxML_info.$prefix1", "Scores/$prefix1.txt")) {
			&errorMessage(__LINE__, "Cannot move \"$maindir/RAxML_info.$prefix1\".");
		}
		foreach my $file (glob("RAxML_*.$prefix1")) {
			unlink($file);
		}
		close(TEMP);
	}
}

sub makeAAModelFiles {
	my $model = shift(@_);
	my $datfilepath = shift(@_);
	my $datfile;
	unless (open($datfile, '>', $datfilepath)) {
		&errorMessage(__LINE__, 'Cannot make file.');
	}
	if ($model eq 'Poisson') {
		print($datfile <<'_END');
1
1  1
1  1  1
1  1  1  1
1  1  1  1  1
1  1  1  1  1  1
1  1  1  1  1  1  1
1  1  1  1  1  1  1  1
1  1  1  1  1  1  1  1  1
1  1  1  1  1  1  1  1  1  1
1  1  1  1  1  1  1  1  1  1  1
1  1  1  1  1  1  1  1  1  1  1  1
1  1  1  1  1  1  1  1  1  1  1  1  1
1  1  1  1  1  1  1  1  1  1  1  1  1  1
1  1  1  1  1  1  1  1  1  1  1  1  1  1  1
1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1
1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1
1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1
1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1

0.05  0.05  0.05  0.05  0.05  0.05  0.05  0.05  0.05  0.05  0.05  0.05  0.05  0.05  0.05  0.05  0.05  0.05  0.05  0.05
_END
	}
	elsif ($model eq 'BLOSUM62') {
		print($datfile <<'_END');
0.73579038969751
0.48539105546575  1.29744670513370
0.54316182089867  0.50096440855513  3.18010004821610
1.45999531047000  0.22782657420895  0.39735894989702  0.24083661480204
1.19970570460200  3.02083361006360  1.83921614699200  1.19094570339600  0.32980150463028
1.17094904279990  1.36057419042030  1.24048850863960  3.76162520836850  0.14074889181440  5.52891917792820
1.95588357495950  0.41876330851753  1.35587234448450  0.79847324896839  0.41820319228376  0.60984630538281  0.42357999217628
0.71624144499779  1.45614116633600  2.41450143420810  0.77814266402188  0.35405810983129  2.43534113114010  1.62689105698170  0.53985912495418
0.60589900368677  0.23203644514174  0.28301732627800  0.41855573246161  0.77489402279418  0.23620245120365  0.18684804693170  0.18929629237636  0.25271844788492
0.80001653051838  0.62271166969249  0.21188815961519  0.21813157759360  0.83184264014158  0.58073709318144  0.37262517508685  0.21772115923623  0.34807220979697  3.89096377330350
1.29520126678330  5.41111514148890  1.59313704345740  1.03244792495210  0.28507880090648  3.94527767451460  2.80242715167870  0.75204244030271  1.02250703588900  0.40619358664202  0.44557027426059
1.25375826666350  0.98369298745695  0.64844127878707  0.22262189795786  0.76768882347954  2.49489607711270  0.55541539747043  0.45943617357855  0.98431152535870  3.36479776310420  6.03055937957160  1.07306118433190
0.49296467974759  0.37164469320875  0.35486124922252  0.28173069420651  0.44133747118660  0.14435695975031  0.29140908416530  0.36816646445253  0.71453370392764  1.51735932595390  2.06483970323750  0.26692475051102  1.77385516883050
1.17327590092390  0.44813366171831  0.49488704370192  0.73062827299842  0.35600849876863  0.85857057567418  0.92656393484598  0.50408659952683  0.52700733915060  0.38835540920564  0.37455568747097  1.04738345072150  0.45412362510273  0.23359790962888
4.32509268705660  1.12278310420960  2.90410165645600  1.58275414206530  1.19718841509420  1.93487092459650  1.76989323893730  1.50932625322360  1.11702976291050  0.35754441245967  0.35296918452729  1.75216591781950  0.91872341574605  0.54002764482413  1.16912957771570
1.72917801948500  0.91466595456337  1.89817363453320  0.93418750943056  1.11983135851600  1.27748029459560  1.07109723600730  0.64143601140497  0.58540709022472  1.17909119726010  0.91525985769421  1.30387520079870  1.48854805372180  0.48820611879305  1.00545168314880  5.15155629227040
0.46583936772479  0.42638231012175  0.19148204624678  0.14534504627853  0.52766441887169  0.75865380864172  0.40763564893830  0.50835892463812  0.30124860078016  0.34198578754023  0.69147463459998  0.33224304063396  0.88810109815193  2.07432489349650  0.25221483002727  0.38792562209837  0.51312812689059
0.71820669758623  0.72051744121611  0.53822251903674  0.26142220896504  0.47023773369610  0.95898974285014  0.59671930034577  0.30805573703500  4.21895396938900  0.67461709322842  0.81124585632307  0.71799348690032  0.95168216224591  6.74726043080080  0.36940531935451  0.79675152076106  0.80101024319939  4.05441900655800
2.18777452200450  0.43838834377202  0.31285879799342  0.25812928941763  1.11635247860620  0.53078579012486  0.52425384633796  0.25334079019018  0.20155597175031  8.31183940545820  2.23140568891310  0.49813847530407  2.57585075531530  0.83811961017754  0.49690841067567  0.56192545744165  2.25307405117630  0.26650873142646  1.00000000000000

0.074  0.052  0.045  0.054  0.025  0.034  0.054  0.074  0.026  0.068  0.099  0.058  0.025  0.047  0.039  0.057  0.051  0.013  0.032  0.073
_END
	}
	elsif ($model eq 'cpREV') {
		print($datfile <<'_END');
  105
  227  357
  175   43 4435
  669  823  538   10
  157 1745  768  400   10
  499  152 1055 3691   10 3122
  665  243  653  431  303  133  379
   66  715 1405  331  441 1269  162   19
  145  136  168   10  280   92  148   40   29
  197  203  113   10  396  286   82   20   66 1745
  236 4482 2430  412   48 3313 2629  263  305  345  218
  185  125   61   47  159  202  113   21   10 1772 1351  193
   68   53   97   22  726   10  145   25  127  454 1268   72  327
  490   87  173  170  285  323  185   28  152  117  219  302  100   43
 2440  385 2085  590 2331  396  568  691  303  216  516  868   93  487 1202
 1340  314 1393  266  576  241  369   92   32 1040  156  918  645  148  260 2151
   14  230   40   18  435   53   63   82   69   42  159   10   86  468   49   73   29
   56  323  754  281 1466  391  142   10 1971   89  189  247  215 2370   97  522   71  346
  968   92   83   75  592   54  200   91   25 4797  865  249  475  317  122  167  760   10  119

0.0755  0.0621  0.0410  0.0371  0.0091  0.0382  0.0495  0.0838  0.0246  0.0806  0.1011  0.0504  0.0220  0.0506  0.0431  0.0622  0.0543  0.0181  0.0307  0.0660
_END
	}
	elsif ($model eq 'Dayhoff') {
		print($datfile <<'_END');
0.267828
0.984474  0.327059
1.199805  0.000000  8.931515
0.360016  0.232374  0.000000  0.000000
0.887753  2.439939  1.028509  1.348551  0.000000
1.961167  0.000000  1.493409  11.388659  0.000000  7.086022
2.386111  0.087791  1.385352  1.240981  0.107278  0.281581  0.811907
0.228116  2.383148  5.290024  0.868241  0.282729  6.011613  0.439469  0.106802
0.653416  0.632629  0.768024  0.239248  0.438074  0.180393  0.609526  0.000000  0.076981
0.406431  0.154924  0.341113  0.000000  0.000000  0.730772  0.112880  0.071514  0.443504  2.556685
0.258635  4.610124  3.148371  0.716913  0.000000  1.519078  0.830078  0.267683  0.270475  0.460857  0.180629
0.717840  0.896321  0.000000  0.000000  0.000000  1.127499  0.304803  0.170372  0.000000  3.332732  5.230115  2.411739
0.183641  0.136906  0.138503  0.000000  0.000000  0.000000  0.000000  0.153478  0.475927  1.951951  1.565160  0.000000  0.921860
2.485920  1.028313  0.419244  0.133940  0.187550  1.526188  0.507003  0.347153  0.933709  0.119152  0.316258  0.335419  0.170205  0.110506
4.051870  1.531590  4.885892  0.956097  1.598356  0.561828  0.793999  2.322243  0.353643  0.247955  0.171432  0.954557  0.619951  0.459901  2.427202
3.680365  0.265745  2.271697  0.660930  0.162366  0.525651  0.340156  0.306662  0.226333  1.900739  0.331090  1.350599  1.031534  0.136655  0.782857  5.436674
0.000000  2.001375  0.224968  0.000000  0.000000  0.000000  0.000000  0.000000  0.270564  0.000000  0.461776  0.000000  0.000000  0.762354  0.000000  0.740819  0.000000
0.244139  0.078012  0.946940  0.000000  0.953164  0.000000  0.214717  0.000000  1.265400  0.374834  0.286572  0.132142  0.000000  6.952629  0.000000  0.336289  0.417839  0.608070
2.059564  0.240368  0.158067  0.178316  0.484678  0.346983  0.367250  0.538165  0.438715  8.810038  1.745156  0.103850  2.565955  0.123606  0.485026  0.303836  1.561997  0.000000  0.279379

0.087127  0.040904  0.040432  0.046872  0.033474  0.038255  0.049530  0.088612  0.033619  0.036886  0.085357  0.080481  0.014753  0.039772  0.050680  0.069577  0.058542  0.010494  0.029916  0.064718
_END
	}
	elsif ($model eq 'JTT') {
		print($datfile <<'_END');
0.531678
0.557967  0.451095
0.827445  0.154899  5.549530
0.574478  1.019843  0.313311  0.105625
0.556725  3.021995  0.768834  0.521646  0.091304
1.066681  0.318483  0.578115  7.766557  0.053907  3.417706
1.740159  1.359652  0.773313  1.272434  0.546389  0.231294  1.115632
0.219970  3.210671  4.025778  1.032342  0.724998  5.684080  0.243768  0.201696
0.361684  0.239195  0.491003  0.115968  0.150559  0.078270  0.111773  0.053769  0.181788
0.310007  0.372261  0.137289  0.061486  0.164593  0.709004  0.097485  0.069492  0.540571  2.335139
0.369437  6.529255  2.529517  0.282466  0.049009  2.966732  1.731684  0.269840  0.525096  0.202562  0.146481
0.469395  0.431045  0.330720  0.190001  0.409202  0.456901  0.175084  0.130379  0.329660  4.831666  3.856906  0.624581
0.138293  0.065314  0.073481  0.032522  0.678335  0.045683  0.043829  0.050212  0.453428  0.777090  2.500294  0.024521  0.436181
1.959599  0.710489  0.121804  0.127164  0.123653  1.608126  0.191994  0.208081  1.141961  0.098580  1.060504  0.216345  0.164215  0.148483
3.887095  1.001551  5.057964  0.589268  2.155331  0.548807  0.312449  1.874296  0.743458  0.405119  0.592511  0.474478  0.285564  0.943971  2.788406
4.582565  0.650282  2.351311  0.425159  0.469823  0.523825  0.331584  0.316862  0.477355  2.553806  0.272514  0.965641  2.114728  0.138904  1.176961  4.777647
0.084329  1.257961  0.027700  0.057466  1.104181  0.172206  0.114381  0.544180  0.128193  0.134510  0.530324  0.089134  0.201334  0.537922  0.069965  0.310927  0.080556
0.139492  0.235601  0.700693  0.453952  2.114852  0.254745  0.063452  0.052500  5.848400  0.303445  0.241094  0.087904  0.189870  5.484236  0.113850  0.628608  0.201094  0.747889
2.924161  0.171995  0.164525  0.315261  0.621323  0.179771  0.465271  0.470140  0.121827  9.533943  1.761439  0.124066  3.038533  0.593478  0.211561  0.408532  1.143980  0.239697  0.165473

0.076862  0.051057  0.042546  0.051269  0.020279  0.041061  0.061820  0.074714  0.022983  0.052569  0.091111  0.059498  0.023414  0.040530  0.050532  0.068225  0.058518  0.014336  0.032303  0.066374
_END
	}
	elsif ($model eq 'LG') {
		print($datfile <<'_END');
0.425093
0.276818  0.751878
0.395144  0.123954  5.076149
2.489084  0.534551  0.528768  0.062556
0.969894  2.807908  1.695752  0.523386  0.084808
1.038545  0.363970  0.541712  5.243870  0.003499  4.128591
2.066040  0.390192  1.437645  0.844926  0.569265  0.267959  0.348847
0.358858  2.426601  4.509238  0.927114  0.640543  4.813505  0.423881  0.311484
0.149830  0.126991  0.191503  0.010690  0.320627  0.072854  0.044265  0.008705  0.108882
0.395337  0.301848  0.068427  0.015076  0.594007  0.582457  0.069673  0.044261  0.366317  4.145067
0.536518  6.326067  2.145078  0.282959  0.013266  3.234294  1.807177  0.296636  0.697264  0.159069  0.137500
1.124035  0.484133  0.371004  0.025548  0.893680  1.672569  0.173735  0.139538  0.442472  4.273607  6.312358  0.656604
0.253701  0.052722  0.089525  0.017416  1.105251  0.035855  0.018811  0.089586  0.682139  1.112727  2.592692  0.023918  1.798853
1.177651  0.332533  0.161787  0.394456  0.075382  0.624294  0.419409  0.196961  0.508851  0.078281  0.249060  0.390322  0.099849  0.094464
4.727182  0.858151  4.008358  1.240275  2.784478  1.223828  0.611973  1.739990  0.990012  0.064105  0.182287  0.748683  0.346960  0.361819  1.338132
2.139501  0.578987  2.000679  0.425860  1.143480  1.080136  0.604545  0.129836  0.584262  1.033739  0.302936  1.136863  2.020366  0.165001  0.571468  6.472279
0.180717  0.593607  0.045376  0.029890  0.670128  0.236199  0.077852  0.268491  0.597054  0.111660  0.619632  0.049906  0.696175  2.457121  0.095131  0.248862  0.140825
0.218959  0.314440  0.612025  0.135107  1.165532  0.257336  0.120037  0.054679  5.306834  0.232523  0.299648  0.131932  0.481306  7.803902  0.089613  0.400547  0.245841  3.151815
2.547870  0.170887  0.083688  0.037967  1.959291  0.210332  0.245034  0.076701  0.119013  10.649107  1.702745  0.185202  1.898718  0.654683  0.296501  0.098369  2.188158  0.189510  0.249313

0.079066  0.055941  0.041977  0.053052  0.012937  0.040767  0.071586  0.057337  0.022355  0.062157  0.099081  0.064600  0.022951  0.042302  0.044040  0.061197  0.053287  0.012066  0.034155  0.069147
_END
	}
	elsif ($model eq 'mtArt') {
		print($datfile <<'_END');
  0.2
  0.2    0.2
  1      4    500
254     36     98     11
  0.2  154    262      0.2    0.2
  0.2    0.2  183    862      0.2  262
200      0.2  121     12     81      3     44
  0.2   41    180      0.2   12    314     15      0.2
 26      2     21      7     63     11      7      3      0.2
  4      2     13      1     79     16      2      1      6    515
  0.2  209    467      2      0.2  349    106      0.2    0.2    3      4
121      5     79      0.2  312     67      0.2   56      0.2  515    885    106
 13      5     20      0.2  184      0.2    0.2    1     14    118    263     11    322
 49      0.2   17      0.2    0.2   39      8      0.2    1      0.2   12     17      5     15
673      3    398     44    664     52     31    226     11      7      8    144    112     36     87
244      0.2  166      0.2  183     44     43      0.2   19    204     48     70    289     14     47    660
  0.2    0.2    8      0.2   22      7     11      2      0.2    0.2   21     16     71     54      0.2    2      0.2
  1      4    251      0.2   72     87      8      9    191     12     20    117     71    792     18     30     46     38
340      0.2   23      0.2  350      0.2   14      3      0.2 1855     85     26    281     52     32     61    544      0.2    2

0.054116  0.018227  0.039903  0.020160  0.009709  0.018781  0.024289  0.068183  0.024518  0.092638  0.148658  0.021718  0.061453  0.088668  0.041826  0.091030  0.049194  0.029786  0.039443  0.057700
_END
	}
	elsif ($model eq 'mtMam') {
		print($datfile <<'_END');
 32
  2    4
 11    0  864
  0  186    0    0
  0  246    8   49    0
  0    0    0  569    0  274
 78   18   47   79    0    0   22
  8  232  458   11  305  550   22    0
 75    0   19    0   41    0    0    0    0
 21    6    0    0   27   20    0    0   26  232
  0   50  408    0    0  242  215    0    0    6    4
 76    0   21    0    0   22    0    0    0  378  609   59
  0    0    6    5    7    0    0    0    0   57  246    0   11
 53    9   33    2    0   51    0    0   53    5   43   18    0   17
342    3  446   16  347   30   21  112   20    0   74   65   47   90  202
681    0  110    0  114    0    4    0    1  360   34   50  691    8   78  614
  5   16    6    0   65    0    0    0    0    0   12    0   13    0    7   17    0
  0    0  156    0  530   54    0    1 1525   16   25   67    0  682    8  107    0   14
398    0    0   10    0   33   20    5    0 2220  100    0  832    6    0    0  237    0    0

0.0692  0.0184  0.0400  0.0186  0.0065  0.0238  0.0236  0.0557  0.0277  0.0905  0.1675  0.0221  0.0561  0.0611  0.0536  0.0725  0.0870  0.0293  0.0340  0.0428
_END
	}
	elsif ($model eq 'mtREV') {
		print($datfile <<'_END');
  23.18
  26.95   13.24
  17.67    1.90  794.38
  59.93  103.33   58.94    1.90
   1.90  220.99  173.56   55.28   75.24
   9.77    1.90   63.05  583.55    1.90  313.56
 120.71   23.03   53.30   56.77   30.71    6.75   28.28
  13.90  165.23  496.13  113.99  141.49  582.40   49.12    1.90
  96.49    1.90   27.10    4.34   62.73    8.34    3.31    5.98   12.26
  25.46   15.58   15.16    1.90   25.65   39.70    1.90    2.41   11.49  329.09
   8.36  141.40  608.70    2.31    1.90  465.58  313.86   22.73  127.67   19.57   14.88
 141.88    1.90   65.41    1.90    6.18   47.37    1.90    1.90   11.97  517.98  537.53   91.37
   6.37    4.69   15.20    4.98   70.80   19.11    2.67    1.90   48.16   84.67  216.06    6.44   90.82
  54.31   23.64   73.31   13.43   31.26  137.29   12.83    1.90   60.97   20.63   40.10   50.10   18.84   17.31
 387.86    6.04  494.39   69.02  277.05   54.11   54.71  125.93   77.46   47.70   73.61  105.79  111.16   64.29  169.90
 480.72    2.08  238.46   28.01  179.97   94.93   14.82   11.17   44.78  368.43  126.40  136.33  528.17   33.85  128.22  597.21
   1.90   21.95   10.68   19.86   33.60    1.90    1.90   10.92    7.08    1.90   32.44   24.00   21.71    7.84    4.21   38.58    9.99
   6.48    1.90  191.36   21.21  254.77   38.82   13.12    3.21  670.14   25.01   44.15   51.17   39.96  465.58   16.21   64.92   38.73   26.25
 195.06    7.64    1.90    1.90    1.90   19.00   21.14    2.53    1.90 1222.94   91.67    1.90  387.54    6.35    8.23    1.90  204.54    5.37    1.90

0.072  0.019  0.039  0.019  0.006  0.025  0.024  0.056  0.028  0.088  0.169  0.023  0.054  0.061  0.054  0.072  0.086  0.029  0.033  0.043
_END
	}
	elsif ($model eq 'mtZoa') {
		print($datfile <<'_END');
   3.3
   1.7  33.6
  16.1   3.2 617.0
 272.5  61.1  94.6   9.5
   7.3 231.0 190.3  19.3  49.1
  17.1   6.4 174.0 883.6   3.4 349.4
 289.3   7.2  99.3  26.0  82.4   8.9  43.1
   2.3  61.7 228.9  55.6  37.5 421.8  14.9   7.4
  33.2   0.2  24.3   1.5  48.8   0.2   7.3   3.4   1.6
  15.6   4.1   7.9   0.5  59.7  23.0   1.0   3.5   6.6 425.2
   0.2 292.3 413.4   0.2   0.2 334.0 163.2  10.1  23.9   8.4   6.7
 136.5   3.8  73.7   0.2 264.8  83.9   0.2  52.2   7.1 449.7 636.3  83.0
  26.5   0.2  12.9   2.0 167.8   9.5   0.2   5.8  13.1  90.3 234.2  16.3 215.6
  61.8   7.5  22.6   0.2   8.1  52.2  20.6   1.3  15.6   2.6  11.4  24.3   5.4  10.5
 644.9  11.8 420.2  51.4 656.3  96.4  38.4 257.1  23.1   7.2  15.2 144.9  95.3  32.2  79.7
 378.1   3.2 184.6   2.3 199.0  39.4  34.5   5.2  19.4 222.3  50.0  75.5 305.1  19.3  56.9 666.3
   3.1  16.9   6.4   0.2  36.1   6.1   3.5  12.3   4.5   9.7  27.2   6.6  48.7  58.2   1.3  10.3   3.6
   2.1  13.8 141.6  13.9  76.7  52.3  10.0   4.3 266.5  13.1   5.7  45.0  41.4 590.5   4.2  29.7  29.0  79.8
 321.9   5.1   7.1   3.7 243.8   9.0  16.3  23.7   0.3 1710.6 126.1  11.1 279.6  59.6  17.9  49.5 396.4  13.7  15.6

0.068880  0.021037  0.030390  0.020696  0.009966  0.018623  0.024989  0.071968  0.026814  0.085072  0.156717  0.019276  0.050652  0.081712  0.044803  0.080535  0.056386  0.027998  0.037404  0.066083
_END
	}
	elsif ($model eq 'PMB') {
		print($datfile <<'_END');
0.674995699
0.589645178  1.189067034
0.462499504  0.605460903  3.573373315
1.065445546  0.31444833   0.589852457  0.246951424
1.111766964  2.967840934  2.299755865  1.686058219  0.245163782
1.046334652  1.201770702  1.277836748  4.399995525  0.091071867  4.15967899
1.587964372  0.523770553  1.374854049  0.734992057  0.31706632   0.596789898  0.463812837
0.580830874  1.457127446  2.283037894  0.839348444  0.411543728  1.812173605  0.877842609  0.476331437
0.464590585  0.35964586   0.426069419  0.266775558  0.417547309  0.315256838  0.30421529   0.180198883  0.285186418
0.804404505  0.520701585  0.41009447   0.269124919  0.450795211  0.625792937  0.32078471   0.259854426  0.363981358  4.162454693
0.831998835  4.956476453  2.037575629  1.114178954  0.274163536  3.521346591  2.415974716  0.581001076  0.985885486  0.374784947  0.498011337
1.546725076  0.81346254   0.737846301  0.341932741  0.618614612  2.067388546  0.531773639  0.465349326  0.380925433  3.65807012   5.002338375  0.661095832
0.546169219  0.303437244  0.425193716  0.219005213  0.669206193  0.406042546  0.224154698  0.35402891   0.576231691  1.495264661  2.392638293  0.269496317  2.306919847
1.241586045  0.65577338   0.711495595  0.775624818  0.198679914  0.850116543  0.794584081  0.588254139  0.456058589  0.366232942  0.430073179  1.036079005  0.337502282  0.481144863
3.452308792  0.910144334  2.572577221  1.440896785  0.99870098   1.348272505  1.205509425  1.402122097  0.799966711  0.530641901  0.402471997  1.234648153  0.945453716  0.613230817  1.217683028
1.751412803  0.89517149   1.823161023  0.994227284  0.847312432  1.320626678  0.949599791  0.542185658  0.83039281   1.114132523  0.779827336  1.290709079  1.551488041  0.718895136  0.780913179  4.448982584
0.35011051   0.618778365  0.422407388  0.362495245  0.445669347  0.72038474   0.261258229  0.37874827   0.72436751   0.516260502  0.794797115  0.43340962   0.768395107  3.29519344   0.499869138  0.496334956  0.38372361
0.573154753  0.628599063  0.720013799  0.436220437  0.55626163   0.728970584  0.50720003   0.284727562  2.210952064  0.570562395  0.811019594  0.664884513  0.93253606   5.894735673  0.433748126  0.593795813  0.523549536  2.996248013
2.063050067  0.388680158  0.474418852  0.275658381  0.998911631  0.634408285  0.527640634  0.314700907  0.305792277  8.002789424  2.113077156  0.526184203  1.737356217  0.983844803  0.551333603  0.507506011  1.89965079   0.429570747  0.716795463

0.075559  0.053764  0.037684  0.044693  0.028483  0.033893  0.053472  0.078036  0.03004  0.059869  0.095793  0.051997  0.021908  0.045001  0.042029  0.068212  0.056413  0.015725  0.035974  0.071456
_END
	}
	elsif ($model eq 'rtREV') {
		print($datfile <<'_END');
   34
   51   35
   10   30  384
  439   92  128    1
   32  221  236   78   70
   81   10   79  542    1  372
  135   41   94   61   48   18   70
   30   90  320   91  124  387   34   68
    1   24   35    1  104   33    1    1   34
   45   18   15    5  110   54   21    3   51  385
   38  593  123   20   16  309  141   30   76   34   23
  235   57    1    1  156  158    1   37  116  375  581  134
    1    7   49    1   70    1    1    7  141   64  179   14  247
   97   24   33   55    1   68   52   17   44   10   22   43    1   11
  460  102  294  136   75  225   95  152  183    4   24   77    1   20  134
  258   64  148   55  117  146   82    7   49   72   25  110  131   69   62  671
    5   13   16    1   55   10   17   23   48   39   47    6  111  182    9   14    1
   55   47   28    1  131   45    1   21  307   26   64    1   74 1017   14   31   34  176
  197   29   21    6  295   36   35    3    1 1048  112   19  236   92   25   39  196   26   59

0.0646  0.0453  0.0376  0.0422  0.0114  0.0606  0.0607  0.0639  0.0273  0.0679  0.1018  0.0751  0.0150  0.0287  0.0681  0.0488  0.0622  0.0251  0.0318  0.0619
_END
	}
	elsif ($model eq 'HIVb') {
		print($datfile <<'_END');
 0.30750700
 0.00500000   0.29554300
 1.45504000   0.00500000  17.66120000
 0.12375800   0.35172100   0.08606420   0.00500000
 0.05511280   3.42150000   0.67205200   0.00500000   0.00500000
 1.48135000   0.07492180   0.07926330  10.58720000   0.00500000   2.56020000
 2.13536000   3.65345000   0.32340100   2.83806000   0.89787100   0.06191370   3.92775000
 0.08476130   9.04044000   7.64585000   1.91690000   0.24007300   7.05545000   0.11974000   0.00500000
 0.00500000   0.67728900   0.68056500   0.01767920   0.00500000   0.00500000   0.00609079   0.00500000   0.10311100
 0.21525600   0.70142700   0.00500000   0.00876048   0.12977700   1.49456000   0.00500000   0.00500000   1.74171000   5.95879000
 0.00500000  20.45000000   7.90443000   0.00500000   0.00500000   6.54737000   4.61482000   0.52170500   0.00500000   0.32231900   0.08149950
 0.01866430   2.51394000   0.00500000   0.00500000   0.00500000   0.30367600   0.17578900   0.00500000   0.00500000  11.20650000   5.31961000   1.28246000
 0.01412690   0.00500000   0.00500000   0.00500000   9.29815000   0.00500000   0.00500000   0.29156100   0.14555800   3.39836000   8.52484000   0.03426580   0.18802500
 2.12217000   1.28355000   0.00739578   0.03426580   0.00500000   4.47211000   0.01202260   0.00500000   2.45318000   0.04105930   2.07757000   0.03138620   0.00500000   0.00500000
 2.46633000   3.47910000  13.14470000   0.52823000   4.69314000   0.11631100   0.00500000   4.38041000   0.38274700   1.21803000   0.92765600   0.50411100   0.00500000   0.95647200   5.37762000
15.91830000   2.86868000   6.88667000   0.27472400   0.73996900   0.24358900   0.28977400   0.36961500   0.71159400   8.61217000   0.04376730   4.67142000   4.94026000   0.01412690   2.01417000   8.93107000
 0.00500000   0.99133800   0.00500000   0.00500000   2.63277000   0.02665600   0.00500000   1.21674000   0.06951790   0.00500000   0.74884300   0.00500000   0.08907800   0.82934300   0.04445060   0.02487280   0.00500000
 0.00500000   0.00991826   1.76417000   0.67465300   7.57932000   0.11303300   0.07926330   0.00500000  18.69430000   0.14816800   0.11198600   0.00500000   0.00500000  15.34000000   0.03043810   0.64802400   0.10565200   1.28022000
 7.61428000   0.08124540   0.02665600   1.04793000   0.42002700   0.02091530   1.02847000   0.95315500   0.00500000  17.73890000   1.41036000   0.26582900   6.85320000   0.72327400   0.00500000   0.07492180   0.70922600   0.00500000   0.04105930

0.060490222  0.066039665  0.044127815  0.042109048  0.020075899  0.053606488  0.071567447  0.072308239  0.022293943  0.069730629  0.098851122  0.056968211  0.019768318  0.028809447  0.046025282  0.050604330  0.053636813  0.033011601  0.028350243  0.061625237
_END
	}
	elsif ($model eq 'HIVw') {
		print($datfile <<'_END');
 0.0744808
 0.6175090   0.1602400
 4.4352100   0.0674539  29.4087000
 0.1676530   2.8636400   0.0604932   0.0050000
 0.0050000  10.6746000   0.3420680   0.0050000   0.0050000
 5.5632500   0.0251632   0.2015260  12.1233000   0.0050000   3.2065600
 1.8685000  13.4379000   0.0604932  10.3969000   0.0489798   0.0604932  14.7801000
 0.0050000   6.8440500   8.5987600   2.3177900   0.0050000  18.5465000   0.0050000   0.0050000
 0.0050000   1.3406900   0.9870280   0.1451240   0.0050000   0.0342252   0.0390512   0.0050000   0.0050000
 0.1602400   0.5867570   0.0050000   0.0050000   0.0050000   2.8904800   0.1298390   0.0489798   1.7638200   9.1024600
 0.5927840  39.8897000  10.6655000   0.8943130   0.0050000  13.0705000  23.9626000   0.2794250   0.2240600   0.8174810   0.0050000
 0.0050000   3.2865200   0.2015260   0.0050000   0.0050000   0.0050000   0.0050000   0.0489798   0.0050000  17.3064000  11.3839000   4.0956400
 0.5979230   0.0050000   0.0050000   0.0050000   0.3629590   0.0050000   0.0050000   0.0050000   0.0050000   1.4828800   7.4878100   0.0050000   0.0050000
 1.0098100   0.4047230   0.3448480   0.0050000   0.0050000   3.0450200   0.0050000   0.0050000  13.9444000   0.0050000   9.8309500   0.1119280   0.0050000   0.0342252
 8.5942000   8.3502400  14.5699000   0.4278810   1.1219500   0.1602400   0.0050000   6.2796600   0.7251570   0.7400910   6.1439600   0.0050000   0.3925750   4.2793900  14.2490000
24.1422000   0.9282030   4.5420600   0.6303950   0.0050000   0.2030910   0.4587430   0.0489798   0.9595600   9.3634500   0.0050000   4.0480200   7.4131300   0.1145120   4.3370100   6.3407900
 0.0050000   5.9656400   0.0050000   0.0050000   5.4989400   0.0443298   0.0050000   2.8258000   0.0050000   0.0050000   1.3703100   0.0050000   0.0050000   0.0050000   0.0050000   1.1015600   0.0050000
 0.0050000   0.0050000   5.0647500   2.2815400   8.3483500   0.0050000   0.0050000   0.0050000  47.4889000   0.1145120   0.0050000   0.0050000   0.5791980   4.1272800   0.0050000   0.9331420   0.4906080   0.0050000
24.8094000   0.2794250   0.0744808   2.9178600   0.0050000   0.0050000   2.1995200   2.7962200   0.8274790  24.8231000   2.9534400   0.1280650  14.7683000   2.2800000   0.0050000   0.8626370   0.0050000   0.0050000   1.3548200

0.0377494  0.0573210  0.0891129  0.0342034  0.0240105  0.0437824  0.0618606  0.0838496  0.0156076  0.0983641  0.0577867  0.0641682  0.0158419  0.0422741  0.0458601  0.0550846  0.0813774  0.0195970  0.0205847  0.0515639
_END
	}
	elsif ($model eq 'VT') {
		print($datfile <<'_END');
1.2412691067876198
1.2184237953498958   1.5720770753326880
1.3759368509441177   0.7550654439001206   7.8584219153689405
2.4731223087544874   1.4414262567428417   0.9784679122774127   0.2272488448121475
2.2155167805137470   5.5120819705248678   3.0143201670924822   1.6562495638176040   0.4587469126746136
2.3379911207495061   1.3542404860613146   2.0093434778398112   9.6883451875685065   0.4519167943192672   6.8124601839937675
3.3386555146457697   1.3121700301622004   2.4117632898861809   1.9142079025990228   1.1034605684472507   0.8776110594765502   1.3860121390169038
0.9615841926910841   4.9238668283945266   6.1974384977884114   2.1459640610133781   1.5196756759380692   7.9943228564946525   1.6360079688522375   0.8561248973045037
0.8908203061925510   0.4323005487925516   0.9179291175331520   0.2161660372725585   0.9126668032539315   0.4882733432879921   0.4035497929633328   0.2888075033037488   0.5787937115407940
1.0778497408764076   0.8386701149158265   0.4098311270816011   0.3574207468998517   1.4081315998413697   1.3318097154194044   0.5610717242294755   0.3578662395745526   1.0765007949562073   6.0019110258426362
1.4932055816372476  10.017330817366002    4.4034547578962568   1.4521790561663968   0.3371091785647479   6.0519085243118811   4.3290086529582830   0.8945563662345198   1.8085136096039203   0.6244297525127139   0.5642322882556321
1.9006455961717605   1.2488638689609959   0.9378803706165143   0.4075239926000898   1.2213054800811556   1.9106190827629084   0.7471936218068498   0.5954812791740037   1.3808291710019667   6.7597899772045418   8.0327792947421148   1.7129670976916258
0.6883439026872615   0.4224945197276290   0.5044944273324311   0.1675129724559251   1.6953951980808002   0.3573432522499545   0.2317194387691585   0.3693722640980460   1.3629765501081097   2.2864286949316077   4.3611548063555778   0.3910559903834828   2.3201373546296349
2.7355620089953550   1.3091837782420783   0.7103720531974738   1.0714605979577547   0.4326227078645523   2.3019177728300728   1.5132807416252063   0.7744933618134962   1.8370555852070649   0.4811402387911145   1.0084320519837335   1.3918935593582853   0.4953193808676289   0.3746821107962129
6.4208961859142883   1.9202994262316166   6.1234512396801764   2.2161944596741829   3.6366815408744255   2.3193703643237220   1.8273535587773553   3.0637776193717610   1.9699895187387506   0.6047491507504744   0.8953754669269811   1.9776630140912268   1.0657482318076852   1.1079144700606407   3.5465914843628927
5.2892514169776437   1.3363401740560601   3.8852506105922231   1.5066839872944762   1.7557065205837685   2.1576510103471440   1.5839981708584689   0.7147489676267383   1.6136654573285647   2.6344778384442731   1.0192004372506540   2.5513781312660280   3.3628488360462363   0.6882725908872254   1.9485376673137556   8.8479984061248178
0.5488578478106930   1.5170142153962840   0.1808525752605976   0.2496584188151770   1.6275179891253113   0.8959082681546182   0.4198391148111098   0.9349753595598769   0.6301954684360302   0.5604648274060783   1.5183114434679339   0.5851920879490173   1.4680478689711018   3.3448437239772266   0.4326058001438786   0.6791126595939816   0.4514203099376473
0.5411769916657778   0.8912614404565405   1.0894926581511342   0.7447620891784513   2.1579775140421025   0.9183596801412757   0.5818111331782764   0.3374467649724478   7.7587442309146040   0.8626796044156272   1.2452243224541324   0.7835447533710449   1.0899165770956820  10.384852333133459    0.4819109019647465   0.9547229305958682   0.8564314184691215   4.5377235790405388
4.6501894691803214   0.7807017855806767   0.4586061981719967   0.4594535241660911   2.2627456996290891   0.6366932501396869   0.8940572875547330   0.6193321034173915   0.5333220944030346  14.872933461519061    3.5458093276667237   0.7801080335991272   4.0584577156753401   1.7039730522675411   0.5985498912985666   0.9305232113028208   3.4242218450865543   0.5658969249032649   1.0000000000000000

0.0770764620135024  0.0500819370772208  0.0462377395993731  0.0537929860758246  0.0144533387583345  0.0408923608974345  0.0633579339160905  0.0655672355884439  0.0218802687005936  0.0591969699027449  0.0976461276528445  0.0592079410822730  0.0220695876653368  0.0413508521834260  0.0476871596856874  0.0707295165111524  0.0567759161524817  0.0127019797647213  0.0323746050281867  0.0669190817443274
_END
	}
	elsif ($model eq 'WAG') {
		print($datfile <<'_END');
0.551571 
0.509848  0.635346 
0.738998  0.147304  5.429420 
1.027040  0.528191  0.265256  0.0302949 
0.908598  3.035500  1.543640  0.616783  0.0988179 
1.582850  0.439157  0.947198  6.174160  0.021352  5.469470 
1.416720  0.584665  1.125560  0.865584  0.306674  0.330052  0.567717 
0.316954  2.137150  3.956290  0.930676  0.248972  4.294110  0.570025  0.249410 
0.193335  0.186979  0.554236  0.039437  0.170135  0.113917  0.127395  0.0304501 0.138190 
0.397915  0.497671  0.131528  0.0848047 0.384287  0.869489  0.154263  0.0613037 0.499462  3.170970 
0.906265  5.351420  3.012010  0.479855  0.0740339 3.894900  2.584430  0.373558  0.890432  0.323832  0.257555 
0.893496  0.683162  0.198221  0.103754  0.390482  1.545260  0.315124  0.174100  0.404141  4.257460  4.854020  0.934276 
0.210494  0.102711  0.0961621 0.0467304 0.398020  0.0999208 0.0811339 0.049931  0.679371  1.059470  2.115170  0.088836  1.190630 
1.438550  0.679489  0.195081  0.423984  0.109404  0.933372  0.682355  0.243570  0.696198  0.0999288 0.415844  0.556896  0.171329  0.161444 
3.370790  1.224190  3.974230  1.071760  1.407660  1.028870  0.704939  1.341820  0.740169  0.319440  0.344739  0.967130  0.493905  0.545931  1.613280 
2.121110  0.554413  2.030060  0.374866  0.512984  0.857928  0.822765  0.225833  0.473307  1.458160  0.326622  1.386980  1.516120  0.171903  0.795384  4.378020 
0.113133  1.163920  0.0719167 0.129767  0.717070  0.215737  0.156557  0.336983  0.262569  0.212483  0.665309  0.137505  0.515706  1.529640  0.139405  0.523742  0.110864 
0.240735  0.381533  1.086000  0.325711  0.543833  0.227710  0.196303  0.103604  3.873440  0.420170  0.398618  0.133264  0.428437  6.454280  0.216046  0.786993  0.291148  2.485390 
2.006010  0.251849  0.196246  0.152335  1.002140  0.301281  0.588731  0.187247  0.118358  7.821300  1.800340  0.305434  2.058450  0.649892  0.314887  0.232739  1.388230  0.365369  0.314730 

0.0866279  0.043972  0.0390894  0.0570451  0.0193078  0.0367281  0.0580589  0.0832518  0.0244313  0.048466  0.086209  0.0620286  0.0195027  0.0384319  0.0457631  0.0695179  0.0610127  0.0143859  0.0352742  0.0708956
_END
	}
	elsif ($model eq 'FLU') {
		print($datfile <<'_END');
0.138658764751059
0.0533665787145181	0.161000889039552
0.584852305649886	0.00677184253227681	7.73739287051356
0.0264470951166826	0.16720700818221	0.0000130249856764315	0.014132062548787
0.353753981649393	3.29271694159791	0.530642655337477	0.145469388422239	0.00254733397966779
1.4842345032161	0.124897616909194	0.0616521921873234	5.37051127867923	0.0000000000391106992668137	1.19562912226203
1.13231312248046	1.19062446519178	0.322524647863997	1.93483278448943	0.116941459124876	0.108051341246072	1.59309882471598
0.214757862168721	1.87956993845887	1.38709603234116	0.887570549414031	0.0218446166959521	5.33031341222104	0.256491863423002	0.0587745274250666
0.149926734229061	0.246117171830255	0.21857197541607	0.0140859174993809	0.00111215807314139	0.0288399502994541	0.0142107118685268	0.0000162662283098296	0.243190142026506
0.0231169515264061	0.296045557460629	0.000835873174542931	0.00573068208525287	0.00561362724916376	1.02036695531654	0.016499535540562	0.00651622937676521	0.321611693603646	3.51207228207807
0.474333610192982	15.3000966197798	2.6468479652886	0.290042980143818	0.00000383228119049152	2.559587177122	3.88148880863814	0.264148929349066	0.347302791211758	0.227707997165566	0.129223639195248
0.0587454231508643	0.890162345593224	0.00525168778853117	0.0417629637305017	0.111457310321926	0.190259181297527	0.313974351356074	0.00150046692269255	0.00127350890508147	9.01795420287895	6.74693648486614	1.33129161941264
0.0804909094320368	0.0160550314767596	0.000836445615590923	0.0000010600102849456	0.10405366623526	0.0326806570137471	0.00100350082518749	0.00123664495412902	0.119028506158521	1.46335727834648	2.98680003596399	0.319895904499071	0.279910508981581
0.659311477863896	0.154027179890711	0.0364417719063219	0.188539456415654	0.000000000000159312060172652	0.712769599068934	0.319558828428154	0.0386317614553493	0.924466914225534	0.0805433268150369	0.634308520867322	0.195750631825315	0.0568693216513547	0.0071324304661639
3.01134451903854	0.950138410087378	3.88131053061457	0.338372183381345	0.336263344504404	0.487822498528951	0.307140298031341	1.58564657669139	0.580704249811294	0.290381075260226	0.570766693213698	0.283807671568883	0.00702658828739369	0.996685669575839	2.08738534433198
5.4182981753166	0.183076905018197	2.14033231636063	0.135481232622983	0.011975265782196	0.60234096342392	0.2801248951174	0.0188080299490973	0.368713573381758	2.90405228596936	0.0449263566753846	1.52696419998775	2.03151132062208	0.000134906239484254	0.54225109402693	2.2068599339404
0.195966354027106	1.36942940801512	0.000536284040016542	0.000014893873721753	0.0941066800969967	0.0440205200833047	0.155245492137294	0.196486447133033	0.0223729191088972	0.0321321499585514	0.431277662888057	0.0000497641445484395	0.0704600385245663	0.814753093809928	0.000431020702277328	0.0998357527014247	0.207066205546908
0.0182892882245349	0.0998554972524385	0.373101926513925	0.525398542949365	0.601692431136271	0.0722059354079545	0.104092870343653	0.0748149970972622	6.44895444648517	0.273934263183281	0.340058468374384	0.0124162215506117	0.874272174533394	5.39392424532822	0.000182294881489116	0.392552239890831	0.124898020409882	0.42775543040588
3.53200526987468	0.103964386383736	0.0102575172450253	0.297123975243582	0.0549045639492389	0.406697814049488	0.285047948309311	0.337229618868315	0.0986313546653266	14.3940521944257	0.890598579382591	0.0731279296372675	4.90484223478739	0.592587985458668	0.0589719751511691	0.0882564232979724	0.654109108255219	0.256900461407996	0.167581646770807

0.0470718	0.0509102	0.0742143	0.0478596	0.0250216	0.0333036	0.0545874	0.0763734	0.0199642	0.0671336	0.0714981	0.0567845	0.0181507	0.0304961	0.0506561	0.0884091	0.0743386	0.0185237	0.0314741	0.0632292
_END
	}
	elsif ($model eq 'stmtREV') {
		print($datfile <<'_END');
0.1159435373
0.2458816714	0.1355713516
0.9578712472	0.0775041665	8.4408676914
0.2327281954	9.1379470330	0.1137687264	0.0582110367
0.3309250853	5.2854173238	0.1727184754	0.8191776581	0.0009722083
0.6946680829	0.0966719296	0.2990806606	7.3729791633	0.0005604799	3.5773486727
2.8076062202	3.0815651393	0.5575702616	2.2627839242	1.1721237455	0.0482085663	3.3184632572
0.2275494971	2.8251848421	9.5228608030	2.3191131858	0.0483235836	4.4138715270	0.0343694246	0.0948383460
0.0627691644	0.5712158076	0.2238609194	0.0205779319	0.1527276944	0.0206129952	0.0328079744	0.1239000315	0.0802374651
0.0305818840	0.1930408758	0.0540967250	0.0018843293	0.2406073246	0.3299454620	0.0373753435	0.0005918940	0.1192904610	1.3184058362
0.2231434272	6.0541970908	4.3977466558	0.1347413792	0.0001480536	5.2864094506	6.8883522181	0.5345755286	0.3991624551	0.2107928508	0.1055933141
0.1874527991	0.2427875732	0.0433577842	0.0000022173	0.0927357503	0.0109238300	0.0663619185	0.0128777966	0.0722334577	4.3016010974	1.1493262595	0.4773694701
0.0458112245	0.0310030750	0.0233493970	0.0000080023	0.8419347601	0.0027817812	0.0361207581	0.0490593583	0.0197089530	0.3634155844	2.1032860162	0.0861057517	0.1735660361
1.5133910481	0.7858555362	0.3000131148	0.3337627573	0.0036260499	1.5386413234	0.5196922389	0.0221252552	1.0171151697	0.0534088166	6.0377879080	0.4350064365	0.1634497017	0.3545179411
2.3008246523	0.7625702322	1.9431704326	0.6961369276	2.3726544756	0.1837198343	0.9087013201	2.5477016916	0.3081949928	0.1713464632	2.7297706102	0.3416923226	0.0730798705	4.0107845583	8.4630191575
4.3546170435	1.0655012755	1.6534489471	0.0985354973	0.1940108923	0.3415280861	0.2794040892	0.1657005971	0.2704552047	2.3418182855	0.0426297282	1.2152488582	4.6553742047	0.0068797851	1.1613183519	2.2213527952
0.0565037747	6.7852754661	0.0000010442	0.0000002842	0.9529353202	0.0009844045	0.0002705734	0.5068170211	0.0000932799	0.0050518699	0.3163744815	0.0000023280	0.1010587493	0.2890102379	0.0041564377	0.0495269526	0.0002026765
0.0358664532	0.0714121777	0.3036789915	1.3220740967	1.7972997876	0.0066458178	0.3052655031	0.0174305437	21.9842817264	0.1070890246	0.0770894218	0.1929529483	0.0561599188	1.6748429971	0.0021338646	1.8890678523	0.2834320440	0.3134203648
3.2116908598	0.0108028571	0.0860833645	0.0426724431	0.3652373073	0.0287789552	0.1484349765	0.5158740953	0.0059791370	3.3648305163	0.8763855707	0.0776875418	0.9145670668	0.3963331926	0.1080226203	0.0640951379	0.2278998021	0.0388755869	0.1836950254

0.0461811000	0.0534080000	0.0361971000	0.0233326000	0.0234170000	0.0390397000	0.0341284001	0.0389164000	0.0164640000	0.0891534000	0.1617310001	0.0551341000	0.0233262000	0.0911252000	0.0344713001	0.0771077000	0.0418603001	0.0200784000	0.0305429000	0.0643851996
_END
	}
	else {
		&errorMessage(__LINE__, "\"$model\" is invalid model.");
	}
	close($datfile);
}

sub errorMessage {
	my $lineno = shift(@_);
	my $message = shift(@_);
	print("ERROR!: line $lineno\n$message\n");
	if ($interactive) {
		print("Please press enter to abort.\n");
		<STDIN>;
	}
	else {
		print("If you want to read help message, run this script with \"--help\" option.\n");
	}
	exit(1);
}

sub helpMessage {
	print <<"_END";
Usage
=====

aminosan [-hiwfpgton] inputfile

Command line options
====================

-h, --help
  Display this message and exit.

-i, --interactive=Enable|Disable
  Specify whether you want to use interactive mode or not. (default:
Disable)

-w, --calcwhole=Enable|Disable
  Specify whether you want to consider nonpartitioned models on whole
sequences or not. (default: Disable)

-f, --aafreq=Empirical|Estimate
  Specify amino-acid frequencies optimization approach. (default:
Empirical)

-p, --partition=PARTITIONS
  Specify data partitions. (default: none)

-g, --ngammacat=INTEGER
  Specify the number of gamma rate categories. (default: 8)

-v, --pinvar=Enable|Disable
  Specify whether you want to consider invariant model of rate 
heterogeneity or not. (default: Disable)

-m, --ngam=Enable|Disable
  Specify whether you want to consider different Gamma shape for
each partition or not. (default: Disable)

-t, --tree=Common|Different
  If you specify 'Common', the parameters are optimized at a common
tree on each partition. If you specify 'Different', the parameters are
optimized at different tree on each partition. (default: Common)

-o, --output=MrBayes|PHYML|RAxML|Treefinder
  Specify output options. (default: none)

-n, --numthreads=INTEGER
  Specify the number of threads. (default: 1)

-l, --likelihood=CODEML|TF|RAxML
  Specify likelihood calculation software. (default: automatic)

--comparemix=Enable|Disable
  Specify whether you want to compare nonpartitioned, proportional and
separate models on all-loci concatenated sequences. (default: Enable,
if output option of MrBayes, RAxML or Treefinder is enabled.)

--treefile=TREEFILE
  Specify tree file which contain starting tree. (default: none)
Newick or NEXUS tree file was accepted.

Examples of data partition specification
=========================================

part1:1-200,part2:201-500,part3:501-550
_END
	exit;
}
